/* -*- Mode: C; indent-tabs-mode: s; c-basic-offset: 2; tab-width: 2 -*- */
/*
 * countries.c
 * Copyright (C) Mike Crash 2009 <mike@mikecrash.com>
 *
 * main.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * main.c is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <string.h>
#include <ctype.h>
#include "countries.h"

typedef struct
{
  int country;
  const char *code;
  const char *name;
  const char *osmname;
} iso3166s;

iso3166s iso3166 [] =
{
  {  4, "AF", "Afghanistan", NULL},
  {248, "AX", "Aland Islands", NULL},
  {  8, "AL", "Albania", NULL},
  { 12, "DZ", "Algeria", NULL},
  { 16, "AS", "American Samoa", NULL},
  {850, "AD", "Andorra", NULL},
  { 20, "AO", "Angola", NULL},
  { 24, "AI", "Anguilla", NULL},
  {660, "AQ", "Antarctica", NULL},
  { 10, "AG", "Antigua and Barbuda", NULL},
  { 28, "AR", "Argentina", NULL},
  { 32, "AM", "Armenia", NULL},
  { 51, "AW", "Aruba", NULL},
  {533, "AU", "Australia", NULL},
  { 36, "AT", "Austria", NULL},
  { 31, "AZ", "Azerbaijan", NULL},
  { 44, "BS", "Bahamas", NULL},
  { 48, "BH", "Bahrain", NULL},
  { 50, "BD", "Bangladesh", NULL},
  { 52, "BB", "Barbados", NULL},
  { 56, "BY", "Belarus", NULL},
  { 84, "BE", "Belgium", NULL},
  {112, "BZ", "Belize", NULL},
  {204, "BJ", "Benin", NULL},
  { 60, "BM", "Bermuda", NULL},
  { 64, "BT", "Bhutan", NULL},
  { 68, "BO", "Bolivia", NULL},
  { 70, "BA", "Bosnia and Herzegovina", NULL},
  { 72, "BW", "Botswana", NULL},
  { 74, "BV", "Bouvet Island", NULL},
  { 76, "BR", "Brazil", NULL},
  { 86, "IO", "British Indian Ocean Territory", NULL},
  { 92, "BN", "Brunei Darussalam", NULL},
  { 96, "BG", "Bulgaria", NULL},
  {100, "BF", "Burkina Faso", NULL},
  {854, "BI", "Burundi", NULL},
  {108, "KH", "Cambodia", NULL},
  {184, "CM", "Cameroon", NULL},
  {148, "CA", "Canada", NULL},
  {499, "CV", "Cape Verde", NULL},
  {203, "KY", "Cayman Islands", NULL},
  {156, "CF", "Central African Republic", NULL},
  {208, "TD", "Chad", NULL},
  {180, "CL", "Chile", NULL},
  {212, "CN", "China", NULL},
  {214, "CX", "Christmas Island", NULL},
  {262, "CC", "Cocos (Keeling) Islands", NULL},
  {818, "CO", "Colombia", NULL},
  {218, "KM", "Comoros", NULL},
  {232, "CG", "Congo", NULL},
  {233, "CD", "Congo, the Democratic Republic of the", NULL},
  {231, "CK", "Cook Islands", NULL},
  {234, "CR", "Costa Rica", NULL},
  {238, "CI", "Cote d'Ivoire ? Côte d'Ivoire", NULL},
  {242, "HR", "Croatia", NULL},
  {608, "CU", "Cuba", NULL},
  {246, "CY", "Republic of Cyprus", NULL},
  {246, "CY", "Northern Cyprus", NULL},
  {250, "CZ", "Czech Republic", NULL},
  {254, "DK", "Denmark", NULL},
  {260, "DJ", "Djibouti", NULL},
  {258, "DM", "Dominica", NULL},
  {266, "DO", "Dominican Republic", NULL},
  {270, "EC", "Ecuador", NULL},
  {288, "EG", "Egypt", NULL},
  {292, "SV", "El Salvador", NULL},
  {308, "GQ", "Equatorial Guinea", NULL},
  {304, "ER", "Eritrea", NULL},
  {268, "EE", "Estonia", NULL},
  {312, "ET", "Ethiopia", NULL},
  {316, "FK", "Falkland Islands", NULL},
  {320, "FO", "Faroe Islands", "Territorial waters of Faroe Islands"},
  {324, "FJ", "Fiji", NULL},
  {624, "FI", "Finland", NULL},
  {831, "FR", "France", NULL},
  {328, "GF", "French Guiana", NULL},
  {332, "PF", "French Polynesia", NULL},
  {334, "TF", "French Southern Territories", NULL},
  {340, "GA", "Gabon", NULL},
  {344, "GM", "Gambia", NULL},
  {152, "GE", "Georgia", NULL},
  {191, "DE", "Germany", NULL},
  {356, "GH", "Ghana", NULL},
  {360, "GI", "Gibraltar", NULL},
  {368, "GR", "Greece", NULL},
  {364, "GL", "Greenland", NULL},
  {372, "GD", "Grenada", NULL},
  {352, "GP", "Guadeloupe", NULL},
  {380, "GU", "Guam", NULL},
  {376, "GT", "Guatemala", NULL},
  {388, "GG", "Guernsey", NULL},
  {392, "GN", "Guinea", NULL},
  {887, "GW", "Guinea-Bissau", NULL},
  {832, "GY", "Guyana", NULL},
  {710, "HT", "Haiti", NULL},
  {239, "HM", "Heard Island and McDonald Islands", NULL},
  {410, "VA", "Vatican", "Vatican City"},
  {400, "HN", "Honduras", NULL},
  {136, "HK", "Hong Kong", NULL},
  {116, "HU", "Hungary", NULL},
  {120, "IS", "Iceland", NULL},
  {124, "IN", "India", NULL},
  {132, "ID", "Indonesia", NULL},
  {634, "IR", "Iran", NULL},
  {398, "IQ", "Iraq", NULL},
  {404, "IE", "Ireland", NULL},
  {296, "IM", "Isle of Man", NULL},
  {166, "IL", "Israel", NULL},
  {170, "IT", "Italy", NULL},
  {174, "JM", "Jamaica", NULL},
  {178, "JP", "Japan", NULL},
  {188, "JE", "Jersey", NULL},
  {192, "JO", "Jordan", NULL},
  {414, "KZ", "Kazakhstan", NULL},
  {196, "KE", "Kenya", NULL},
  {417, "KI", "Kiribati", NULL},
  {418, "KP", "North Korea", NULL},
  {426, "KR", "South Korea", NULL},
  {422, "KW", "Kuwait", NULL},
  {430, "KG", "Kyrgyzstan", NULL},
  {434, "LA", "Lao People's Democratic Republic", NULL},
  {438, "LV", "Latvia", NULL},
  {440, "LB", "Lebanon", NULL},
  {428, "LS", "Lesotho", NULL},
  {442, "LR", "Liberia", NULL},
  {446, "LY", "Libyan Arab Jamahiriya", NULL},
  {450, "LI", "Liechtenstein", NULL},
  {348, "LT", "Lithuania", NULL},
  {807, "LU", "Luxembourg", NULL},
  {458, "MO", "Macao", NULL},
  {454, "MK", "Macedonia", NULL},
  {462, "MG", "Madagascar", NULL},
  {466, "MW", "Malawi", NULL},
  {470, "MY", "Malaysia", NULL},
  {833, "MV", "Maldives", NULL},
  {504, "ML", "Mali", NULL},
  {584, "MT", "Malta", NULL},
  {474, "MH", "Marshall Islands", NULL},
  {480, "MQ", "Martinique", NULL},
  {478, "MR", "Mauritania", NULL},
  {175, "MU", "Mauritius", NULL},
  {581, "YT", "Mayotte", NULL},
  {484, "MX", "Mexico", NULL},
  {583, "FM", "Micronesia", NULL},
  {498, "MD", "Moldova", NULL},
  {492, "MC", "Monaco", NULL},
  {496, "MN", "Mongolia", NULL},
  {500, "ME", "Montenegro", NULL},
  {508, "MS", "Montserrat", NULL},
  {104, "MA", "Morocco", NULL},
  {516, "MZ", "Mozambique", NULL},
  {520, "MM", "Myanmar", NULL},
  {276, "NA", "Namibia", NULL},
  {524, "NR", "Nauru", NULL},
  {562, "NP", "Nepal", NULL},
  {566, "NL", "Netherlands", NULL},
  {558, "AN", "Netherlands Antilles", NULL},
  {570, "NC", "New Caledonia", NULL},
  {530, "NZ", "New Zealand", NULL},
  {528, "NI", "Nicaragua", NULL},
  {574, "NE", "Niger", NULL},
  {578, "NG", "Nigeria", NULL},
  {540, "NU", "Niue", NULL},
  {554, "NF", "Norfolk Island", NULL},
  {512, "MP", "Northern Mariana Islands", NULL},
  {586, "NO", "Norway", NULL},
  {585, "OM", "Oman", NULL},
  {275, "PK", "Pakistan", NULL},
  {591, "PW", "Palau", NULL},
  {598, "PS", "Palestinian National Authority", NULL},
  {600, "PA", "Panama", NULL},
  {604, "PG", "Papua New Guinea", NULL},
  {612, "PY", "Paraguay", NULL},
  {384, "PE", "Peru", NULL},
  {616, "PH", "Philippines", NULL},
  {630, "PN", "Pitcairn", NULL},
  {620, "PL", "Poland", NULL},
  { 40, "PT", "Portugal", NULL},
  {638, "PR", "Puerto Rico", NULL},
  {226, "QA", "Qatar", NULL},
  {642, "RE", "Reunion ? Réunion", NULL},
  {643, "RO", "Romania", NULL},
  {646, "RU", "Russian Federation", NULL},
  {300, "RW", "Rwanda", NULL},
  {652, "BL", "Saint Barthélemy", NULL},
  {663, "SH", "Saint Helena", NULL},
  {666, "KN", "Saint Kitts and Nevis", NULL},
  {222, "LC", "Saint Lucia", NULL},
  {882, "MF", "Saint Martin (French part)", NULL},
  {674, "PM", "Saint Pierre and Miquelon", NULL},
  {682, "VC", "Saint Vincent and the Grenadines", NULL},
  {686, "WS", "Samoa", NULL},
  {408, "SM", "San Marino", NULL},
  {580, "ST", "Sao Tome and Principe", NULL},
  {690, "SA", "Saudi Arabia", NULL},
  {694, "SN", "Senegal", NULL},
  {702, "RS", "Serbia", NULL},
  {703, "SC", "Seychelles", NULL},
  {705, "SL", "Sierra Leone", NULL},
  {706, "SG", "Singapore", NULL},
  {784, "SK", "Slovakia", NULL},
  {826, "SI", "Slovenia", NULL},
  {840, "SB", "Solomon Islands", NULL},
  {688, "SO", "Somalia", NULL},
  {140, "ZA", "South Africa", NULL},
  {736, "GS", "South Georgia and the South Sandwich Islands", NULL},
  {740, "ES", "Spain", NULL},
  {654, "LK", "Sri Lanka", NULL},
  {662, "SD", "Sudan", NULL},
  {659, "SR", "Suriname", NULL},
  {678, "SJ", "Svalbard and Jan Mayen", NULL},
  {670, "SZ", "Swaziland", NULL},
  {748, "SE", "Sweden", NULL},
  {760, "CH", "Switzerland", NULL},
  { 90, "SY", "Syrian Arab Republic", NULL},
  {724, "TW", "Taiwan", NULL},
  {744, "TJ", "Tajikistan", NULL},
  {144, "TZ", "Tanzania", NULL},
  {752, "TH", "Thailand", NULL},
  {756, "TL", "Timor-Leste", NULL},
  {762, "TG", "Togo", NULL},
  {834, "TK", "Tokelau", NULL},
  {764, "TO", "Tonga", NULL},
  {158, "TT", "Trinidad and Tobago", NULL},
  {768, "TN", "Tunisia", NULL},
  {772, "TR", "Turkey", NULL},
  {776, "TM", "Turkmenistan", NULL},
  {780, "TC", "Turks and Caicos Islands", NULL},
  {788, "TV", "Tuvalu", NULL},
  {792, "UG", "Uganda", NULL},
  {795, "UA", "Ukraine", NULL},
  {796, "AE", "United Arab Emirates", NULL},
  {798, "GB", "United Kingdom", NULL},
  {800, "US", "United States", NULL},
  {804, "UM", "United States Minor Outlying Islands", NULL},
  {858, "UY", "Uruguay", NULL},
  {860, "UZ", "Uzbekistan", NULL},
  {162, "VU", "Vanuatu", NULL},
  {548, "VE", "Venezuela", NULL},
  {336, "VN", "Viet Nam", NULL},
  {862, "VG", "Virgin Islands, British", NULL},
  {704, "VI", "Virgin Islands, U.S.", NULL},
  {626, "WF", "Wallis and Futuna", NULL},
  {876, "EH", "Western Sahara", NULL},
  {894, "YE", "Yemen", NULL},
  {732, "ZM", "Zambia", NULL},
  {716, "ZW", "Zimbabwe", NULL}
};

int country_iso2code(char *country)
{
  int i, cnt;

  if (country == NULL)
    return 0;

  //must be two characters
  if (strlen(country) != 2)
    return 0;

  //to uppercase
  char *s = (char*)country;
  while (*s != 0)
  {
    *s = (char)toupper((int)*s);
    s++;
  }

  cnt = sizeof(iso3166) / sizeof(iso3166s);
  for (i=0; i<cnt; i++)
  {
    if (strcmp(country, iso3166[i].code) == 0)
      return iso3166[i].country;
  }

  return 0;
}

int country_name2code(char *country)
{
  int i, cnt, res;

  if (country == NULL)
    return 0;

  if (strlen(country) == 0)
    return 0;

  res = 0;
  cnt = sizeof(iso3166) / sizeof(iso3166s);
  for (i=0; i<cnt; i++)
  {
    if (strcmp(country, iso3166[i].name) == 0)
    {
      res = iso3166[i].country;
      break;
    }
  }

  //printf("country_name2code '%s' = %d\n", country, res);
  return res;
}

int country_osmname2code(char *country)
{
  int i, cnt, res;

  if (country == NULL)
    return 0;

  if (strlen(country) == 0)
    return 0;

  res = 0;
  cnt = sizeof(iso3166) / sizeof(iso3166s);
  for (i=0; i<cnt; i++)
  {
    if ((iso3166[i].osmname && (strcmp(country, iso3166[i].osmname) == 0)) ||
        (strcmp(country, iso3166[i].name) == 0))
    {
      res = iso3166[i].country;
      break;
    }
  }

  //printf("country_name2code '%s' = %d\n", country, res);
  return res;
}

const char *country_code2name(int country)
{
  int i, cnt;

  cnt = sizeof(iso3166) / sizeof(iso3166s);
  for (i=0; i<cnt; i++)
  {
    if (iso3166[i].country == country)
      return iso3166[i].name;
  }

  return NULL;
}

