/* -*- Mode: C; indent-tabs-mode: s; c-basic-offset: 2; tab-width: 2 -*- */
/*
 * diacritics.c
 * Copyright (C) Mike Crash 2009 <mike@mikecrash.com>
 *
 * main.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * main.c is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <string.h>

//why the TRANSLIT on iconv doesn't work here, although the command line works good?
/*
#include <iconv.h>

int utf8_to_ascii(char *src, char *dst)
{
  iconv_t cd;
  size_t len, cnt, avail;
  char *res = dst;

  cd = iconv_open ("ASCII//TRANSLIT//IGNORE", "UTF-8");
  if (cd == (iconv_t) -1)
  {
    return -1;
  }

  len = strlen(src);
  avail = len;

  cnt = iconv (cd, &src, &len, &res, &avail);
  iconv_close(cd);
  if (cnt == -1)
  {
    return -1;
  }
  *res = 0;

  return 0;
}
*/

const char *dia[][2] = {
  {"Á", "A"},
  {"Ä", "A"},
  {"À", "A"},
  {"À", "A"},
  {"Å", "A"},
  {"Ą", "A"},
  {"Ä", "A"},
  {"Æ", "A"},
  {"á", "a"},
  {"à", "a"},
  {"â", "a"},
  {"ã", "a"},
  {"ä", "a"},
  {"å", "a"},
  {"ą", "a"},
  {"æ", "a"},
  {"Č", "C"},
  {"Ć", "C"},
  {"č", "c"},
  {"ć", "c"},
  {"É", "E"},
  {"Ě", "E"},
  {"È", "E"},
  {"Ê", "E"},
  {"Ë", "E"},
  {"Ę", "E"},
  {"é", "e"},
  {"ě", "e"},
  {"è", "e"},
  {"ê", "e"},
  {"ë", "e"},
  {"ę", "e"},
  {"Ł", "L"},
  {"Ĺ", "L"},
  {"ł", "l"},
  {"ĺ", "l"},
  {"Í", "I"},
  {"Ì", "I"},
  {"Î", "I"},
  {"Ï", "I"},
  {"í", "i"},
  {"ì", "i"},
  {"î", "i"},
  {"ï", "i"},
  {"Ň", "N"},
  {"Ń", "N"},
  {"ň", "n"},
  {"ń", "n"},
  {"Ó", "O"},
  {"Ò", "O"},
  {"Ô", "O"},
  {"Õ", "O"},
  {"Ö", "O"},
  {"Ø", "O"},
  {"ó", "o"},
  {"ò", "o"},
  {"ô", "o"},
  {"õ", "o"},
  {"ö", "o"},
  {"ø", "o"},
  {"Ů", "U"},
  {"Ú", "U"},
  {"Ù", "U"},
  {"Û", "U"},
  {"Ü", "U"},
  {"ú", "u"},
  {"ů", "u"},
  {"ù", "u"},
  {"û", "u"},
  {"ü", "u"},
  {"Ý", "Y"},
  {"ý", "y"},
  {"ÿ", "y"},
  {"š", "s"},
  {"ś", "s"},
  {"Š", "S"},
  {"Ś", "S"},
  {"ř", "r"},
  {"Ř", "R"},
  {"ň", "n"},
  {"Ň", "N"},
  {"ť", "t"},
  {"Ť", "T"},
  {"Ž", "Z"},
  {"Ź", "Z"},
  {"Ż", "Z"},
  {"ž", "z"},
  {"ź", "z"},
  {"ż", "z"}
};

int utf8_to_ascii(const char *src, char *dst, int shrink=0)
{
  unsigned int i,j;
  int found;

  while (*src)
  {
    // if UTF-8 character
    if (*src & 0x80)
    {
      //go through all known characters
      i = 0;
      while (i<(sizeof(dia)/sizeof(dia[0])))
      {
        j = 0;
        found = 1;
        //compare
        while (j<strlen(dia[i][0]))
        {
          if (dia[i][0][j]!=src[j])
          {
            found = 0;
            break;
          }
          j++;
        }
        //if match, replace
        if (found)
        {
          src = src+strlen(dia[i][0])-1;
          *dst = dia[i][1][0];
          dst++;
          break;
        }
        i++;
      }
      // no known character
      if (!found && !shrink)
      {
        //get length of UTF-8 character
        if ((*src & 0xe0)==0xc0) i = 1; else
        if ((*src & 0xf0)==0xe0) i = 2; else
        if ((*src & 0xf8)==0xf0) i = 3;
          else i = 1;
        //replace with questionmark
        *dst = '?';
        src+=i;
        dst++;
      }
    } else
    {
      if (!shrink || ( ((*src>='a')&&(*src<='z')) || ((*src>='A')&&(*src<='Z')) || ((*src>='0')&&(*src<='9')) ) )
      {
        *dst = *src;
        dst++;
      }
    }
    src++;
  }
  *dst = 0;
  return 0;
}
