/* -*- Mode: C; indent-tabs-mode: s; c-basic-offset: 2; tab-width: 2 -*- */
/*
 * map.h
 * Copyright (C) Mike Crash 2009 <mike@mikecrash.com>
 *
 * main.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * main.c is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef _WIN32
#pragma warning( disable : 4200 )
#endif

#ifndef _MCMAP_H
#define _MCMAP_H

// all ..._size  define size in map file, may be different in memory, use sizeof() instead

#define MAP_VERSION 8

#define map_char_size 1
#define map_int_size 2
#define map_long_size 4

//rounding of latitude and longitude for sectors
#define MAPSECTRECT 20000
#define MAPSECTOR(a) ((a)<0 ? (a) / MAPSECTRECT * MAPSECTRECT - MAPSECTRECT : (a) / MAPSECTRECT * MAPSECTRECT) 

//maximum nodes distance in meters 
#define MAPMAXNODEDIST 1000

typedef enum
{
  //way types
  WT_MOTORWAY,
  WT_MOTORWAYLINK,
  WT_TRUNK,
  WT_TRUNKLINK,
  WT_PRIMARY,
  WT_PRIMARYLINK,
  WT_SECONDARY,
  WT_TERTIARY,
  WT_UNCLASSIFIED,
  WT_RESIDENTIAL,
  WT_LIVING,
  WT_SERVICE,
  WT_CYCLE,
  WT_TRACK1,
  WT_TRACK2,
  WT_TRACK3,
  WT_TRACK4,
  WT_TRACK5,
  WT_BRIDLE,
  WT_PATH,
  WT_FOOT,
  WT_STEPS,
  WT_FERRY,

  //natural way types
  NT_RAIL,
  NT_TRAM,
  NT_RUNWAY,
  NT_POWERLINE,
  NT_PIER,
	NT_WEIR,
  NT_RIVER,
  NT_STREAM,
  NT_WALL,
  NT_FENCE,
  NT_BOUNDARY,

  //location types
  POI_CITY,
  POI_TOWN,
  POI_VILLAGE,
  POI_HAMLET,
  POI_SUBURB,
  POI_LOCALITY,

  //poi types
  POI_FUEL,
  POI_PUB,
  POI_RESTAURANT,
  POI_FASTFOOD,
  POI_CAFE,
  POI_ICECREAM,
  POI_TEE,
  POI_TOWER,
  POI_FOUNTAIN,
  POI_RECYCLING,
  POI_TOILETS,
  POI_POSTBOX,
  POI_CAMP,
  POI_PHONE,
  POI_ATM,
  POI_BUS,
  POI_TRAM,
  POI_TRAIN,
  POI_AIRPORT,
  POI_FORD,
  POI_ADDRESS,

  //land POIs
  POI_TREE,
  POI_STONE,
  POI_PEAK,
  POI_POWERTOWER,
  POI_POWERGENERATOR,

  //plane types
  POI_HOTEL,
  POI_CHURCH,
  POI_FIRESTATION,
  POI_MUSEUM,
  POI_HISTORIC,
  POI_HOSPITAL,
  POI_DOCTORS,
  POI_POSTOFFICE,
  POI_SHOP,
  POI_SCHOOL,
  POI_BANK,
  POI_POLICE,
  POI_PHARMACY,
  POI_TOWNHALL,
  POI_PRISON,
  POI_SHELTER,

  PT_BUILDING,
  PT_PLAYGROUND,
  PT_PITCH,
  PT_MARSH,
  PT_WATER,
  PT_SWIMMINGPOOL,
  PT_BASIN,
  PT_WATERPARK,
  PT_STADIUM,
  PT_PARKING,
  PT_CEMETERY,
  PT_SAND,
  PT_PARK,
  PT_FARM,
  PT_ORCHARD,
  PT_POWERSTATION,
  PT_FOREST,
  PT_INDUSTRIAL,
  PT_COMMERCIAL,
  PT_RESIDENTIAL,
  PT_SEA
} maptypes;

#define maxstreettype WT_FERRY
#define maxwaytype NT_BOUNDARY
#define cntwaytypes (NT_BOUNDARY+1)

#define firstplanetype WT_MOTORWAY
#define maxplanetype PT_SEA
#define cntplanetypes (PT_SEA-WT_MOTORWAY+1)

#define firstloctype POI_CITY
#define maxloctype POI_LOCALITY
#define cntloctypes (POI_LOCALITY-POI_CITY+1)

#define firstpoitype POI_FUEL
#define maxpoitype PT_SEA
#define cntpoitypes (PT_SEA-POI_FUEL+1)

#define maxmaptypes PT_SEA
#define cntmaptypes (PT_SEA+1)

typedef enum
{
  CT_CITY,
  CT_TOWN,
  CT_VILLAGE,
  CT_HAMLET,
  CT_SUBURB   //part of city, this is not currently used
} citytypes;

//maximum value of city types
#define maxaddresstypes CT_HAMLET
#define maxcitytypes CT_SUBURB
#define cntcitytypes (CT_SUBURB+1)

// FILE HEADER
typedef struct
{
  char header[6];      //string MCMAP
  int  mapversion;
  long sectorsize;     //sector size - currently must be same as MAPSECTRECT
  long flags;          //map flags
  long nodesindex;     //pointer to nodes index table
  long nodes;          //pointer to nodes table
  long planesindex;    //pointer to planes index
  long planes;         //pointer to planes as water, forest etc.
  long locsindex;      //pointer to lcations index table
  long locs;           //pointer to locations
  long poisindex;      //pointer to POIs index table
  long pois;           //pointer to POIs
  long routesindex;    //pointer to tourist and cycle routes index
  long routes;         //pointer to tourist and cycle routes
  long contoursindex;  //pointer to contours index
  long contours;       //pointer to tourist and cycle routes
  long zoomindex;      //pointer to large zoom ways index
  long zoom;           //pointer to large zoom ways
  long countries;      //pointer to countries table
  long cities;         //pointer to cities table
  long streets;        //pointer to streets table
  long addresses;      //pointer to addresses table
  long text;           //pointer to string text in UTF-8
} map_info;

#define map_info_size 92

//map flags - what's present in map
#define MF_WAYS            (1<<0)  // roads and ways
#define MF_TRACKS          (1<<1)  // tracks, footways, cycleroads
#define MF_NATURAL         (1<<2)  // natural ways like rivers, streams or powerline
#define MF_LOCATIONS       (1<<3)  // cities, towns, ...
#define MF_LANDPOIS        (1<<4)  // peaks, trees, ...
#define MF_CITYPOIS        (1<<5)  // amenities and leisures
#define MF_PLANES          (1<<6)  // forest, water, ...
#define MF_BUILDINGS       (1<<7)  // buildings
#define MF_CONTOURS       (1<<8)  // contours
#define MF_STREETS       (1<<9)  // streets
#define MF_ADDRESSES       (1<<10)  // streets


// PART HEADER
typedef struct
{
  char partid;
  long count;
} map_part;

#define map_part_size 5

#define MPID_NODESINDEX         1
#define MPID_NODES              2
#define MPID_PLANESINDEX        3
#define MPID_PLANES             4
#define MPID_LOCSINDEX          5
#define MPID_LOCS               6
#define MPID_POISINDEX          7
#define MPID_POIS               8
#define MPID_ROUTESINDEX        9
#define MPID_ROUTES            10
#define MPID_CONTOURSINDEX     11
#define MPID_CONTOURS          12
#define MPID_ZOOMINDEX         13
#define MPID_ZOOM              14
#define MPID_COUNTRIES         16
#define MPID_CITIES            17
#define MPID_STREETS           18
#define MPID_ADDRESSES         19
#define MPID_TEXT              24

// map position latitude and longitude
typedef struct
{
  long lat;
  long lon;
} map_position;

#define map_position_size 8

// one closed polygon
typedef struct
{
  int count;
  map_position points[];
} map_polygon;

#define map_polygon_size 2

//if multipolygon, first polygon outer, the rest inner, more than one outer polygons not permitted
typedef struct
{
  int count;
  map_polygon *polygon[];
} map_multipolygon;

#define map_multipolygon_size 2

// NODE SECTORS
typedef struct
{
  long lat;
  long lon;
  long count;                 //count of nodes
  long filepos;               //position of nodes in nodes section
  long size;                  //size of nodes in nodes section
} map_sector;

#define map_sector_size 20

// NODES
// sorted by node id in sector
typedef struct
{
  long node;              //id of connected node
  unsigned char waytype;
  unsigned char speed;
  unsigned int flags;
  long text;              //relative position of text (name and/or note) from start of text table
} map_cross;

#define map_cross_size 12

typedef struct
{
  unsigned int flags;         //turn flags
  unsigned char count;        //count of nodes
  long nodes[];               //node id's representing turn restriction, first node is next node, other that leads to current node, in reverse order
} map_turnrestriction;

#define map_turnrestriction_size 3

typedef struct
{
  long id;               //OSM ID
  map_position pos;
  unsigned int flags;
  unsigned char count;
  map_cross cross[];
  //unsigned char turncount;
  //map_turnrestriction turn[];
} map_node;

#define map_node_size 15  //without cross

//node flags
#define NF_BARRIER_GATE         (1<<0)
#define NF_BARRIER_LIFTGATE     (1<<1)
#define NF_BARRIER_BLOCK        (1<<2)
#define NF_SEMAPHORE            (1<<7)
#define NF_TURNRESTRICTION      (1<<15)   //there is turn restriction data after cross

//turn restriction flags
#define TRF_NODESCOUNTMASK      (0xf)      //this is count of node id's that represent the turn restriction 
#define TRF_EXCEPT_FOOT         (1<<8)     //except this vehicles
#define TRF_EXCEPT_BICYCLE      (1<<9)
#define TRF_EXCEPT_MOTORCYCLE   (1<<10)
#define TRF_EXCEPT_MOTORCAR     (1<<11)
#define TRF_EXCEPT_TRUCKS       (1<<12)
#define TRF_EXCEPT_PSV          (1<<13)
#define TRF_ALLOW               (1<<15)    //the only allowed direction
#define TRF_RESTRICT            (0<<15)    //this is restricted direction - default

//way type flags
#define WF_DIR            (1<<0)
#define WF_ONEWAY         (1<<1)
#define WF_ROUNDABOUT     (1<<2)
#define WF_TOLL           (1<<3)

#define WF_NOFOOT         (1<<4)
#define WF_NOBICYCLE      (1<<5)
#define WF_NOMOTORCYCLE   (1<<6)
#define WF_NOMOTORCAR     (1<<7)
#define WF_NOTRUCKS       (1<<8)
#define WF_NOPSV          (1<<9)

#define WF_LEVEL          0x7
#define WF_LEVELSHIFT     10
#define WF_BRIDGE         (1<<13)
#define WF_TUNNEL         (1<<14)
#define WF_FORD            (1<<15)

/*
#define WF_RED            (1<<15)
#define WF_GREEN          (1<<16)
#define WF_BLUE           (1<<17)
#define WF_YELLOW         (1<<18)
#define WF_BROWN          (1<<19)
#define WF_ORANGE         (1<<20)
#define WF_PURPLE         (1<<21)
#define WF_MTB            (1<<22)
#define WF_CYCLE          (1<<23)
#define WF_SKI            (1<<24)

#define WF_COLOR_START    15
#define WF_COLOR_END      21
#define WF_COLOR_MTB      22
#define WF_COLOR_CYCLE    23
#define WF_COLOR_SKI      24
#define WF_COLOR_MASK_FOOT    (0x7f<<15)
#define WF_COLOR_MASK_MTB     (0x1ff<<15)
#define WF_COLOR_MASK_CYCLE   WF_CYCLE
#define WF_COLOR_MASK_SKI     WF_SKI
#define WF_COLOR_COUNT        12
*/

// PLANES
typedef struct
{
  long lat;
  long lon;
  char zoom;
  long count;
  long filepos;
  long size;
} map_planesector;

#define map_planesector_size 21

typedef struct
{
  unsigned char planetype;
  map_multipolygon mp;
} map_plane;

#define map_plane_size 1    //without multipolygon

//Locations
typedef struct
{
  map_position pos;
  long count;
  long filepos;
  long size;
} map_locindex;

#define map_locindex_size 20

typedef struct
{
  unsigned char poitype;
  map_position pos;
  long name;  //relative position of name in text section
} map_loc;

#define map_loc_size 13

//POI
typedef struct
{
  map_position pos;
  long count;
  long filepos;
  long size;
} map_poiindex;

#define map_poiindex_size 20

typedef struct
{
  unsigned char poitype;
  map_position pos;
  unsigned char flags;   //various flags for POIs
  long name;  //relative position of name in text section
} map_poi;

#define map_poi_size 14

#define PF_FUEL_GASOLINE 0x01
#define PF_FUEL_DIESEL 0x02
#define PF_FUEL_LPG 0x04
#define PF_FUEL_CNG 0x08


// ADDRESS
typedef struct
{
  long id;
  long citiesfilepos;       //file position to country cities relative to cities section
} map_country;

#define map_country_size 8

typedef struct
{
  long id;
  long country;
  map_position pos;
  long name;               //file position to text
  long streetsfilepos;     //file position to streets
} map_city;

#define map_city_size 24

typedef struct
{
  long id;
  long city;
  map_position pos;
  long addressesfilepos;    //file position to addresses/house numbers
  long name;                //file position to text
} map_street;

#define map_street_size 24

typedef struct
{
  long id;
  long street;
  map_position pos;
  long name;               //file position to text
} map_address;

#define map_address_size 20

//area
typedef struct
{
  char areatype;
  char flags;
  int count;
  map_position pos;
} map_area;

//text
typedef struct
{
  int size;          //total size of text - sum of name and note including ending character
  char text[];       //text ends with char 0 for name and 1 for note
} map_text;

#define map_text_size 2

// errors
#define MERR_HEADER 1             //Invalid header
#define MERR_VERSION 2            //Invalid map version

#endif
