/* -*- Mode: C; indent-tabs-mode: s; c-basic-offset: 2; tab-width: 2 -*- */
/*
 * mcthread.c
 * Copyright (C) Mike Crash 2009 <mike@mikecrash.com>
 * 
 * main.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * main.c is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <time.h>
//#include <sys/time.h>
#include <mcthread.h>


//thread
int mcthread_create(mcthread *thread, void *(*start_routine)(void*), void *arg, int nice)
{
  pthread_attr_t attr;
  pthread_attr_init(&attr);
  pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_JOINABLE);

  return pthread_create(thread, &attr, start_routine, arg);
}

int mcthread_join(mcthread *thread)
{
  return pthread_join(*thread, NULL);
}

//mutex
int mcmutex_init(mcmutex *mutex)
{
  return pthread_mutex_init(mutex, NULL);
}

int mcmutex_destroy(mcmutex *mutex)
{
  return pthread_mutex_destroy(mutex);
}

int mcmutex_lock(mcmutex *mutex)
{
  return pthread_mutex_lock(mutex);
}

int mcmutex_unlock(mcmutex *mutex)
{
  return pthread_mutex_unlock(mutex);
}

//semaphore

int mcsema_init(mcsema *sema)
{
  return sem_init(sema, 0, 0);
}

int mcsema_destroy(mcsema *sema)
{
  return sem_destroy(sema);
}

int mcsema_signal(mcsema *sema)
{
  return sem_post(sema);
}

int mcsema_wait(mcsema *sema)
{
  return sem_wait(sema);
}

int mcsema_waittimed(mcsema *sema, long miliseconds)
{
  struct timespec timer;
  clock_gettime(CLOCK_REALTIME, &timer);
  timer.tv_sec += miliseconds / 1000;
  timer.tv_nsec += (miliseconds % 1000) * 1000000;
  if (timer.tv_nsec > 1000000000L)
  {
    timer.tv_sec++;
    timer.tv_nsec -= 1000000000L;
  }
  return sem_timedwait(sema, &timer);
}
