/* -*- Mode: C; indent-tabs-mode: s; c-basic-offset: 2; tab-width: 2 -*- */
/*
 * map_polygon.c
 * Copyright (C) Mike Crash 2009 <mike@mikecrash.com>
 * 
 * main.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * main.c is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "polygon.h"
#include "projection.h"
//#include "map_common.h"

map_polygon *map_newpolygon(int count)
{
  if (count<=0)
    return NULL;
  map_polygon *p = (map_polygon *)malloc(sizeof(map_polygon *) + sizeof(map_position) * count);
  p->count = count;
  return p;
}

map_multipolygon *map_newmultipolygon(int count)
{
  map_multipolygon *mp = (map_multipolygon *)malloc(sizeof(map_multipolygon *) + sizeof(map_polygon *) * count);
  mp->count = count;
  memset(mp->polygon, 0, sizeof(map_polygon*)*count); 
  return mp;
}

map_multipolygons *map_newmultipolygons(int count)
{
  map_multipolygons *mps = (map_multipolygons *)malloc(sizeof(map_multipolygons *) + sizeof(map_multipolygon *) * count);
  mps->count = count;
  memset(mps->mp, 0, sizeof(map_multipolygon*)*count); 
  return mps;
}

map_polygon *map_duplicatepolygon(map_polygon *poly)
{
  if (poly == NULL)
    return NULL;
  map_polygon *np = map_newpolygon (poly->count);
  memcpy(np->points, poly->points, poly->count * sizeof(map_position));
  return np;
}

map_multipolygon *map_duplicatemultipolygon(map_multipolygon *mp)
{
  if (mp == NULL)
    return NULL;
  map_multipolygon *nmp = map_newmultipolygon (mp->count);
  for (int i=0; i<mp->count; i++)
    nmp->polygon[i] = map_duplicatepolygon(mp->polygon[i]);
  return nmp;
}

map_multipolygons *map_duplicatemultipolygons(map_multipolygons *mps)
{
  if (mps == NULL)
    return NULL;
  map_multipolygons *nmps = map_newmultipolygons (mps->count);
  for (int i=0; i<mps->count; i++)
    nmps->mp[i] = map_duplicatemultipolygon(mps->mp[i]);
  return nmps;
}

int map_polygonarea(map_polygon *polygon, long *lat1, long *lon1, long *lat2, long *lon2)
{
  int i;
  
  if (polygon==NULL)
    return -1;

  *lat1 = polygon->points[0].lat;
  *lon1 = polygon->points[0].lon;
  *lat2 = *lat1;
  *lon2 = *lon1;
  for (i=1; i<polygon->count; i++)
  {
    if (polygon->points[i].lat < *lat1)
      *lat1 = polygon->points[i].lat;
    if (polygon->points[i].lon < *lon1)
      *lon1 = polygon->points[i].lon;
    if (polygon->points[i].lat > *lat2)
      *lat2 = polygon->points[i].lat;
    if (polygon->points[i].lon > *lon2)
      *lon2 = polygon->points[i].lon;
  }
  return 0;
}

//return size of multipolygon data in file (size of long = 4)
//do not count the last point - it is duplicated and we don't write it to file
long map_multipolygonsize(map_multipolygon *mp)
{
  long size = map_multipolygon_size;  //size of polygons count - int (maximum=65536)
  int i;
  for (i=0; i<mp->count; i++)
  {
    size += map_polygon_size + mp->polygon[i]->count * map_position_size;
  }
  return size;
}

//is point inside the polygon?
int map_pointinpolygon(map_position *pos, map_polygon *poly)
{
  int i, j, num=0;
  if (poly == NULL)
    return -1;
  j = poly->count-1;
  //printf("Check %ld,%ld in %d - %p %p\n", pos->lat, pos->lon, poly->count, pos, poly);
  for (i=0; i<poly->count; i++)
  {
    /*sprintf(s, "  %ld,%ld -> %ld,%ld\n", poly->points[j].lat, poly->points[j].lon, poly->points[i].lat, poly->points[i].lon);
    map_log(s);*/
    /*if ((pos->lat==50268241)&&(pos->lon==14521001))
    {
      char s[100];
      sprintf(s, "  %ld,%ld -> %ld,%ld\n", poly->points[j].lat, poly->points[j].lon, poly->points[i].lat, poly->points[i].lon);
      map_log(s);
    }*/
    long lat1 = poly->points[i].lat < poly->points[j].lat ? poly->points[i].lat : poly->points[j].lat;
    long lat2 = poly->points[i].lat > poly->points[j].lat ? poly->points[i].lat : poly->points[j].lat;
    if ((pos->lat>=lat1) && (pos->lat<lat2))
    {
      if ((poly->points[i].lon + 
          (pos->lat - poly->points[i].lat) * 
          (poly->points[j].lon - poly->points[i].lon) / 
          (poly->points[j].lat - poly->points[i].lat)) <= pos->lon)
      {
        /*char s[100];
        sprintf(s, "    YES\n");
        map_log(s);*/
        //char s[100];
        //printf(" %d %ld,%ld -> %ld,%ld\n", i, poly->points[j].lat, poly->points[j].lon, poly->points[i].lat, poly->points[i].lon);
        //sprintf(s, "    YES\n");
        //map_log(s);
        num++;
      }
    }
    j = i;
  }
  if ((num % 2)==1)
  {
    //printf("  Point %ld,%ld inside (%d)\n", pos->lat, pos->lon, num);
    return 1;
  }
  //printf("  Point %ld,%ld outside (%d)\n", pos->lat, pos->lon, num);
  return 0;
}

int map_pointinmultipolygon(map_position *pos, map_multipolygon *mp)
{
  int i;

  if (map_pointinpolygon(pos, mp->polygon[0]) == 0)
    return 0;

  for (i=1; i<mp->count; i++)
  {
    if (map_pointinpolygon(pos, mp->polygon[i]) == 1)
      return 0;
  }

  return 1;
}

//is polygon inside the polygon?
//check all points are insade and no cross exists
int map_polygoninpolygon(map_polygon *inner, map_polygon *outer)
{
  int i = 0;
  if ((inner == NULL) || (outer == NULL))
    return -1;
  /*char s[30];
  sprintf(s, "Check polygon with %d points in %d\n", inner->count, outer->count);
  map_log(s);*/
  while (i<inner->count)
  {
    if (map_pointinpolygon(&inner->points[i], outer) == 0)
      return 0;
    i++;
  }
  if (map_polygoncrosspolygon(inner, outer))
    return 0;
  else
    return 1;
}

//check that olygons cross each other
int map_polygoncrosspolygon(map_polygon *inner, map_polygon *outer)
{
  int pindex = 0;
  int pindex2 = inner->count-1;
  while (pindex<inner->count)
  {
    int cindex = 0;
    int cindex2 = outer->count-1;
    while (cindex<outer->count)
    {
      map_position pos;
      if (wgs84_cross (inner->points[pindex2].lat, inner->points[pindex2].lon,
                       inner->points[pindex].lat, inner->points[pindex].lon,
                       outer->points[cindex2].lat, outer->points[cindex2].lon, 
                       outer->points[cindex].lat, outer->points[cindex].lon,
                       &pos.lat, &pos.lon) == 1)
      {
        return 1;
      }
      cindex2 = cindex;
      cindex++;
    }
    pindex2 = pindex;
    pindex++;
  }  
  return 0;
}

//check polygon is convex, eg. it self crosses
int map_polygonselfcross(map_polygon *polygon)
{
  int pindex = 0;
  int pindex2 = polygon->count-1;
  int p = 1;  //do not check first and last abscisse
  while (pindex<(polygon->count-2))
  {
    int cindex = pindex+2;
    int cindex2 = pindex+1;
    while (cindex<(polygon->count-p))
    {
      map_position pos;
      if (wgs84_cross (polygon->points[pindex2].lat, polygon->points[pindex2].lon,
                       polygon->points[pindex].lat, polygon->points[pindex].lon,
                       polygon->points[cindex2].lat, polygon->points[cindex2].lon, 
                       polygon->points[cindex].lat, polygon->points[cindex].lon,
                       &pos.lat, &pos.lon) == 1)
      {
        return 1;
      }
      cindex2 = cindex;
      cindex++;
    }
    pindex2 = pindex;
    pindex++;
    p = 0;
  }  
  return 0;
}

//return new polygon, that goes always in clockwise direction
map_polygon *map_polygonclockwise(map_polygon *polygon, int counterclockwise)
{
  //detect ordering - clockwise or counterclockwise and fix it
  //we need always clockwise ordering
  int i;
  long v = 0;
  map_polygon *newpoly;
  int l = polygon->count-1;
  for (i=0; i<polygon->count; i++)
  {
    v += (polygon->points[l].lon * polygon->points[i].lat - polygon->points[i].lon * polygon->points[l].lat);
    l = i;
  }
  if (counterclockwise)
    v = -v;
  if (v>0)
  {
    //reverse direction
    newpoly = map_newpolygon (polygon->count);
    for (i=0; i<polygon->count; i++)
    {
      newpoly->points[i] = polygon->points[polygon->count-1-i];
      //printf("%d - %ld,%ld\n", i, newpoly->points[i].lat, newpoly->points[i].lon);
    }
  }
  else
  {
    //good ordering - just copy
    newpoly = map_duplicatepolygon(polygon);
    /*for (i=0; i<newpoly->count; i++)
    {
      printf("%d - %ld,%ld\n", i, newpoly->points[i].lat, newpoly->points[i].lon);
    }*/
  }
  return newpoly;
}


//split one unsorted multipolygon to multiple with one outer and any inner polygons
//return pointer to the array of multipolygons
//TODO change return value to map_multipolygons
int map_multipolygons_split(map_multipolygon *mp, int *count, map_multipolygon ***mps)
{
  int i,j;
  //char s[100];
  
  if (mp==NULL)
    return -1;
  if (mp->count == 0)
    return -1;

  //num of outer polys found    
  int polys = 0;
  //array with inner polys
  i = sizeof(int) * mp->count * mp->count;
  int *arr = (int*)malloc(i);
  memset(arr, 0, i);
  //array of counts of outer poly + inner polys 
  i = sizeof(int) * mp->count;
  int *inner = (int*)malloc(i);
  memset(inner, 0, i);
  //array of poly usage
  i = sizeof(int) * mp->count;
  int *used = (int *)malloc(i);
  memset(used, 0, i);
  
  /*sprintf(s, "Split multipolygon %p with %d polygons\n", mp, mp->count);
  map_log(s);*/

  //detect all polys
  for (i=0; i<mp->count; i++)
  {
    /*sprintf(s, "  Polygon %p with %d points\n", mp->polygon[i], mp->polygon[i]->count);
    map_log(s);*/
    int outer = 1;
    //check if it is outer - it is not inside of any other polygon
    if (used[i]==0)
    {
      for (j=0; j<mp->count; j++)
      {
        if (i!=j)
        {
          if (map_polygoninpolygon(mp->polygon[i], mp->polygon[j]) == 1)
          {
            outer = 0;
            break;
          }
        }
      }
      //if outer, find all polys inside if any
      if (outer)
      {
        //first poly is outer
        used[i] = 1;
        arr[polys*mp->count + inner[polys]] = i;
        inner[polys]++;
        /*sprintf(s, "  Outer %d\n", i);
        map_log(s);*/
        
        //and find the inner
        for (j=0; j<mp->count; j++)
        {
          if ((used[j]==0) && (j!=i))
          {
            if (map_polygoninpolygon(mp->polygon[j], mp->polygon[i]) == 1)
            {
              arr[polys*mp->count + inner[polys]] = j;
              inner[polys]++;
              used[j] = 1;
              /*sprintf(s, "    Inner %d\n", j);
              map_log(s);*/
            }
          }
        }
        polys++;
      }
    }
  }
  //create new multipolys
  /*sprintf(s, "Polys %d\n", polys);
  map_log(s);*/
  map_multipolygon **mpst = (map_multipolygon **)malloc(sizeof(map_multipolygon *) * polys);
  for (i=0; i<polys; i++)
  {
    mpst[i] = map_newmultipolygon(inner[i]);
    /*sprintf(s, "MP %d count %d %p\n", i, inner[i], mpst[i]);
    map_log(s);*/
    for (j=0; j<inner[i]; j++)
    {
      mpst[i]->polygon[j] = mp->polygon[arr[i*mp->count+j]];
      /*sprintf(s, " P%d %p count %d arr %d %p\n", j, mpst[i], mpst[i]->polygon[j]->count, arr[i*mp->count+j], mp->polygon[arr[i*mp->count+j]]);
      map_log(s);*/
    }
  }
  *count = polys;
  *mps = mpst;
  free(mp);
  free(inner);
  free(used);
  free(arr);
  return 0;
}

char *polygon2ascii(map_polygon *polygon)
{
  if (polygon == NULL)
    return NULL;

  long len = polygon->count*16+12;
  char *str = (char*)malloc(len);
  char *arr = str;

  //polygon points count
  sprintf(arr, "0x%08x", polygon->count);
  arr += 10;
  //polygon points
  for (int j=0; j<polygon->count; j++)
  {
    long lat = polygon->points[j].lat & 0xffffffff;  //fix for 64-bit systems, stretch to 32 bit
    long lon = polygon->points[j].lon & 0xffffffff;
    sprintf(arr, "%08lx%08lx", lat, lon);
    arr += 16;
  }
  return str;
}

map_polygon *ascii2polygon(unsigned char *arr)
{
  int i, j, k;

  int cnt = (arr[0]<<24) + (arr[1]<<16) + (arr[2]<<8) + arr[3];
  arr += 4;
  map_polygon *poly = map_newpolygon(cnt);
  map_position *pos = poly->points;

  for (i=0; i<cnt; i++)
  {
    pos->lat = 0;
    pos->lon = 0;
    for (j=0; j<4; j++)
    {
      pos->lat = (pos->lat << 8) + *arr;
      arr++;
    }
    for (j=0; j<4; j++)
    {
      pos->lon = (pos->lon << 8) + *arr;
      arr++;
    }
#if __WORDSIZE == 64
    if (pos->lat & 0x80000000)
      pos->lat |= 0xffffffff00000000;
    if (pos->lon & 0x80000000)
      pos->lon |= 0xffffffff00000000;
#endif
    //printf("%d - %d - %lld\n", ri[i].type, ri[i].role, ri[i].id);
    pos++;
  }
  return poly;
}

char *multipolygon2ascii(map_multipolygon *mp)
{
  if (mp == NULL)
    return NULL;

  long len = 11;
  for (int i=0; i<mp->count; i++)
  {
    len += mp->polygon[i]->count * 16 + 8;
  }
  char *str = (char*)malloc(len);
  char *arr = str;

  //multipolygon count
  sprintf(arr, "0x%08x", mp->count);
  arr += 10;
  //multipolygons
  for (int i=0; i<mp->count; i++)
  {
    //polygon points count
    sprintf(arr, "%08x", mp->polygon[i]->count);
    arr += 8;
    //polygon points
    for (int j=0; j<mp->polygon[i]->count; j++)
    {
      long lat = mp->polygon[i]->points[j].lat & 0xffffffff;  //fix for 64-bit systems, stretch to 32 bit
      long lon = mp->polygon[i]->points[j].lon & 0xffffffff;
      sprintf(arr, "%08lx%08lx", lat, lon);
      arr += 16;
    }
  }
  return str;
}

map_multipolygon *ascii2multipolygon(const char *str)
{
  int i, j, k;
  const unsigned char *arr = (const unsigned char*)str;

  int polys = (arr[0]<<24) + (arr[1]<<16) + (arr[2]<<8) + arr[3];
  arr += 4;
  map_multipolygon *mp = map_newmultipolygon(polys);

  for (k=0; k<polys; k++)
  {
    int cnt = (arr[0]<<24) + (arr[1]<<16) + (arr[2]<<8) + arr[3];
    arr += 4;
    map_polygon *poly = map_newpolygon(cnt);
    mp->polygon[k] = poly;
    map_position *pos2 = poly->points;

    for (i=0; i<cnt; i++)
    {
      pos2->lat = 0;
      pos2->lon = 0;
      for (j=0; j<4; j++)
      {
        pos2->lat = (pos2->lat << 8) + *arr;
        arr++;
      }
      for (j=0; j<4; j++)
      {
        pos2->lon = (pos2->lon << 8) + *arr;
        arr++;
      }
      //fix for 64-bit systems
#if __WORDSIZE == 64
    if (pos2->lat & 0x80000000)
      pos2->lat |= 0xffffffff00000000;
    if (pos2->lon & 0x80000000)
      pos2->lon |= 0xffffffff00000000;
#endif
      //printf("%d - %d - %lld\n", ri[i].type, ri[i].role, ri[i].id);
      pos2++;
    }
  }
  return mp;
}

int map_freepolygon(map_polygon *p)
{
  if (p==NULL)
    return -1;
  free(p);
  return 0;
}

int map_freemultipolygon(map_multipolygon *mp)
{
  int i;
  if (mp==NULL)
    return -1;
  for (i=0; i<mp->count; i++)
  {
    map_freepolygon(mp->polygon[i]);
  }
  free(mp);
  return 0;
}

int map_freemultipolygons(map_multipolygons *mp)
{
  int i;
  if (mp==NULL)
    return -1;
  for (i=0; i<mp->count; i++)
  {
    map_freemultipolygon(mp->mp[i]);
  }
  free(mp);
  return 0;
}

int fixpolygon(map_polygon *polygon)
{
  //remove centroid as first coordinate, if second and last is the same
  if (wgs84_compare(polygon->points[polygon->count-1].lat, polygon->points[polygon->count-1].lon, 
                    polygon->points[1].lat, polygon->points[1].lon) == 0)
  {
    for (int j=0; j<polygon->count; j++)
    {
      polygon->points[j] = polygon->points[j+1];
    }
    polygon->count--;
  }
  //remove last point if same as the first 
  if (wgs84_compare(polygon->points[polygon->count-1].lat, polygon->points[polygon->count-1].lon, 
                    polygon->points[0].lat, polygon->points[0].lon) == 0)
  {
    polygon->count--;
  }
  return 0;
}

map_multipolygons *load_multipolygons(const char *filename)
{
  char s[100];
  int polyscnt[255];
  int subpolyscnt[255];
  int polycnt = 0;
  int subpolycnt = 0;
  map_polygon **polys = NULL;
  map_polygon **subpolys = NULL;
  map_multipolygons *mps = NULL;

  FILE *f = fopen(filename, "r");
  if (f == NULL)
  {
    //map_log(LOG_ERROR, "Cannot open multipolygon file %s\n", filename);
    return NULL;
  }

  // determine number of polygons and subpolygons and read it
  // run two steps - first for number of polygons and point count
  // second read the data
  for (int step=0; step<2; step++)
  {
    int line = 0;
    int subline = 0;
    int sub = 0;
    polycnt = 0;
    subpolycnt = 0;
    while (!feof(f))
    {
      fgets(s, sizeof(s), f);
      if (strlen(s) == 0)
        continue;
      if (line == 0)
      {
        //this is filename - do nothing
      }
      else if (strstr(s, "END") == s)
      {
        if (subline == 0)
        {
          break;
        }
        else
        {
          if (sub)
            subpolyscnt[subpolycnt-1] = subline-1;
          else
            polyscnt[polycnt-1] = subline-1;
          subline = 0;
        }
      }
      else if (subline == 0)
      {
        //this is polygon name
        if (s[0] == '!')
        {
          if (step)
          {
            subpolys[subpolycnt] = map_newpolygon(subpolyscnt[subpolycnt]);
          }
          else if (subpolycnt>0)
          {
            subpolys[subpolycnt-1]->count = subline - 1;
          }
          subpolycnt++;
          sub = 1;
        }
        else
        {
          if (step)
          {
            polys[polycnt] = map_newpolygon(polyscnt[polycnt]);
          }
          else if (polycnt>0)
          {
            polys[polycnt-1]->count = subline - 1;
          }
          polycnt++;
          sub = 0;
        }
        subline++;
      }
      else
      {
        //check structure
        char *l;
        long lat = 0, lon = 0;
        lon = (long)(strtod(s, &l) * 1000000);
        if (l[0] == ',')
          l++;
        lat = (long)(strtod(l, NULL) * 1000000);
        if (((lat == 0) && (lon==0)) || (lat<-180000000) || (lat>180000000) || (lon<-180000000) || (lon>180000000))
        {
          //error in file
          fclose(f);
          //map_log(LOG_ERROR, "Bad structure in multipolygon file %s\n", filename);
          return NULL;
        }
        if (step == 1)
        {
          if (sub)
          {
            subpolys[subpolycnt-1]->points[subline-1].lat = lat;
            subpolys[subpolycnt-1]->points[subline-1].lon = lon;
          }
          else
          {
            polys[polycnt-1]->points[subline-1].lat = lat;
            polys[polycnt-1]->points[subline-1].lon = lon;
          }
        }
        subline++;
      }
      line++;
    }
    if (step == 0)
    {
      if (polycnt)
        polys = (map_polygon **)malloc(sizeof(map_polygon*) * polycnt);
      if (subpolycnt)
        subpolys = (map_polygon **)malloc(sizeof(map_polygon*) * subpolycnt);
      fseek(f, 0, SEEK_SET);
    }
  }
  fclose(f);

  if (polycnt == 0)
  {
    //map_log(LOG_ERROR, "No multipolygon file %s\n", filename);
    return NULL;
  }

  //detect centroid if present in polygons, remove duplicated last point
  for (int i=0; i<polycnt; i++)
  {
    fixpolygon(polys[i]);
  }
  for (int i=0; i<subpolycnt; i++)
  {
    fixpolygon(subpolys[i]);
  }

  //now create multipolygons and insert subpolygons
  mps = map_newmultipolygons(polycnt);
  for (int i=0; i<polycnt; i++)
  {
    //detect number of slave subpolygons
    int c = 0;
    for (int j=0; j<subpolycnt; j++)
    {
      if (map_polygoninpolygon(polys[i], subpolys[j]))
      {
        c++;
      }
    }
    mps->mp[i] = map_newmultipolygon(c+1);
    mps->mp[i]->polygon[0] = polys[i];
    c = 1;
    for (int j=0; j<subpolycnt; j++)
    {
      if (subpolys[j] && (map_polygoninpolygon(polys[i], subpolys[j])))
      {
        mps->mp[i]->polygon[c] = subpolys[j];
        subpolys[j] = NULL;
        c++;
      }
    }
    
    polys[i] = NULL;
  }

  if (polys)
    free(polys);
  if (subpolys)
    free(subpolys);
  
  return mps;
}

int save_polygon(map_polygon *polygon, const char *filename)
{
  char s[100];
  FILE *f = fopen(filename, "w");
  if (f == NULL)
  {
    //map_log(LOG_ERROR, "Cannot open polygon file %s\n", filename);
    return -1;
  }
  fputs("poly\n", f);
  sprintf(s, "%d\n", polygon->count);
  fputs(s, f);
  for (int i=0; i<polygon->count; i++)
  {
    sprintf(s, "  %f  %f\n", (double)polygon->points[i].lon/1000000, (double)polygon->points[i].lat/1000000);
    fputs(s, f);
  }
  fputs("END\n", f);
  fputs("END\n", f);
  fclose(f);
  return 0;
}
