/* -*- Mode: C; indent-tabs-mode: s; c-basic-offset: 2; tab-width: 2 -*- */
/*
 * typeio.c
 * Copyright (C) Mike Crash 2009 <mike@mikecrash.com>
 * 
 * main.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * main.c is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <string.h>
#include "typeio.h"

size_t fwrite_long(const long *data, size_t count, FILE *stream)
{
  size_t i;
  size_t res = 0;
  for (i=0; i<count; i++)
  {
    if ((res = fwrite(data++, 4, 1, stream)) != 1)
      return res;
  }
  return res;
}

size_t fwrite_ulong(const unsigned long *data, size_t count, FILE *stream)
{
  size_t i;
  size_t res = 0;
  for (i=0; i<count; i++)
  {
    if ((res = fwrite(data++, 4, 1, stream)) != 1)
      return res;
  }
  return res;
}

size_t fwrite_int(const int *data, size_t count, FILE *stream)
{
  size_t i;
  size_t res = 0;
  for (i=0; i<count; i++)
  {
    if ((res = fwrite(data++, 2, 1, stream)) != 1)
      return res;
  }
  return res;
}

size_t fwrite_uint(const unsigned int *data, size_t count, FILE *stream)
{
  size_t i;
  size_t res = 0;
  for (i=0; i<count; i++)
  {
    if ((res = fwrite(data++, 2, 1, stream)) != 1)
      return res;
  }
  return res;
}

size_t fwrite_char(const char *data, size_t count, FILE *stream)
{
  size_t i;
  size_t res = 0;
  for (i=0; i<count; i++)
  {
    if ((res = fwrite(data++, 1, 1, stream)) != 1)
      return res;
  }
  return res;
}

size_t fwrite_uchar(const unsigned char *data, size_t count, FILE *stream)
{
  size_t i;
  size_t res = 0;
  for (i=0; i<count; i++)
  {
    if ((res = fwrite(data++, 1, 1, stream)) != 1)
      return res;
  }
  return res;
}

size_t fread_long(long *data, size_t count, FILE *stream)
{
  size_t i;
  size_t res = 0;
  for (i=0; i<count; i++)
  {
    *data = 0;
    if ((res = fread(data, 4, 1, stream)) != 1)
    {
      return res;
    }
#if  __WORDSIZE == 64
    if (*data & 0x80000000)
      *data |=0xffffffff00000000;
#endif
    data++;
  }
  return res;
}

size_t fread_ulong(unsigned long *data, size_t count, FILE *stream)
{
  size_t i;
  size_t res = 0;
  for (i=0; i<count; i++)
  {
    *data = 0;
    if ((res = fread(data++, 4, 1, stream)) != 1)
      return res;
  }
  return res;
}

size_t fread_int(int *data, size_t count, FILE *stream)
{
  size_t i;
  size_t res = 0;
  for (i=0; i<count; i++)
  {
    *data = 0;
    if ((res = fread(data++, 2, 1, stream)) != 1)
      return res;
  }
  return res;
}

size_t fread_uint(unsigned int *data, size_t count, FILE *stream)
{
  size_t i;
  size_t res = 0;
  for (i=0; i<count; i++)
  {
    *data = 0;
    if ((res = fread(data++, 2, 1, stream)) != 1)
      return res;
  }
  return res;
}

size_t fread_char(char *data, size_t count, FILE *stream)
{
  size_t i;
  size_t res = 0;
  for (i=0; i<count; i++)
  {
    *data = 0;
    if ((res = fread(data++, 1, 1, stream)) != 1)
      return res;
  }
  return res;
}

size_t fread_uchar(unsigned char *data, size_t count, FILE *stream)
{
  size_t i;
  size_t res = 0;
  for (i=0; i<count; i++)
  {
    *data = 0;
    if ((res = fread(data++, 1, 1, stream)) != 1)
      return res;
  }
  return res;
}

long read_long(char **data)
{
  unsigned char *buf = (unsigned char*)*data;
  long res = (((long)buf[3]) << 24) + (((long)buf[2])<<16) + (((long)buf[1])<<8) + buf[0];
#if __WORDSIZE == 64
  if (res & 0x80000000)
    res |= 0xffffffff00000000;
#endif
  *data += 4;
  return res;
}

unsigned long read_ulong(char **data)
{
  unsigned char *buf = (unsigned char*)*data;
  unsigned long res =(( (unsigned long)buf[3])<<24) + (((unsigned long)buf[2])<<16) + (((unsigned long)buf[1])<<8) + (unsigned long)buf[0];
  *data += 4;
  return res;
}
int read_int(char **data)
{
  unsigned char *buf = (unsigned char*)*data;
  int res =(((int)buf[1])<<8) + (int)buf[0];
  *data += 2;
  return res;
}

unsigned int read_uint(char **data)
{
  unsigned char *buf = (unsigned char*)*data;
  unsigned int res = (((unsigned int)buf[1])<<8) + (unsigned int)buf[0];
  *data += 2;
  return res;
}

char read_char(char **data)
{
  char *buf = *data;
  char res = buf[0];
  *data += 1;
  return res;
}

unsigned char read_uchar(char **data)
{
  unsigned char *buf = (unsigned char*)*data;
  unsigned char res = (unsigned char)buf[0];
  *data += 1;
  return res;
}

void read_string(char **data, char *s, int len)
{
  memcpy(s, *data, len);
  *data += len;
}
