#include "stdafx.h"
#include <iostream>
#include <fstream>
#include <math.h>
using namespace std;

#define BYTE    unsigned char 
#define WORD    unsigned short
#define DWORD   unsigned long  
#define NULL	0
#define RGB(r,g,b)    ( ((DWORD)(BYTE)r)|((DWORD)((BYTE)g)<<8)|((DWORD)((BYTE)b)<<16) )
#define GetRValue(RGBColor)    (BYTE) (RGBColor)
#define GetGValue(RGBColor)    (BYTE) (((DWORD)RGBColor) >> 8)
#define GetBValue(RGBColor)    (BYTE) (((DWORD)RGBColor) >> 16)

#pragma pack(2)


struct BITMAPFILEHEADER
{
    WORD    bfType;
    DWORD   bfSize;
    WORD    bfReserved1;
    WORD    bfReserved2;
    DWORD   bfOffBits;
};

struct BITMAPINFOHEADER
{
    DWORD    biSize;
    DWORD    biWidth;
    DWORD    biHeight;
    WORD    biPlanes;
    WORD    biBitCount;
    DWORD    biCompression;
    DWORD    biSizeImage;
    DWORD    biXPelsPerMeter;
    DWORD    biYPelsPerMeter;
    DWORD    biClrUsed;
    DWORD    biClrImportant;
};



class CBitmap
{
    DWORD*    Colors;
    DWORD    Width;
    DWORD    Height;
	

public:
    
    DWORD GetWidth(){return Width;};
    DWORD GetHeight(){return Height;};

    CBitmap()
    {
        Colors = NULL;
        Width = Height = 0;
    }
    ~CBitmap()
    {
        if(Colors)delete[] Colors;
    }

    DWORD& Access(int x, int y)
    {
        return Colors[x*Height+y];
    }





bool Load(const char* FileName)
    {
        BITMAPFILEHEADER    BitmapFileHeader;
        BITMAPINFOHEADER    BitmapInfoHeader;

        ifstream infile(FileName, ios::binary);
        infile.read((char*)&BitmapFileHeader, sizeof(BitmapFileHeader));
        infile.read((char*)&BitmapInfoHeader, sizeof(BitmapInfoHeader));

        if(BitmapFileHeader.bfType != 0x4d42 
            || BitmapInfoHeader.biClrUsed!=0
            || BitmapInfoHeader.biBitCount!=24 
            || BitmapInfoHeader.biCompression!=0 
            || BitmapInfoHeader.biPlanes!=1)return false;

        Width = BitmapInfoHeader.biWidth;
        Height = BitmapInfoHeader.biHeight;

        BYTE* ColorsData = new BYTE[3*Width*Height];
        Colors = new DWORD[Width*Height];
        
        infile.read((char*)ColorsData, 3*Width*Height);

        DWORD    Offset = 0;

        for(int y=Height-1;y>=0;y--)
        {
            for(int x=0;x<(int)Width;x++, Offset+=3)
            {
                BYTE B = ColorsData[Offset + 0];
                BYTE G = ColorsData[Offset + 1];
                BYTE R = ColorsData[Offset + 2];
                Access(x,y) = RGB(R,G,B);
            }
        }

        delete[] ColorsData;

        infile.close();
        return true;
    }

    bool Save(const char* FileName)
    {
        if(Colors==NULL)return false;

        BITMAPFILEHEADER    BitmapFileHeader = {
            0x4d42, //Bmp Mark
            sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER) + 3 * Width * Height, //File Size
            0,    //Not Used
            0,    //Not Used
            sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER)    //Colors Offset in File
        };
        
        BITMAPINFOHEADER    BitmapInfoHeader = {
            sizeof(BITMAPINFOHEADER),    //Structure Size
            Width,
            Height,
            1,        //Number of Plans
            24,        //Bits Per colors 
            0,        //Compression Ration
            0,        //Original Image Size (When Compressed)
            2835,    //Number of Pixel Per Meter (Vertical)
            2835,    //Number of Pixel Per Meter (Horizontal)
            0,        //Number of Used Colors (Indexed Mode)
            0,        //Number of Important Colors (Indexed Mode)
        };

        BYTE* ColorsData = new BYTE[3*Width*Height];

        DWORD    Offset = 0;

        for(int y=Height-1;y>=0;y--)
        {
            for(int x=0;x<(int)Width;x++, Offset+=3)
            {
                DWORD    Color = Access(x,y);
                ColorsData[Offset + 0] = GetBValue(Color);
                ColorsData[Offset + 1] = GetGValue(Color);
                ColorsData[Offset + 2] = GetRValue(Color);
            }
        }


        ofstream outfile(FileName, ios::binary);
        outfile.write((char*)&BitmapFileHeader, sizeof(BitmapFileHeader));
        outfile.write((char*)&BitmapInfoHeader, sizeof(BitmapInfoHeader));
        outfile.write((char*)ColorsData, 3*Width*Height);
        outfile.close();

        delete[] ColorsData;
        return false;
    }

void Create(int W, int H, DWORD BackColor)
       {
               if(Colors)
                    delete[] Colors;
                
                Width = W;
                Height = H;
                 
                if(Width*Height==0)
                {
                      Colors = NULL; 
                       return;
                 }
 
                 Colors = new DWORD[Width*Height];
                  
                 for(DWORD y=0;y<Height;y++)
                       for(DWORD x=0;x<Width;x++)
                               Access(x,y) = BackColor;
         }
 

void RedScale (CBitmap& Result)
{
	         DWORD R;
         for(int y=0;y <Height;y++)
        {
            for(int x=0;x<(int)Width;x++)
            {
			DWORD Color = Access(x,y);
              R= GetRValue(Color) ;
              Result.Access(x,y) = RGB(R,0,0);   
            }
        }

}
};





int _tmain(int argc, _TCHAR* argv[])
{
CBitmap image,Result;

image.Load(" Menna.bmp ");
Result.Create ( image.GetWidth(), image.GetHeight(), RGB  (0, 0, 0 ) );
image.RedScale(Result); 
Result.Save ( "Menna_2.bmp" );

return 0;

}

