/* -*- Mode: C; indent-tabs-mode: s; c-basic-offset: 2; tab-width: 2 -*- */
/*
 * draw_imlib2.c
 * Copyright (C) Mike Crash 2009 <mike@mikecrash.com>
 * 
 * main.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * main.c is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <math.h>
#include "draw_imlib2.h"

#define BITSPERLONG 32
#define TOP2BITS(x) ((x & (3 << (BITSPERLONG-2))) >> (BITSPERLONG-2))

int logi2(int f)
{
  int l = log(f)/log(1.40);
  return l;
}

long isqrt(long x)
{
  long a = 0L;			/* accumulator */
  long r = 0L;      /* remainder */
  long e = 0L;      /* trial product */

  int i;

  for (i = 0; i < BITSPERLONG/2; i++)
  {
    r = (r << 2) + TOP2BITS(x);
    x <<= 2;
    a <<= 1;
    e = (a << 1) + 1;
    if (r >= e)
    {
      r -= e;
      a++;
    }
  }
  return a;
}

void draw_init()
{
  imlib_add_path_to_font_path("/usr/share/fonts/truetype/ttf-dejavu");
}

void draw_deinit()
{
}

drw_image draw_create(int width, int height)
{
  drw_image img = imlib_create_image(width, height);
  imlib_context_set_anti_alias(1);
  return img;
}

void draw_destroy(drw_image img)
{
  imlib_context_set_image(img);
  imlib_free_image();
}

void draw_setcolor(long color)
{
  int red = (color >> 16) & 0xff;
  int green = (color >> 8) & 0xff;
  int blue = color & 0xff;
  imlib_context_set_color(red, green, blue, 0xff);
}

drw_font draw_loadfont(char *fontname)
{
  return imlib_load_font(fontname);
}

void draw_freefont(drw_font font)
{
  imlib_context_set_font(font);
  imlib_free_font();
}

void draw_setfont(drw_font font)
{
  imlib_context_set_font(font);
}

void draw_line(drw_image img, int x1, int y1, int x2, int y2, int width)
{
  long x, y;
  long nx;
  long ny;

  imlib_context_set_image(img);

  if (width == 1)
  {
    imlib_image_draw_line(x1, y1, x2, y2, 1);
    return;
  }

  ImlibPolygon poly;

  int w = width/2-1;
  imlib_image_fill_ellipse(x1, y1, w, w);

  if ((x1==x2) && (y1==y2))
    return;

  imlib_image_fill_ellipse(x2, y2, w, w);

  if (x1==x2)
  {
    nx = width / 2;
    ny = 0;
  }
  else if (y1==y2)
  {
    nx = 0;
    ny = width/2;
  }
  else
  {
    long long x0 = (x2 - x1);
    x0 = x0 * x0;
    long long y0 = (y2 - y1);
    y0 = y0 * y0;
    long long w2 = width*width;
    long long tx = w2 * x0 / (x0+y0);
    tx = trunc(sqrt(tx)+0.7);
    long long ty = w2 * y0 / (x0+y0);
    ty = trunc(sqrt(ty)+0.7);

    if (x1>x2) tx = -tx;
    if (y1>y2) ty = -ty;

    nx = ty/2;
    ny = -tx/2;
  }

  //printf("w=%d, nx=%ld, ny=%ld, w=%f\n", width, nx, ny, sqrt(nx*nx+ny*ny));

  poly = imlib_polygon_new();
  x = x1 + nx;
  y = y1 + ny;
  imlib_polygon_add_point(poly, x, y);
  x = x1 - nx;
  y = y1 - ny;
  imlib_polygon_add_point(poly, x, y);
  x = x2 - nx;
  y = y2 - ny;
  imlib_polygon_add_point(poly, x, y);
  x = x2 + nx;
  y = y2 + ny;
  imlib_polygon_add_point(poly, x, y);
  imlib_image_fill_polygon(poly);
  imlib_polygon_free(poly);
}

void draw_rectangle(drw_image img, int x1, int y1, int x2, int y2, int fill)
{
  imlib_context_set_image(img);
  if (fill)
  {
    imlib_image_fill_rectangle(x1, y1, x2, y2);
  }
  else
  {
  }
}

void draw_polygon(drw_image img, drw_pos *polygon, int count, int fill)
{
  int i;
  ImlibPolygon poly;
  imlib_context_set_image(img);

  poly = imlib_polygon_new();
  for (i=0; i<count; i++)
  {
    imlib_polygon_add_point(poly, polygon->x, polygon->y);
    polygon++;
  }
  imlib_image_fill_polygon(poly);
  imlib_polygon_free(poly);
}


void draw_text_angle(drw_image img, double angle)
{
  imlib_context_set_image(img);
  imlib_context_set_direction(IMLIB_TEXT_TO_ANGLE);
  imlib_context_set_angle(angle);
}

void draw_text(drw_image img, int x, int y, int valign, int halign, double angle, char *text)
{
  int w, h;

  imlib_context_set_image(img);
  imlib_context_set_direction(IMLIB_TEXT_TO_ANGLE);
  imlib_context_set_angle(angle);
  imlib_get_text_size(text, &w, &h);

  if (valign == 0)
    y -= h/2;
  else if (valign == 1)
    y -= h;

  if (valign == 0)
    x -= w/2;
  else if (valign == 1)
    x -= w;

  imlib_text_draw(x, y, text);
}

int draw_text_width(drw_image img, char *text)
{
  int w, h;
  imlib_context_set_image(img);
  imlib_context_set_direction(IMLIB_TEXT_TO_RIGHT);
  imlib_get_text_size(text, &w, &h);
  //imlib_text_draw_with_return_metrics(-100000, -100000, text, &textwidth, &textheight, &texthor, &textvert);
  return w;
}

void draw_blend(drw_image img, drw_image blend, int x, int y, int w, int h)
{
  imlib_context_set_image(img);
  imlib_blend_image_onto_image(blend, 1, 0, 0, w, h, x, y, w, h);
}

drw_image draw_loadimage(char *path)
{
  return imlib_load_image(path);
}

void draw_getsize(drw_image img, int *width, int *height)
{
  imlib_context_set_image(img);
  *width = imlib_image_get_width();
  *height = imlib_image_get_height();
}

int *draw_getdata(drw_image img)
{
  imlib_context_set_image(img);
  return (int *)imlib_image_get_data();
}
