/* -*- Mode: C; indent-tabs-mode: s; c-basic-offset: 2; tab-width: 2 -*- */
/*
 * map_country.c
 * Copyright (C) Mike Crash 2009 <mike@mikecrash.com>
 * 
 * main.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * main.c is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _MCGPX_H
#define _MCGPX_H

#define GPXPOINTSLEN 100

typedef struct
{
  double time;
  double lat;
  double lon;
  double altitude;
  double speed;
  double course;
  double eph;
  double epv;
} gpx_point;

typedef struct gpx_points
{
  int count;
  int split;
  gpx_point points[GPXPOINTSLEN];
  struct gpx_points *next;
  struct gpx_points *prev;
} gpx_points;

typedef struct
{
  FILE *f;
  char *filename;
  int count;
  //last saved point
  double lat;
  double lon;
  double altitude;
  //course from last saved point
  int aja;
  double angle;
  double rise;
  struct gpx_points *first;
  struct gpx_points *last;
} gpx_track;

extern gpx_track *gpxtrack;
extern gpx_track *gpxlog;

int gpx_init();
int gpx_deinit();
int gpx_free(gpx_track *gpx);

int gpx_lock();
int gpx_unlock();

int gpx_load(char *filename);
int gpx_close();

int gpx_log(char *filename);
int gpx_logpoint(gpx_point *point);
int gpx_log_split();
int gpx_log_stop();

char *attr_str(const char **attr, const char *name);
int attr_long(const char **attr, const char *name, long *val);
int attr_double(const char **attr, const char *name, double *val);



#endif
