/* -*- Mode: C; indent-tabs-mode: s; c-basic-offset: 2; tab-width: 2 -*- */
/*
 * gui.c
 * Copyright (C) Mike Crash 2009 <mike@mikecrash.com>
 * 
 * main.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * main.c is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <Evas.h>
#include <Ecore.h>
#include <Ecore_Evas.h>
#include <Edje.h>
#include "mcconf.h"
#include "mctime.h"
#include "functions.h"
#include "projection.h"
#include "route.h"
#include "navigation.h"
#include "map_read.h"
#include "map_render.h"
#include "map_styles.h"
#include "gui.h"
#include "gui_progress.h"
#include "gui_address.h"
#include "gui_itinerary.h"
#include "gui_gps.h"
#include "gui_route.h"
#include "gui_track.h"
#include "gui_satellites.h"
#include "gui_fileopen.h"
#include "gui_text.h"

#define GUI_FPS 15

char GPSNAN[] = "- - -";

//The pointer to a canvas wrapper
Ecore_Evas *ecore_evas = NULL;
//The pointer to an Evas canvas
Evas *evas = NULL;
//The pointer to a given Edje object
Evas_Object *edje = NULL;
//Width and height for resizing Evas/Edje objects
Evas_Coord win_width=480, win_height=640;
//timer for drawing
Ecore_Timer *timer = NULL;
//nav arrow
Evas_Object *arrowobj = NULL;

//Image to draw the map
Evas_Object *mapobj = NULL;
int mapobj_mouse_x, mapobj_mouse_y;
map_position mapobj_pos;
int mapobj_zoom;
int mapobj_angle;
int mapobj_resized;

int mouse_isdown = 0;
int mouse_ismove = 0;
route_progress lastprogress;
long lastspeed = -1;
long lastalt = -1;
long lastangle = -1;
route_dir lastnextturn = dir_none;
long lasttoturn = 0;   //distance to next turn
long lasttogo = 0;     //distance to destination
long lasttotake = 0;   //seconds to destination
int lastnav = 1;

long chronoseconds = -1;
long chronolen = -1;
gui_mode guimode = mode_none;

//Pointers to Ecore handlers
Ecore_Event_Handler* appclose = NULL;


//**********************************************************************
// Set navigation arrow
//**********************************************************************

char arrows[cntroutedir][cntroutedir] =
{
  { "none" },
  { "straight" },
  { "r_uturn" },
  { "l_uturn" },
  { "hardleft" }, 
  { "left" },
  { "softleft" },
  { "softright" },
  { "right" },
  { "hardright" },
  { "rr_straight" },
  { "rr_uturn" },
  { "rr_hardleft" },
  { "rr_left" },
  { "rr_softleft" },
  { "rr_softright" },
  { "rr_right" },
  { "rr_hardright" },
  { "rl_straight" },
  { "rl_uturn" },
  { "rl_hardleft" },
  { "rl_left" },
  { "rl_softleft" },
  { "rl_softright" },
  { "rl_right" },
  { "rl_hardright" },
  { "direction" },
  { "waypoint" },
  { "finish" }
};

int nav_setarrow(route_dir dir)
{
  if (dir>=cntroutedir)
    return -1;
  char s[255];
  sprintf(s, "%s/themes/Default/arrow_%s.png", config->datadir, arrows[dir]);
  //printf("%s\n", s);
  evas_object_image_file_set(arrowobj, s, NULL);
  return 0;
}

//**********************************************************************
// Resize of controls
//**********************************************************************

void progress_resize(int x, int y, int w, int h)
{
  x += w * 3 / 20;
  y += w / 50;
  w = w * 14 / 20;
  h = w * 10 / 50;
  gui_prg_resize(x, y, w, h);
}

void menu_gps_resize(int x, int y, int w, int h)
{
  x += w / 50;
  y = y + h - w * 24 / 50 - w / 50;
  w = w * 48/50;
  h = w * 24/50;
  gui_gps_resize(x, y, w, h);
}

void menu_route_resize(int x, int y, int w, int h)
{
  x += w / 50;
  y = y + h - w * 24 / 50 - w / 50;
  w = w * 48/50;
  h = w * 24/50;
  gui_route_resize(x, y, w, h);
}

void menu_track_resize(int x, int y, int w, int h)
{
  x += w / 50;
  y = y + h - w * 24 / 50 - w / 50;
  w = w * 48/50;
  h = w * 24/50;
  gui_track_resize(x, y, w, h);
}

//**********************************************************************
// Handlers
//**********************************************************************

void win_resize(Ecore_Evas *ee)
{
  int x, y, w, h;
  int minw, minh;
  int maxw, maxh;
  
  //printf("Resize\n");
  if (ee) 
  {
    ecore_evas_geometry_get(ee, NULL, NULL, &w, &h);
    ecore_evas_size_min_get(ee, &minw, &minh);
    ecore_evas_size_max_get(ee, &maxw, &maxh);
    if ((w >= minw) && (h >= minh) && (h <= maxh) && (w <= maxw))
    {
      //Evas_Object *o = NULL;
      //if ((o = evas_object_name_find(ecore_evas_get(ecore_evas), "background")))
      win_width = w;
      win_height = h;
      gui_addr_resize(w, h);
      gui_itinerary_resize(w, h);
      gui_sat_resize(w, h);
      gui_fileopen_resize(w, h);
      gui_text_resize(w, h);
      evas_object_resize(edje, w, h);
      edje_object_part_geometry_get(edje, "map", &x, &y ,&w, &h);
      evas_object_image_data_set(mapobj, NULL);
      nav_setsize(w, h);
      progress_resize(x, y, w, h);
      menu_gps_resize(x, y, w, h);
      menu_route_resize(x, y, w, h);
      menu_track_resize(x, y, w, h);
      mapobj_resized = 1;
    }
  }
}

void map_mousedown(void *data, Evas_Object *obj, const char *emission, const char *source)
{
  //printf("Down!!!\n");
  Evas_Coord mouse_x, mouse_y;
  evas_pointer_output_xy_get(evas, &mouse_x, &mouse_y);
  mapobj_mouse_x = mouse_x;
  mapobj_mouse_y = mouse_y;
  nav_tracking(0);
  nav_getpos(&mapobj_pos, &mapobj_angle);
  mapobj_zoom = nav_getzoom();
  mouse_isdown = 1;
  //printf("Down %d,%d -> %ld, %ld  %d\n", mouse_x, mouse_y, mapobj_pos.lat, mapobj_pos.lon, mapobj_zoom);
}

void map_mouseup(void *data, Evas_Object *obj, const char *emission, const char *source)
{
  //printf("Up!!!\n");
  nav_tracking(1);
  if ((guimode!=mode_none) && (mouse_ismove==0))
  {
    map_position pos;
    int x,y;
    int angle;
    nav_getpos(&pos, &angle);
    nav_getanchor(&x, &y);
    x = mapobj_mouse_x - x;
    y = y - mapobj_mouse_y;
    long size = 1 << mapobj_zoom;
    wgs84_screen2pos(x*size, y*size, pos.lat, pos.lon, mapobj_angle, &pos.lat, &pos.lon);
    switch (guimode)
    {
      case mode_destination:
        nav_route(&pos, routeadd_destination, NULL);
        break;
      case mode_waypoint:
        nav_route(&pos, routeadd_waypoint, NULL);
        break;
      case mode_direction:
        nav_route(&pos, routeadd_direction, NULL);
        break;
      default:
        break;
    }
  }
  mouse_isdown = 0;
  mouse_ismove = 0;
}

void map_mousemove(void *data, Evas_Object *obj, const char *emission, const char *source)
{
  Evas_Coord mouse_x, mouse_y;

  if (mouse_isdown==1)
  {
    map_position pos;
    long size;
    evas_pointer_canvas_xy_get(evas, &mouse_x, &mouse_y);
    mouse_x = mapobj_mouse_x - mouse_x;
    mouse_y = mouse_y - mapobj_mouse_y;
    if ((abs(mouse_x)<10)&&(abs(mouse_y)<10))
    {
      mouse_ismove = 1;
    }
    if (mouse_ismove)
    {
      size = 1 << mapobj_zoom;
      wgs84_screen2pos(mouse_x*size, mouse_y*size, mapobj_pos.lat, mapobj_pos.lon, mapobj_angle, &pos.lat, &pos.lon);
      nav_setpos(&pos, mapobj_angle);
    }
    //printf("Move %d,%d -> %ld, %ld -> %ld, %ld\n", mouse_x, mouse_y, mapobj_pos.lat, mapobj_pos.lon, pos.lat, pos.lon);
  }
}

//**********************************************************************
// Menu
//**********************************************************************

void menu_zoom(void *data, Evas_Object *obj, const char *emission, const char *source)
{
  int zoom = nav_getzoom();
  if (strcmp(emission,"out")==0)
  {
    if (zoom>=mapzooms-1)
      return;
    if (zoom==20)
      return;
    zoom ++;
    nav_setzoom(zoom);
  }
  else if (strcmp(emission,"in")==0)
  {
    if (zoom==5)
      return;
    zoom --;
    nav_setzoom(zoom);
  }
  printf("Zoom %d\n", zoom);
}

void menu_gps(void *data, Evas_Object *obj, const char *emission, const char *source)
{
  int x, y, w, h;
  gui_route_hide();
  gui_track_hide();
  edje_object_part_geometry_get(edje, "map", &x, &y ,&w, &h);
  x += w / 50;
  y = y + h - w * 24 / 50 - w / 50;
  w = w * 48/50;
  h = w * 24/50;
  gui_gps_show(evas, x, y, w, h);
}

void menu_route(void *data, Evas_Object *obj, const char *emission, const char *source)
{
  if (nav->route)
  {
    route_stop();
  }
  else if (guimode != mode_none)
  {
    gui_setmode(mode_none);
  }
  else
  {
    int x, y, w, h;
    gui_track_hide();
    gui_gps_hide();
    edje_object_part_geometry_get(edje, "map", &x, &y ,&w, &h);
    x += w / 50;
    y = y + h - w * 24 / 50 - w / 50;
    w = w * 48/50;
    h = w * 24/50;
    gui_route_show(evas, x, y, w, h);
  }
}

void menu_track(void *data, Evas_Object *obj, const char *emission, const char *source)
{
  int x, y, w, h;
  gui_route_hide();
  gui_gps_hide();
  edje_object_part_geometry_get(edje, "map", &x, &y ,&w, &h);
  x += w / 50;
  y = y + h - w * 24 / 50 - w / 50;
  w = w * 48/50;
  h = w * 24/50;
  gui_track_show(evas, x, y, w, h);
}

//**********************************************************************
// Timer
//**********************************************************************

Eina_Bool gui_timer(void *data)
//int gui_timer(void *data)
{
  //map draw
  int w, h;
  char s[100];

  void *p = (void *)mr_getdata(&nav->map_img, &w, &h);
  if (p)
  {
    if (mapobj_resized)
    {
      evas_object_image_size_set(mapobj, w, h);
      mapobj_resized = 0;
    }
    evas_object_image_data_set(mapobj, p);
    evas_object_image_pixels_dirty_set(mapobj, 1);
  }

  //GPS info
  long lat, lon;
  int fix, speed, alt, angle, eph, epv;
  if (nav_gpsinfo(&fix, &lat, &lon, &speed, &alt, &angle, &eph, &epv) == 0)
  {
    if ((speed != lastspeed) && (speed>=0))
    {
      sprintf(s, "%d", speed);
      edje_object_part_text_set(edje, "nav_speed", s);
      lastspeed = speed;
    }
    if ((alt != lastalt) && (alt>=0) && (alt<=10000))
    {
      nice_dist(s, alt, 1);
      edje_object_part_text_set(edje, "nav_alt", s);
      lastalt = alt;
    }
    if ((angle != lastangle) && (angle>=-180) && (angle<=180))
    {
      sprintf(s, "%d°", angle);
      edje_object_part_text_set(edje, "nav_angle", s);
      lastangle = angle;
    }
  }
  else
  {
    lastspeed = -1;
    lastalt = -1;
    lastangle = 1000;
    edje_object_part_text_set(edje, "nav_speed", "- - -");
    edje_object_part_text_set(edje, "nav_alt", "- - -");
    edje_object_part_text_set(edje, "nav_angle", "- - -");
  }

  //route info
  route_dir nextturn;
  long toturn, togo, totake;
  if (nav_routeinfo(&nextturn, &toturn, &togo, &totake) == 0)
  {
    if (nextturn != lastnextturn)
    {
      nav_setarrow(nextturn);
      lastnextturn = nextturn;
    }
    if (toturn != lasttoturn)
    {
      nice_dist(s, toturn, 10);
      edje_object_part_text_set(edje, "nav_nextdist", s);
      lasttoturn = toturn;
    }
    if ((togo != lasttogo) && (config->chrono_ena==0))
    {
      nice_dist(s, togo, 10);
      edje_object_part_text_set(edje, "nav_dist", s);
      lasttogo = togo;
    }
    if (totake != lasttotake)
    {
      long hour = totake / 36000;
      long min = (totake  % 36000 + 30) / 600;
      if (config->chrono_ena==0)
      {
        sprintf(s, "%ld:%02ld", hour, min);
        edje_object_part_text_set(edje, "nav_disttime", s);
      }
      time_t t = time(NULL) + totake / 10;
      struct tm *timeinfo = localtime(&t);
      strftime(s, sizeof(s), "%H:%M", timeinfo);
      edje_object_part_text_set(edje, "nav_resttime", s);
      lasttotake = totake;
    }
    lastnav = 1;
  }
  else if (lastnav)
  {
    nav_setarrow(dir_none);
    edje_object_part_text_set(edje, "nav_nextdist", "");
    edje_object_part_text_set(edje, "nav_dist", GPSNAN);
    edje_object_part_text_set(edje, "nav_disttime", GPSNAN);
    time_t t = time(NULL);
    struct tm *timeinfo = localtime(&t);
    strftime(s, sizeof(s), "%H:%M", timeinfo);
    edje_object_part_text_set(edje, "nav_resttime", s);
    lastnav = 0;
    lasttotake = 0;
    lasttogo = 0;
  }
  else
  {
    time_t t = time(NULL);
    struct tm *timeinfo = localtime(&t);
    strftime(s, sizeof(s), "%H:%M", timeinfo);
    edje_object_part_text_set(edje, "nav_resttime", s);
  }

  //chrono
  if (config->chrono_ena)
  {
    char s[30];
    long mtime = timer_get(&config->chrono_start) / 1000;
    if (mtime != chronoseconds)
    {
      chronoseconds = mtime;
      long hours = mtime / 3600;
      mtime -= hours * 3600;
      long minutes = mtime / 60;
      mtime -= minutes * 60;
      long seconds = mtime;
      if (hours>0)
        sprintf(s, "%ld:%02ld:%02ld", hours, minutes, seconds);
      else
        sprintf(s, "%02ld:%02ld", minutes, seconds);
      edje_object_part_text_set(edje, "nav_disttime", s);
    }
    if (config->chrono_len != chronolen)
    {
      chronolen = config->chrono_len;
      nice_dist(s, config->chrono_len, 1);
      edje_object_part_text_set(edje, "nav_dist", s);
    }
  }
  else
  {
    chronoseconds = -1;
    chronolen = -1;
    //edje_object_part_text_set(edje, "nav_disttime", GPSNAN);
    //edje_object_part_text_set(edje, "nav_dist", GPSNAN);
  }

  //route progress
  route_progress progress;
  if (nav_progress(&progress) == 0)
  {
    if (progress.progress>=0)
    {
      if (lastprogress.progress<0)
      {
        int x, y, w, h;
        edje_object_part_geometry_get(edje, "map", &x, &y ,&w, &h);
        x += w * 3 / 20;
        y += w / 50;
        w = w * 14 / 20;
        h = w * 10 / 50;
        gui_prg_show(evas, x, y, w, h);
      }
      if ((progress.progress!=lastprogress.progress) || (progress.nodes!=lastprogress.nodes)||(progress.length!=lastprogress.length))
      {
        snprintf(s, sizeof(s), "Routing - %d nodes, %ld km", progress.nodes, progress.length/1000);
        gui_prg_set(progress.progress, s);
        //printf("Progress: %d\n", progress.progress);
      }
    }
    else
    {
      gui_prg_hide();
    }
    lastprogress = progress;
  }
  return 1;
}

Eina_Bool gui_close_handler(void *data, int type, void *event)
//int gui_close_handler(void *data, int type, void *event)
{
  return gui_close();
}

//**********************************************************************
// GUI Init
//**********************************************************************

//Handles cleaning up and closing the application
int gui_close()
{
  ecore_event_handler_del(appclose);
  ecore_timer_del(timer);
  evas_object_image_data_set(mapobj, NULL);
  nav_deinit();
  gui_addr_free();
  gui_itinerary_free();
  gui_prg_free();
  gui_gps_free();
  gui_route_free();
  gui_track_free();
  gui_sat_free();
  gui_fileopen_free();
  gui_text_free();
  ecore_main_loop_quit();
  return 0;
}

//gui init
int gui_init(int argc, char **argv)
{
  //Init components
  if (!ecore_init()) 
    return EXIT_FAILURE;
  if (!ecore_evas_init()) 
    return EXIT_FAILURE;
  if (!edje_init()) 
    return EXIT_FAILURE;

  //Check the canvas wrapper is created correctly
  ecore_evas = ecore_evas_software_x11_new(NULL, 0, 0, 0, 480, 640);
  if (!ecore_evas) 
    return EXIT_FAILURE;

  //We set some window attributes and make the wrapper visible
  ecore_evas_title_set(ecore_evas, "MC Navi");
  ecore_evas_name_class_set(ecore_evas, "mcnavi", "MCNavi");
  ecore_evas_show(ecore_evas);

  //Get the pointer to the canvas and add edje object
  evas = ecore_evas_get(ecore_evas);
  edje = edje_object_add(evas);

  //load the edje object
  char *edjefile = (char*)malloc(strlen(config->datadir) + 30);
  sprintf(edjefile, "%s/themes/Default/main.edj", config->datadir);
  edje_object_file_set(edje, edjefile, "main");
  free(edjefile);

  //Setting the object and canvas to the minimal size of the object
  evas_object_move(edje, 0, 0);
  //edje_object_size_min_get(edje, &win_width, &win_height);
  evas_object_resize(edje, win_width, win_height);
  ecore_evas_resize(ecore_evas, win_width, win_height);
  evas_object_show(edje);

  appclose = ecore_event_handler_add(ECORE_EVENT_SIGNAL_EXIT, gui_close_handler, NULL);
  ecore_evas_callback_resize_set(ecore_evas, win_resize);

  //get the map object
  int x, y, w, h;
  edje_object_part_geometry_get(edje, "map", &x, &y ,&w, &h);
  mapobj = (Evas_Object *)edje_object_part_object_get(edje, "map");
  arrowobj = (Evas_Object *)edje_object_part_object_get(edje, "nav_arrow");

  edje_object_signal_callback_add(edje, "map_mousedown", "map", map_mousedown, NULL);
  edje_object_signal_callback_add(edje, "map_mouseup", "map", map_mouseup, NULL);
  edje_object_signal_callback_add(edje, "map_mousemove", "map", map_mousemove, NULL);
  edje_object_signal_callback_add(edje, "menu_gps", "menu", menu_gps, NULL);
  edje_object_signal_callback_add(edje, "menu_route", "menu", menu_route, NULL);
  edje_object_signal_callback_add(edje, "menu_track", "menu", menu_track, NULL);
  edje_object_signal_callback_add(edje, "*", "zoom", menu_zoom, NULL);

  //init navigation
  nav_init();
  nav_setsize(w, h);
  mapobj_resized = 1;

  //create drawing timer  
  timer = ecore_timer_add(1.0/GUI_FPS, gui_timer, NULL);

  return 0;
}

//gui start
int gui_start()
{
  //Starting the main application loop
  ecore_main_loop_begin();

  //Once the window is closed (ending the loop) we clean up our mess
  //edje_shutdown();  //this will crash, is it needed?
  ecore_evas_shutdown();
  ecore_shutdown();
  return 0;
}

int gui_setmode(gui_mode mode)
{
  guimode = mode;
  switch (mode)
  {
    case mode_none:
      edje_object_signal_emit(edje, "mode,state,none", "mcnavi");
      break;
    case mode_destination:
      edje_object_signal_emit(edje, "mode,state,destination", "mcnavi");
      break;
    case mode_waypoint:
      edje_object_signal_emit(edje, "mode,state,waypoint", "mcnavi");
      break;
    case mode_direction:
      edje_object_signal_emit(edje, "mode,state,direction", "mcnavi");
      break;
  }
  return 0;
}
