/* -*- Mode: C; indent-tabs-mode: s; c-basic-offset: 2; tab-width: 2 -*- */
/*
 * address.c
 * Copyright (C) Mike Crash 2009 <mike@mikecrash.com>
 *
 * main.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * main.c is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdlib.h>
#include <Evas.h>
#include <Ecore.h>
#include <Ecore_Evas.h>
#include <Edje.h>
#include <string.h>
#include "mcconf.h"
#include "gui_address.h"
#include "gui_route.h"
#include "map_address.h"
#include "map_country.h"
#include "map_city.h"
#include "map_street.h"
#include "map_house.h"
#include "route.h"
#include "diacritics.h"
#include "functions.h"
#include "navigation.h"

#define listnum 8
#define maxnamelen 50

#define AD_BOOKMARKS 1
#define AD_COUNTRY 2
#define AD_CITY 3
#define AD_STREET 4
#define AD_ADDRESS 5

Evas_Object *edje_addr = NULL;
void gui_addr_cont(char *label);

typedef struct
{
  long lat;
  long lon;
  char name[];
} tbookmark;

//entering data
char *mas_label = NULL;
int mas_adtype;
char *mas_name;
int mas_len;
void *mas_list[listnum];
long mas_filepos[maxnamelen];  //filepos of previous searches

//search data
tcountry *mas_country = NULL;  //selected country
tcity *mas_city = NULL;        //selected city
tstreet *mas_street = NULL;    //selected street
thouse *mas_address = NULL;    //selected house address

//result
map_position mas_pos;

//routing
guiroute gui_route;

void gui_route_next(map_position *pos, char *name)
{
  switch (gui_route.type)
  {
    case GRT_MOVE:
    {
      gui_addr_hide();
      nav_setpos(pos, 0);
      break;
    }
    case GRT_ROUTE:
    {
      gui_addr_hide();
      if (config->route_add == routeadd_destination)
      {
        if (nav_route(pos, routeadd_destination, name) != 0)
        {
          free(name);
        }
      }
      else if (config->route_add == routeadd_waypoint)
      {
        if (nav_route(pos, routeadd_waypoint, name) != 0)
        {
          free(name);
        }
      }
      else if (config->route_add == routeadd_direction)
      {
        if (nav_route(pos, routeadd_direction, name) != 0)
        {
          free(name);
        }
      }
      break;
    }
  }
}

void free_list(int dont)
{
  int i;
  for (i=0; i<listnum; i++)
  {
    if (mas_list[i])
    {
      if (dont != i)
      {
        free(mas_list[i]);
      }
      mas_list[i] = NULL;
    }
  }
}

void free_mas()
{
  if (mas_city)
  {
    free(mas_city);
    mas_city = NULL;
  }
  if (mas_country)
  {
    free(mas_country);
    mas_country = NULL;
  }
  if (mas_street)
  {
    free(mas_street);
    mas_street = NULL;
  }
  if (mas_address)
  {
    free(mas_address);
    mas_address = NULL;
  }
}

void free_all()
{
  free_list(-1);
  if (mas_country)
  {
    free(mas_country);
    mas_country = NULL;
  }
  if (mas_city)
  {
    free(mas_city);
    mas_city = NULL;
  }
}

void update_bookmarks()
{
  char navname[50];
  char s[100];
  int i = 0;

  if (mas_label)
    sprintf(navname, "%s bookmark: %s", mas_label, mas_name);
  else
    sprintf(navname, "Bookmark: %s", mas_name);
  edje_object_part_text_set(edje_addr, "navname", navname);

  char *homedir=getenv("HOME");
  snprintf(s, sizeof(s), "%s/.mcnavi/bookmarks", homedir);
  FILE *f = fopen(s, "r");
  if (f)
  {
    while (i<listnum)
    {
      //read line
      if (fgets(s, sizeof(s), f) == NULL)
        break;
      char *lat = strchr(s, '=');
      if (lat==NULL)
        continue;
      *lat = 0;
      lat++;
      char *lon = strchr(lat, ',');
      if (lon==NULL)
        continue;
      *lon = 0;
      lon++;

      //check name
      if (mas_name[0])
      {
        char *cname = (char*)malloc(strlen(s));
        utf8_to_ascii(s, cname);
        //printf("cname=%s masname=%s\n", cname, mas_name);
        if (strcasepos(cname, mas_name) != 0)
        {
          free(cname);
          continue;
        }
        free(cname);
      }

      //add to list
      tbookmark *tb = (tbookmark*)malloc(sizeof(tbookmark) + strlen(s) + 1);
      tb->lat = atol(lat);
      tb->lon = atol(lon);
      strcpy(tb->name, s);
      mas_list[i] = tb;
      //printf("%s %ld,%ld\n",tb->name, tb->lat, tb->lon);
      sprintf(navname, "navname%d", i+1);
      edje_object_part_text_set(edje_addr, navname, tb->name);
      i++;
    }
    fclose(f);
  }

  while (i<listnum)
  {
    sprintf(navname, "navname%d", i+1);
    edje_object_part_text_set(edje_addr, navname, "");
    i++;
  }
}

void update_country()
{
  mapaddrsearch *as;
  tcountry *tc;
  char navname[50];
  int i = 0;

  if (mas_label)
    sprintf(navname, "%s country: %s", mas_label, mas_name);
  else
    sprintf(navname, "Country: %s", mas_name);
  edje_object_part_text_set(edje_addr, "navname", navname);

  if (mas_name[0] == 0)
  {
    edje_object_part_text_set(edje_addr, "navname1", "<Bookmarks>");
    i++;
    mas_list[0] = NULL;
  }

  as = map_country_start(mas_name);
  while ((i<listnum) && (tc = map_country_get(as)))
  {
    mas_list[i] = tc;
    //printf("%s\n",tc->name);
    sprintf(navname, "navname%d", i+1);
    edje_object_part_text_set(edje_addr, navname, tc->name);
    i++;
  }
  while (i<listnum)
  {
    sprintf(navname, "navname%d", i+1);
    edje_object_part_text_set(edje_addr, navname, "");
    i++;
  }
  map_country_stop(as);
}

void update_city()
{
  mapaddrsearch *as;
  tcity *tc;
  char navname[50];
  int i = 0;
  long filepos;

  if (mas_label)
    sprintf(navname, "%s city: %s", mas_label, mas_name);
  else
    sprintf(navname, "City: %s", mas_name);
  edje_object_part_text_set(edje_addr, "navname", navname);

  filepos = mas_len==0 ? mas_country->citiesfilepos : mas_filepos[mas_len-1];
  as = map_city_start(mas_name, mas_country->id, filepos);
  while ((i<listnum) && (tc = map_city_get(as)))
  {
    mas_list[i] = tc;
    if (i==0)
      mas_filepos[mas_len] = tc->filepos;
    //printf("T: %s %ld %ld,%ld\n",tc->name, tc->filepos, tc->lat, tc->lon);
    sprintf(navname, "navname%d", i+1);
    edje_object_part_text_set(edje_addr, navname, tc->name);
    i++;
  }
  while (i<listnum)
  {
    sprintf(navname, "navname%d", i+1);
    edje_object_part_text_set(edje_addr, navname, "");
    i++;
  }
  map_city_stop(as);
}

void update_street()
{
  mapaddrsearch *as;
  tstreet *ts;
  char navname[50];
  int i = 0;
  int first = 1;
  long filepos;

  if (mas_label)
    sprintf(navname, "%s street: %s", mas_label, mas_name);
  else
    sprintf(navname, "Street: %s", mas_name);
  edje_object_part_text_set(edje_addr, "navname", navname);

  if (mas_name[0] == 0)
  {
    snprintf(navname, sizeof(navname), "(Center of %s)", mas_city->name);
    edje_object_part_text_set(edje_addr, "navname1", navname);
    i++;
    mas_list[0] = NULL;
  }

  filepos = mas_len==0 ? mas_city->streetsfilepos : mas_filepos[mas_len-1];
  if (filepos!=0xffffffff)
  {
    as = map_street_start(mas_name, mas_city->id, filepos);
    while ((i<listnum) && (ts = map_street_get(as)))
    {
      mas_list[i] = ts;
      if (first)
      {
        mas_filepos[mas_len] = ts->filepos;
        first = 0;
      }
      //printf("T: %s %ld %ld,%ld\n",tc->name, tc->filepos, tc->lat, tc->lon);
      sprintf(navname, "navname%d", i+1);
      edje_object_part_text_set(edje_addr, navname, ts->name);
      i++;
    }
    map_street_stop(as);
  }
  while (i<listnum)
  {
    sprintf(navname, "navname%d", i+1);
    edje_object_part_text_set(edje_addr, navname, "");
    i++;
  }
}

void update_address()
{
  mapaddrsearch *as;
  thouse *ts;
  char navname[50];
  int i = 0;
  int first = 1;
  long filepos;

  if (mas_label)
    sprintf(navname, "%s address: %s", mas_label, mas_name);
  else
    sprintf(navname, "Address: %s", mas_name);
  edje_object_part_text_set(edje_addr, "navname", navname);

  if (mas_name[0] == 0)
  {
    snprintf(navname, sizeof(navname), "(Street %s)", mas_street->name);
    edje_object_part_text_set(edje_addr, "navname1", navname);
    i++;
    mas_list[0] = NULL;
  }

  filepos = mas_len==0 ? mas_street->addressesfilepos : mas_filepos[mas_len-1];
  if (filepos!=0xffffffff)
  {
    as = map_house_start(mas_name, mas_street->id, filepos);
    while ((i<listnum) && (ts = map_house_get(as)))
    {
      mas_list[i] = ts;
      if (first)
      {
        mas_filepos[mas_len] = ts->filepos;
        first = 0;
      }
      //printf("T: %s %ld %ld,%ld\n",tc->name, tc->filepos, tc->lat, tc->lon);
      sprintf(navname, "navname%d", i+1);
      edje_object_part_text_set(edje_addr, navname, ts->name);
      i++;
    }
    map_house_stop(as);
  }
  while (i<listnum)
  {
    sprintf(navname, "navname%d", i+1);
    edje_object_part_text_set(edje_addr, navname, "");
    i++;
  }
}

void update_addr()
{
  switch (mas_adtype)
  {
    case AD_BOOKMARKS:
      update_bookmarks();
      break;
    case AD_COUNTRY:
      update_country();
      break;
    case AD_CITY:
      update_city();
      break;
    case AD_STREET:
      update_street();
      break;
    case AD_ADDRESS:
      update_address();
      break;
  }
}

void butpress(void *data, Evas_Object *obj, const char *emission, const char *source)
{
  //printf("%s\n%s\n", emission, source);
  if (strcmp(emission, "esc") == 0)
  {
    free_list(-1);
    gui_addr_hide();
  }
  else if (strcmp(emission, "back") == 0)
  {
    if (mas_len>0)
    {
      mas_len--;
      mas_name[mas_len] = 0;
      free_list(-1);
      update_addr();
    }
    else if (mas_adtype==AD_BOOKMARKS)
    {
      mas_adtype = AD_COUNTRY;
      mas_len = 0;
      mas_name[mas_len] = 0;
      free_list(-1);
      update_country();
    }
    else if (mas_adtype==AD_ADDRESS)
    {
      mas_adtype = AD_STREET;
      mas_len = 0;
      mas_name[mas_len] = 0;
      free_list(-1);
      update_addr();
    }
    else if (mas_adtype==AD_STREET)
    {
      mas_adtype = AD_CITY;
      mas_len = 0;
      mas_name[mas_len] = 0;
      free_list(-1);
      update_addr();
    }
    else if (mas_adtype==AD_CITY)
    {
      mas_adtype = AD_COUNTRY;
      mas_len = 0;
      mas_name[mas_len] = 0;
      free_list(-1);
      update_addr();
    }
  }
  else if (strlen(emission) == 1)
  {
    if (mas_list[0] || (mas_len==0))
    {
      mas_name[mas_len] = emission[0];
      mas_len++;
      mas_name[mas_len] = 0;
      free_list(-1);
      update_addr();
    }
  }
}

void navpress(void *data, Evas_Object *obj, const char *emission, const char *source)
{
  unsigned int i = atoi(emission) - 1;
  if (i>=listnum)
    return;
  //printf("%s %s %d\n", emission, source, i);
  switch (mas_adtype)
  {
    case AD_BOOKMARKS:
      if (mas_list[i])
      {
        tbookmark *tb = (tbookmark *)mas_list[i];
        char *name = (char*)malloc(strlen(tb->name)+1);
        strcpy(name, tb->name);
        mas_pos.lat = tb->lat;
        mas_pos.lon = tb->lon;
        free_mas();
        gui_route_next(&mas_pos, name);
      }
      break;
    case AD_COUNTRY:
      if (i==0)
      {
        mas_adtype = AD_BOOKMARKS;
        free_list(-1);
        update_bookmarks();
        break;
      }
      mas_country = (tcountry*)mas_list[i];
      free_list(i);
      mas_adtype = AD_CITY;
      mas_len = 0;
      mas_name[0] = 0;
      update_city();
      break;
    case AD_CITY:
      mas_city = (tcity*)mas_list[i];
      printf("City %s on %ld,%ld\n", mas_city->name, mas_city->pos.lat, mas_city->pos.lon);
      free_list(i);
      mas_adtype = AD_STREET;
      mas_len = 0;
      mas_name[0] = 0;
      update_street();
      break;
    case AD_STREET:
      if ((mas_name[0] == 0) && (i==0))
      {
        if ((mas_city->pos.lat!=0) || (mas_city->pos.lon!=0))
        {
          mas_pos = mas_city->pos;
          char *name = (char*)malloc(strlen(mas_city->name) + 1);
          sprintf(name, "%s", mas_city->name);
          free_mas();
          gui_route_next(&mas_pos, name);
        }
        break;
      }
      mas_street = (tstreet*)mas_list[i];
      mas_pos = mas_street->pos;
      printf("Street %s on %ld,%ld\n", mas_street->name, mas_street->pos.lat, mas_street->pos.lon);
      free_list(i);
      mas_adtype = AD_ADDRESS;
      mas_len = 0;
      mas_name[0] = 0;
      update_address();
      break;
    case AD_ADDRESS:
      if ((mas_name[0] == 0) && (i==0))
      {
        if ((mas_street->pos.lat!=0) || (mas_street->pos.lon!=0))
        {
          mas_pos = mas_street->pos;
          char *name = (char*)malloc(strlen(mas_city->name) + strlen(mas_street->name) + 3);
          sprintf(name, "%s, %s", mas_city->name, mas_street->name);
          free_mas();
          gui_route_next(&mas_pos, name);
        }
        break;
      }
      mas_address = (thouse*)mas_list[i];
      if ((mas_address->pos.lat==0) && (mas_address->pos.lon==0))
      {
        mas_address = NULL;
        break;
      }
      mas_pos = mas_address->pos;
      char *name = (char*)malloc(strlen(mas_city->name) + strlen(mas_street->name) + strlen(mas_address->name) + 4);
      sprintf(name, "%s, %s %s", mas_city->name, mas_street->name, mas_address->name);
      printf("Address %s on %ld,%ld\n", mas_address->name, mas_address->pos.lat, mas_address->pos.lon);
      free_list(i);
      mas_len = 0;
      mas_name[0] = 0;
      free_mas();
      gui_route_next(&mas_pos, name);
      break;
  }
}

void gui_addr_cont(const char *label)
{
  if (mas_label)
  {
    free(mas_label);
    mas_label = NULL;
  }
  if (label)
  {
    int len = strlen(label) + 1;
    mas_label = (char*)malloc(len);
    strcpy(mas_label, label);
  }
  if (mas_name == NULL)
  {
    mas_name = (char*)malloc(maxnamelen);
  }
  evas_object_show(edje_addr);
  mas_adtype = AD_COUNTRY;
  mas_name[0] = 0;
  mas_len = 0;
  update_country();
}

void gui_addr_show(Evas *evas, guiroutetype rt, const char *label, int width, int height)
{
  if (edje_addr == NULL)
  {
    edje_addr = edje_object_add(evas);
    char *edjefile = (char*)malloc(strlen(config->datadir) + 30);
    sprintf(edjefile, "%s/themes/Default/address.edj", config->datadir);
    edje_object_file_set(edje_addr, edjefile, "menu_address");
    free(edjefile);
    evas_object_move(edje_addr, 0, 0);
    evas_object_resize(edje_addr, width, height);
    edje_object_signal_callback_add(edje_addr, "*", "but", butpress, NULL);
    edje_object_signal_callback_add(edje_addr, "*", "navname", navpress, NULL);
  }
  gui_route.type = rt;
  gui_route.step = 0;
  gui_addr_cont(label);
}

void gui_addr_resize(int width, int height)
{
  if (edje_addr)
  {
    evas_object_resize(edje_addr, width, height);
  }
}

void gui_addr_hide(void)
{
  evas_object_hide(edje_addr);
}

void gui_addr_free(void)
{
  if (edje_addr != NULL)
  {
    evas_object_del(edje_addr);
    edje_addr = NULL;
  }
  if (mas_name != NULL)
  {
    free(mas_name);
    mas_name = NULL;
  }
  free_list(-1);
}
