/* -*- Mode: C; indent-tabs-mode: s; c-basic-offset: 2; tab-width: 2 -*- */
/*
 * gui.c
 * Copyright (C) Mike Crash 2009 <mike@mikecrash.com>
 * 
 * main.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * main.c is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <Elementary.h>

Evas_Object *lb_street;
Evas_Object *mapimg;
Evas_Object *lb_distance;
Evas_Object *lb_totdistance;
Evas_Object *lb_time;
Evas_Object *lb_timedistance;
Evas_Object *dirimg;

static void win_del(void *data, Evas_Object *obj, void *event_info)
{
  elm_exit();
}

int gui_init(int argc, char **argv)
{
  elm_init(argc, argv);
  return 0;
}

int gui_start()
{
	// main window
  Evas_Object *win = elm_win_add(NULL, "mcnavi", ELM_WIN_BASIC);
  elm_win_title_set(win, "mcnavi");
  evas_object_smart_callback_add(win, "delete-request", win_del, NULL);

  // background
  Evas_Object *bg = elm_bg_add(win);
  evas_object_size_hint_weight_set(bg, 1.0, 1.0);
  elm_win_resize_object_add(win, bg);
  evas_object_show(bg);

  // main table
  Evas_Object *tb = elm_table_add(win);
  elm_win_resize_object_add(win, tb);
	//elm_table_homogenous_set(tb, 1);
  evas_object_size_hint_weight_set(tb, 1.0, 1.0);
  evas_object_show(tb);
  
	// street label
	lb_street = elm_label_add(win);
	elm_label_label_set(lb_street, "Street");
	evas_object_size_hint_weight_set(lb_street, 1, 0);
	evas_object_size_hint_align_set(lb_street, 0.5, 0.5);
  elm_table_pack(tb, lb_street, 0, 0, 1, 1);
  evas_object_show(lb_street);

  // map
  mapimg = elm_image_add(win);
	evas_object_size_hint_weight_set(mapimg, 1, 1);
	evas_object_size_hint_align_set(mapimg, -1.0, -1.0);
	elm_table_pack(tb, mapimg, 0, 1, 1, 1);
  evas_object_show(mapimg);
  
  // frame for display
  Evas_Object *fd = elm_frame_add(win);
  elm_frame_style_set(fd, "pad_small");
//evas_object_size_hint_weight_set(fd, 1.0, 1.0);
	evas_object_size_hint_align_set(fd, -1.0, -1.0);
	elm_table_pack(tb, fd, 0, 2, 1, 1);
  evas_object_show(fd);

  // display table
  Evas_Object *tb2 = elm_table_add(win);
	//elm_table_homogenous_set(tb2, 1);
  evas_object_size_hint_weight_set(tb2, 1, 1);
	evas_object_size_hint_align_set(tb2, -1, -1);
  elm_frame_content_set(fd, tb2);
  evas_object_show(tb2);
  
  // direction image
  dirimg = elm_image_add(win);
	evas_object_size_hint_weight_set(dirimg, 1, 1);
	evas_object_size_hint_align_set(dirimg, -1.0, -1.0);
	elm_table_pack(tb2, dirimg, 1, 0, 1, 2);
  elm_image_file_set(dirimg, "../pixmaps/arrow_left.png", NULL);
  evas_object_show(dirimg);
  
	// distance label
	lb_distance = elm_label_add(win);
	elm_label_label_set(lb_distance, "20km");
	evas_object_size_hint_weight_set(lb_distance, 1, 1);
	evas_object_size_hint_align_set(lb_distance, 0.5, 0.5);
  elm_object_scale_set(lb_distance, 2.0);
  elm_table_pack(tb2, lb_distance, 0, 0, 1, 1);
  evas_object_show(lb_distance);

	// total distance label
	lb_totdistance = elm_label_add(win);
	elm_label_label_set(lb_totdistance, "88km");
	evas_object_size_hint_weight_set(lb_totdistance, 1, 1);
	evas_object_size_hint_align_set(lb_totdistance, 0.5, 0.5);
  elm_object_scale_set(lb_totdistance, 1.0);
  elm_table_pack(tb2, lb_totdistance, 0, 1, 1, 1);
  evas_object_show(lb_totdistance);

	// time distance label
	lb_timedistance = elm_label_add(win);
	elm_label_label_set(lb_timedistance, "1:25");
	evas_object_size_hint_weight_set(lb_timedistance, 1, 1);
	evas_object_size_hint_align_set(lb_timedistance, 0.5, 0.5);
  elm_object_scale_set(lb_timedistance, 1.0);
  elm_object_scale_set(lb_timedistance, 2.0);
  elm_table_pack(tb2, lb_timedistance, 2, 0, 1, 1);
  evas_object_show(lb_timedistance);

	// time label
	lb_time = elm_label_add(win);
	elm_label_label_set(lb_time, "16:45");
	evas_object_size_hint_weight_set(lb_time, 1, 1);
	evas_object_size_hint_align_set(lb_time, 0.5, 0.5);
  elm_table_pack(tb2, lb_time, 2, 1, 1, 1);
  evas_object_show(lb_time);

  //evas_object_image_alpha_set(mapimg, 255);
  //evas_object_image_colorspace_set(mapimg, EVAS_COLORSPACE_ARGB8888);
  elm_image_file_set(mapimg, "../pixmaps/arrow_left_big.png", NULL);
  /*evas_object_image_size_set(mapimg, 100, 100);
  evas_object_image_fill_set(mapimg, 0, 0, 100, 100);
  //evas_object_color_set(mapimg,255,255,255,255);
  long *data = (long*)malloc(100*100*4);
  memset(data, 0xff, 100*100*4);
  evas_object_image_data_set(mapimg, data);
  evas_object_image_data_update_add(mapimg, 0,0,100,100);*/
  //evas_object_image_save(mapimg,"a.png",NULL,"quality=100");

  int w,h;
  evas_object_image_size_get(mapimg, &w, &h);
  void *p = evas_object_image_data_get(mapimg, NULL);
  printf("Size: %dx%d %p\n", w, h, p);

  // run
  evas_object_resize(win, 300, 300);
	evas_object_show(win);
  elm_run();
  elm_shutdown();
  return 0;
}

