/* -*- Mode: C; indent-tabs-mode: s; c-basic-offset: 2; tab-width: 2 -*- */
/*
 * gui_nav.c
 * Copyright (C) Mike Crash 2009 <mike@mikecrash.com>
 * 
 * main.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * main.c is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdlib.h>
#include <Evas.h>
#include <Ecore.h>
#include <Ecore_Evas.h>
#include <Edje.h>
#include <string.h>
#include <fnmatch.h>
#include <dirent.h>
#include "mcconf.h"
#include "gui_fileopen.h"

#define itinum 10

Evas_Object *edje_fileopen = NULL;

typedef struct fo_file
{
  char *filename;
  struct fo_file *prev;
  struct fo_file *next;
} fo_file;

#define fo_step 10
DIR *fo_dir = NULL;
fo_file *fo_files = NULL;   //first entry in list of files
fo_file *fo_last = NULL;    //last entry in list that was displayed
char *fo_path = NULL;       //path
char *fo_wildcard = NULL;   //file search wildcard
int fo_count = 0;           //count of displayed files
int fo_select = 0;          //selected value
int (*fo_callback)(char *) = NULL;

fo_file *fileopen_nextfile(void)
{
  if (fo_files && (fo_last == NULL))
  {
    fo_last = fo_files;
    return fo_last;
  }
  else if (fo_last && fo_last->next)
  {
    fo_last = fo_last->next;
    return fo_last;
  }
  else if (fo_dir)
  {
    struct dirent *dp;
    while ((dp=readdir(fo_dir)) != NULL)
    {
      //char *tmp;
      //tmp = path_cat(fo_path, dp->d_name);
      //printf("%s\n", dp->d_name);
      if (fnmatch(fo_wildcard, dp->d_name, FNM_FILE_NAME | FNM_CASEFOLD) == 0)
      {
        fo_file *f = (fo_file *)malloc(sizeof(fo_file));
        f->filename = (char*)malloc(strlen(dp->d_name)+1);
        f->next = NULL;
        f->prev = NULL;
        strcpy(f->filename, dp->d_name);
        if (fo_files == NULL)
        {
          fo_last = f;
          fo_files = f;
          return fo_last;
        }
        else
        {
          fo_last->next = f;
          f->prev = fo_last;
          fo_last = f;
          return fo_last;
        }
      }
      //free(tmp);
      //tmp=NULL;
    }
    closedir(fo_dir);
    fo_dir = NULL;
  }
}

void fileopen_back(int step)
{
  while (fo_last && (step))
  {
    fo_last = fo_last->prev;
    step--;
  }
}

void fileopen_forward(int step)
{
  while (fo_last && fo_last->next && (step))
  {
    fo_last = fo_last->next;
    step--;
  }
}

void fileopen_clear()
{
  if (fo_dir)
  {
    closedir(fo_dir);
    fo_dir = NULL;
  }
  if (fo_wildcard)
  {
    free(fo_wildcard);
    fo_wildcard = NULL;
  }
  if (fo_path)
  {
    free(fo_path);
    fo_path = NULL;
  }
  fo_file *f = fo_files;
  while (f)
  {
    fo_file *fn = f->next;
    free(f->filename);
    free(f);
    f = fn;
  }
  fo_files = NULL;
  fo_last = NULL;
}

void fileopen_select(int sel)
{
  if ((sel<0)||(sel>10))
    return;

  int i;
  for (i=1; i<=10; i++)
  {
    char name[50];
    if (i==sel)
      sprintf(name, "file%d,state,select", i);
    else
      sprintf(name, "file%d,state,deselect", i);
    edje_object_signal_emit(edje_fileopen, name, "mcnavi");
  }
}

void fileopen_list()
{
  int i = 0;
  char line[20];

  //get list
  fo_file *f;
  while ((i<fo_step) && (f = fileopen_nextfile()))
  {
    //printf("file%d: %s\n", i, f->filename);
    char *name = (char*)malloc(strlen(f->filename)+1);
    strcpy(name, f->filename);
    if (char *ext = strrchr(name, '.'))
      *ext = 0;
    sprintf(line, "file%d", i+1);
    edje_object_part_text_set(edje_fileopen, line, name);
    i++;
  }
  fo_count = i;

  //clear rest
  while (i<fo_step)
  {
    sprintf(line, "file%d", i+1);
    edje_object_part_text_set(edje_fileopen, line, "");
    i++;
  }
}

void fileopen_click(void *data, Evas_Object *obj, const char *emission, const char *source)
{
  int sel = atoi(emission);
  fo_select = sel;
  fileopen_select(sel);
}

void fileopen_cancel(void *data, Evas_Object *obj, const char *emission, const char *source)
{
  gui_fileopen_hide();
}

void fileopen_up(void *data, Evas_Object *obj, const char *emission, const char *source)
{
  fo_select = 0;
  fileopen_select(0);
  fileopen_back(fo_step + fo_count);
  fileopen_list();
}

void fileopen_down(void *data, Evas_Object *obj, const char *emission, const char *source)
{
  fo_select = 0;
  fileopen_select(0);
  if (fo_count == fo_step)
    fileopen_list();
}

void fileopen_ok(void *data, Evas_Object *obj, const char *emission, const char *source)
{
  if ((fo_select == 0) || (fo_last == NULL))
    return;
  fo_file *f = fo_last;
  int i = fo_count-fo_select;
  while (i>0)
  {
    f = f->prev;
    i--;
  }
  char *filename = (char*)malloc(strlen(f->filename)+strlen(fo_path)+2);
  sprintf(filename, "%s/%s", fo_path, f->filename);
  gui_fileopen_hide();
  fo_callback(filename);
  free(filename);
}

void gui_fileopen_show(Evas *evas, int width, int height, const char *path, const char *wildcard, int (*callback)(char *))
{
  if (edje_fileopen == NULL)
  {
    edje_fileopen = edje_object_add(evas);
    char *edjefile = (char *)malloc(strlen(config->datadir) + 30);
    sprintf(edjefile, "%s/themes/Default/fileopen.edj", config->datadir);
    edje_object_file_set(edje_fileopen, edjefile, "menu_fileopen");
    free(edjefile);
    evas_object_move(edje_fileopen, 0, 0);
    evas_object_resize(edje_fileopen, width, height);
    edje_object_signal_callback_add(edje_fileopen, "cancel", "fileopen", fileopen_cancel, NULL);
    edje_object_signal_callback_add(edje_fileopen, "ok", "fileopen", fileopen_ok, NULL);
    edje_object_signal_callback_add(edje_fileopen, "up", "fileopen", fileopen_up, NULL);
    edje_object_signal_callback_add(edje_fileopen, "down", "fileopen", fileopen_down, NULL);
    edje_object_signal_callback_add(edje_fileopen, "*", "select", fileopen_click, NULL);
/*    edje_object_signal_callback_add(edje_itinerary, "load", "itinerary", itinerary_load, NULL);
    edje_object_signal_callback_add(edje_itinerary, "save", "itinerary", itinerary_save, NULL);
    edje_object_signal_callback_add(edje_itinerary, "clear", "itinerary", itinerary_clear, NULL);
    edje_object_signal_callback_add(edje_itinerary, "delete", "itinerary", itinerary_delete, NULL);
    edje_object_signal_callback_add(edje_itinerary, "ok", "itinerary", itinerary_ok, NULL);
    //edje_object_signal_callback_add(edje_itinerary, "*", "navname", navpress, NULL);*/
  }
  evas_object_show(edje_fileopen);
  fo_dir = opendir(path);
  fo_wildcard = (char *)malloc(strlen(wildcard)+1);
  strcpy(fo_wildcard, wildcard);
  fo_path = (char *)malloc(strlen(path)+1);
  strcpy(fo_path, path);
  fo_select = 0;
  fo_last = NULL;
  fo_files = NULL;
  fo_callback = callback;
  fileopen_select(0);
  fileopen_list();
}

void gui_fileopen_resize(int width, int height)
{
  if (edje_fileopen)
  {
    evas_object_resize(edje_fileopen, width, height);
  }
}

void gui_fileopen_hide(void)
{
  evas_object_hide(edje_fileopen);
  fileopen_clear();
}

void gui_fileopen_free(void)
{
  if (edje_fileopen != NULL)
  {
    evas_object_del(edje_fileopen);
    edje_fileopen = NULL;
  }
}
