/* -*- Mode: C; indent-tabs-mode: s; c-basic-offset: 2; tab-width: 2 -*- */
/*
 * gui_track.c
 * Copyright (C) Mike Crash 2009 <mike@mikecrash.com>
 * 
 * main.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * main.c is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdlib.h>
#include <time.h>
#include <Evas.h>
#include <Ecore.h>
#include <Ecore_Evas.h>
#include <Edje.h>
#include <string.h>
#include "mcconf.h"
#include "mctime.h"
#include "gui.h"
#include "gui_gps.h"
#include "gui_address.h"
#include "gui_satellites.h"
#include "gui_text.h"
#include "navigation.h"

//gui
Evas *evas_gps_parent = NULL;
Evas_Object *edje_gps = NULL;
int edje_gps_w, edje_gps_h;
int edje_gps_visible = 0;

//chronometer
void gui_gps_chrono(void *data, Evas_Object *obj, const char *emission, const char *source)
{
  if (config->chrono_ena)
  {
    config->chrono_ena = 0;
    edje_object_signal_emit(edje_gps, "gpschrono,state,off", "mcnavi");
  }
  else
  {
    config->chrono_ena = 1;
    config->chrono_len = 0;
    timer_start(&config->chrono_start);
    edje_object_signal_emit(edje_gps, "gpschrono,state,on", "mcnavi");
  }
}
//satelites
void gui_gps_sat(void *data, Evas_Object *obj, const char *emission, const char *source)
{
  gui_gps_hide();
  gui_sat_show(evas_gps_parent, win_width, win_height);
}

//log track
void gui_gps_move(void *data, Evas_Object *obj, const char *emission, const char *source)
{
  gui_gps_hide();
  gui_addr_show(evas_gps_parent, GRT_MOVE, "Move to", win_width, win_height);
}

//save bookmark
int gui_gps_bookmarkcallback(char *text)
{
  char s[255];
  char *homedir=getenv("HOME");
  snprintf(s, sizeof(s), "%s/.mcnavi/bookmarks", homedir);
  FILE *f = fopen(s, "a");
  if (f == NULL)
  {
    printf("Cannot write bookmark to %s\n", s);
    return -1;
  }

  map_position pos;
  int angle;
  if (nav_getpos(&pos, &angle) == 0)
  {
    fprintf(f, "%s=%ld,%ld\n", text, pos.lat, pos.lon);
  }
  fclose(f);
  
  return 0;
}

void gui_gps_bookmarks(void *data, Evas_Object *obj, const char *emission, const char *source)
{
  gui_gps_hide();
  gui_text_show(evas_gps_parent, "Bookmark name", win_width, win_height, gui_gps_bookmarkcallback);
}

//exit
void gui_gps_exit(void *data, Evas_Object *obj, const char *emission, const char *source)
{
  gui_gps_hide();
  gui_close();
}

//track show
void gui_gps_show(Evas *evas, int x, int y, int width, int height)
{
  if (edje_gps_visible)
  {
    gui_gps_hide();
    return;
  }
  if (edje_gps == NULL)
  {
    char buf[255];
    edje_gps = edje_object_add(evas);
    snprintf(buf, sizeof(buf), "%s/themes/Default/main.edj", config->datadir);
    edje_object_file_set(edje_gps, buf, "menu_gps");
    evas_object_move(edje_gps, x, y);
    evas_object_resize(edje_gps, width, height);
    edje_gps_w = width;
    edje_gps_h = height;
    edje_object_signal_callback_add(edje_gps, "sat", "gps", gui_gps_sat, NULL);
    edje_object_signal_callback_add(edje_gps, "move", "gps", gui_gps_move, NULL);
    edje_object_signal_callback_add(edje_gps, "bookmarks", "gps", gui_gps_bookmarks, NULL);
    edje_object_signal_callback_add(edje_gps, "chrono", "gps", gui_gps_chrono, NULL);
    edje_object_signal_callback_add(edje_gps, "exit", "gps", gui_gps_exit, NULL);
    evas_gps_parent = evas;
  }
  if (config->chrono_ena)
    edje_object_signal_emit(edje_gps, "gpschrono,state,on", "mcnavi");
  else
    edje_object_signal_emit(edje_gps, "gpschrono,state,off", "mcnavi");
  evas_object_show(edje_gps);

  edje_gps_visible = 1;
}

void gui_gps_resize(int x, int y, int width, int height)
{
  if (edje_gps)
  {
    evas_object_move(edje_gps, x, y);
    evas_object_resize(edje_gps, width, height);
    edje_gps_w = width;
    edje_gps_h = height;
  }
}

void gui_gps_hide(void)
{
  evas_object_hide(edje_gps);
  edje_gps_visible = 0;
}

void gui_gps_free(void)
{
  if (edje_gps != NULL)
  {
    evas_object_del(edje_gps);
    edje_gps = NULL;
  }
}
