/* -*- Mode: C; indent-tabs-mode: s; c-basic-offset: 2; tab-width: 2 -*- */
/*
 * gui_nav.c
 * Copyright (C) Mike Crash 2009 <mike@mikecrash.com>
 * 
 * main.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * main.c is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdlib.h>
#include <Evas.h>
#include <Ecore.h>
#include <Ecore_Evas.h>
#include <Edje.h>
#include <string.h>
#include "mcconf.h"
#include "route.h"
#include "gui.h"
#include "gui_itinerary.h"
#include "gui_fileopen.h"
#include "gui_text.h"
#include "diacritics.h"
#include "navigation.h"

#define itinum 10

Evas *evas_itinerary_parent = NULL;
Evas_Object *edje_itinerary = NULL;

int it_start = 1;      //item=0 is first itinerary item (start/current position), avoid it to show
int it_selected = 0;   //selected item
int it_count = 0;      //count of itinerary items

int itinerary_count()
{
  route_itinerary *ri = route->itinerary;
  int i = 0;
  while (ri)
  {
    ri = ri->next;
    i++;
  }
  return i;
}

void itinerary_select(int sel)
{
  if ((sel<0)||(sel>10))
    return;

  int i;
  for (i=it_start; i<=it_start+10; i++)
  {
    char itiname[50];
    if ((i==sel) && (i<it_count))
      sprintf(itiname, "itinerary%d,state,select", i);
    else
      sprintf(itiname, "itinerary%d,state,deselect", i);
    edje_object_signal_emit(edje_itinerary, itiname, "mcnavi");
  }
}

void itinerary_list()
{
  route_itinerary *ri = route->itinerary;
  //move to first itinerary to show
  int i = 0;
  while (ri && (i<it_start))
  {
    ri = ri->next;
    i++;
  }
  //show
  i = 0;
  while (ri && (i<itinum))
  {
    char itiname[50];
    //name
    sprintf(itiname, "itinerary%d", i+1);
    if (ri->name)
      edje_object_part_text_set(edje_itinerary, itiname, ri->name);
    else
      edje_object_part_text_set(edje_itinerary, itiname, "unnamed");
    //type
    if (ri->nodetype == nt_destination)
      sprintf(itiname, "itinerary%d,state,destination", i+1);
    else if (ri->nodetype == nt_waypoint)
      sprintf(itiname, "itinerary%d,state,waypoint", i+1);
    else if (ri->nodetype == nt_direction)
      sprintf(itiname, "itinerary%d,state,direction", i+1);
    edje_object_signal_emit(edje_itinerary, itiname, "mcnavi");
    ri = ri->next;
    i++;
  }
  //clear the rest
  while (i<itinum)
  {
    char itiname[50];
    sprintf(itiname, "itinerary%d", i+1);
    edje_object_part_text_set(edje_itinerary, itiname, "");
    sprintf(itiname, "itinerary%d,state,none", i+1);
    edje_object_signal_emit(edje_itinerary, itiname, "mcnavi");
    i++;
  }
  itinerary_select(it_selected);
}

void itinerary_click(void *data, Evas_Object *obj, const char *emission, const char *source)
{
  int sel = atoi(emission);
  it_selected = it_start + sel - 1;
  if (it_selected>it_count)
    it_selected = 0;
  itinerary_select(it_selected);
}

int itinerary_load_callback(char *filename)
{
  nav_load(filename);
  return 0;
}

void itinerary_load(void *data, Evas_Object *obj, const char *emission, const char *source)
{
  gui_itinerary_hide();
  gui_fileopen_show(evas_itinerary_parent, win_width, win_height, config->dir_routes, "*.gpx", itinerary_load_callback);
}

int itinerary_save_callback(char *filename)
{
  nav_save(filename);
  return 0;
}

void itinerary_save(void *data, Evas_Object *obj, const char *emission, const char *source)
{
  gui_itinerary_hide();
  gui_text_show(evas_itinerary_parent, "Save route", win_width, win_height, itinerary_save_callback);
}

void itinerary_clear(void *data, Evas_Object *obj, const char *emission, const char *source)
{
  nav_stop();
  config->route_add = routeadd_destination;
  //set_routing_add(config->route_add);
  gui_itinerary_hide();
}

void itinerary_delete(void *data, Evas_Object *obj, const char *emission, const char *source)
{
  if (it_selected == 0)
    return;

  nav_delete(it_selected);
  gui_itinerary_hide();
}

void itinerary_up(void *data, Evas_Object *obj, const char *emission, const char *source)
{
  if (it_start > 1)
  {
    it_start--;
    itinerary_list();
  }
}

void itinerary_down(void *data, Evas_Object *obj, const char *emission, const char *source)
{
  if (it_start < (it_count - 1))
  {
    it_start++;
    itinerary_list();
  }

}

void itinerary_ok(void *data, Evas_Object *obj, const char *emission, const char *source)
{
  gui_itinerary_hide();
}

void gui_itinerary_show(Evas *evas, int width, int height)
{
  if (edje_itinerary == NULL)
  {
    edje_itinerary = edje_object_add(evas);
    char *edjefile = (char *)malloc(strlen(config->datadir) + 30);
    sprintf(edjefile, "%s/themes/Default/itinerary.edj", config->datadir);
    edje_object_file_set(edje_itinerary, edjefile, "menu_itinerary");
    free(edjefile);
    evas_object_move(edje_itinerary, 0, 0);
    evas_object_resize(edje_itinerary, width, height);
    edje_object_signal_callback_add(edje_itinerary, "load", "itinerary", itinerary_load, NULL);
    edje_object_signal_callback_add(edje_itinerary, "save", "itinerary", itinerary_save, NULL);
    edje_object_signal_callback_add(edje_itinerary, "clear", "itinerary", itinerary_clear, NULL);
    edje_object_signal_callback_add(edje_itinerary, "delete", "itinerary", itinerary_delete, NULL);
    edje_object_signal_callback_add(edje_itinerary, "up", "itinerary", itinerary_up, NULL);
    edje_object_signal_callback_add(edje_itinerary, "down", "itinerary", itinerary_down, NULL);
    edje_object_signal_callback_add(edje_itinerary, "ok", "itinerary", itinerary_ok, NULL);
    edje_object_signal_callback_add(edje_itinerary, "*", "select", itinerary_click, NULL);
    //edje_object_signal_callback_add(edje_itinerary, "*", "navname", navpress, NULL);
    evas_itinerary_parent = evas;
  }
  evas_object_show(edje_itinerary);
  it_count = itinerary_count();
  if (it_selected>it_count)
    it_selected = it_count;
  if (it_count < itinum)
    it_start = 1;
  else if ((it_start+itinum)>it_count)
      it_start = it_count - itinum;
  itinerary_list();
  itinerary_select(0);
}

void gui_itinerary_resize(int width, int height)
{
  if (edje_itinerary)
  {
    evas_object_resize(edje_itinerary, width, height);
  }
}

void gui_itinerary_hide(void)
{
  evas_object_hide(edje_itinerary);
}

void gui_itinerary_free(void)
{
  if (edje_itinerary != NULL)
  {
    evas_object_del(edje_itinerary);
    edje_itinerary = NULL;
  }
}
