/* -*- Mode: C; indent-tabs-mode: s; c-basic-offset: 2; tab-width: 2 -*- */
/*
 * gui_route.c
 * Copyright (C) Mike Crash 2009 <mike@mikecrash.com>
 * 
 * main.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * main.c is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdlib.h>
#include <Evas.h>
#include <Ecore.h>
#include <Ecore_Evas.h>
#include <Edje.h>
#include <string.h>
#include "mcconf.h"
#include "gui.h"
#include "gui_route.h"
#include "gui_address.h"
#include "gui_itinerary.h"
#include "navigation.h"

//gui
Evas *evas_route_parent = NULL;
Evas_Object *edje_route = NULL;
int edje_route_w, edje_route_h;
int edje_route_visible = 0;

routing_vehicle get_routing_vehicle()
{
  double val;
  const char *state;
  state = edje_object_part_state_get(edje_route, "route_vehicle", &val);
  if (strcmp(state, "train") == 0)
    return RV_TRAIN;
  if (strcmp(state, "truck") == 0)
    return RV_TRUCK;
  if (strcmp(state, "car") == 0)
    return RV_CAR;
  if (strcmp(state, "cycle") == 0)
    return RV_CYCLE;
  if (strcmp(state, "mtb") == 0)
    return RV_MTB;
  if (strcmp(state, "foot") == 0)
    return RV_FOOT;
  return RV_CAR;
}

void set_routing_vehicle(routing_vehicle type)
{
  switch (type)
  {
    case RV_TRAIN:
      edje_object_signal_emit(edje_route, "vehicle,state,train", "mcnavi");
      break;
    case RV_TRUCK:
      edje_object_signal_emit(edje_route, "vehicle,state,truck", "mcnavi");
      break;
    case RV_CAR:
      edje_object_signal_emit(edje_route, "vehicle,state,car", "mcnavi");
      break;
    case RV_CYCLE:
      edje_object_signal_emit(edje_route, "vehicle,state,cycle", "mcnavi");
      break;
    case RV_MTB:
      edje_object_signal_emit(edje_route, "vehicle,state,mtb", "mcnavi");
      break;
    case RV_FOOT:
      edje_object_signal_emit(edje_route, "vehicle,state,foot", "mcnavi");
      break;
  }
}

routing_mode get_routing_mode(void)
{
  double val;
  const char *state;
  state = edje_object_part_state_get(edje_route, "route_mode", &val);
  if (strcmp(state, "toll") == 0)
    return routemode_toll;
  else if (strcmp(state, "speed") == 0)
    return routemode_speed;
  else
    return routemode_length;
}

void set_routing_mode(routing_mode mode)
{
  switch (mode)
  {
    case routemode_length:
      edje_object_signal_emit(edje_route, "routemode,state,length", "mcnavi");
      break;
    case routemode_speed:
      edje_object_signal_emit(edje_route, "routemode,state,speed", "mcnavi");
      break;
    case routemode_toll:
      edje_object_signal_emit(edje_route, "routemode,state,toll", "mcnavi");
      break;
  }
}

routing_add get_routing_add(void)
{
  double val;
  const char *state;
  state = edje_object_part_state_get(edje_route, "route_add", &val);
  if (strcmp(state, "destination") == 0)
    return routeadd_destination;
  else if (strcmp(state, "waypoint") == 0)
    return routeadd_waypoint;
  else
    return routeadd_direction;
}

void set_routing_add(routing_add add)
{
  if (add == routeadd_destination)
  {
    edje_object_signal_emit(edje_route, "routeadd,state,destination", "mcnavi");
  }
  else if (add == routeadd_waypoint)
  {
    edje_object_signal_emit(edje_route, "routeadd,state,waypoint", "mcnavi");
  }
  else
  {
    edje_object_signal_emit(edje_route, "routeadd,state,direction", "mcnavi");
  }
}

//itinerary
void gui_route_itinerary(void *data, Evas_Object *obj, const char *emission, const char *source)
{
  gui_route_hide();
  gui_itinerary_show(evas_route_parent, win_width, win_height);
  /*nav_stop();
  config->route_add = routeadd_destination;
  set_routing_add(config->route_add);*/
}

//add next point as address
void gui_route_address(void *data, Evas_Object *obj, const char *emission, const char *source)
{
  gui_route_hide();
  gui_addr_show(evas_route_parent, GRT_ROUTE, (char*)"Route to", win_width, win_height);
}

//add next point from map
void gui_route_hand(void *data, Evas_Object *obj, const char *emission, const char *source)
{
  //gui_route_hide();
  routing_add add = get_routing_add();
  gui_mode mode;
  switch (add)
  {
    case routeadd_destination:
      mode = mode_destination;
      break;
    case routeadd_waypoint:
      mode = mode_waypoint;
      break;
    case routeadd_direction:
      mode = mode_direction;
      break;
  }
  gui_setmode(mode);
  gui_route_hide();
}

//vehicle
void gui_route_vehicle(void *data, Evas_Object *obj, const char *emission, const char *source)
{
  if (config->route_vehicle == RV_FOOT)
    config->route_vehicle = RV_TRAIN;
  else
    config->route_vehicle = (routing_vehicle)(config->route_vehicle + 1);
  set_routing_vehicle(config->route_vehicle);
}

//mode
void gui_route_mode(void *data, Evas_Object *obj, const char *emission, const char *source)
{
  switch (config->route_mode)
  {
    case routemode_toll:
      config->route_mode =routemode_speed;
      break;
    case routemode_speed:
      config->route_mode =routemode_length;
      break;
    case routemode_length:
      config->route_mode =routemode_toll;
      break;
  }
  set_routing_mode(config->route_mode);
}

//add
void gui_route_add(void *data, Evas_Object *obj, const char *emission, const char *source)
{
  if ((route==NULL) || (route->route == NULL))
    config->route_add = routeadd_destination;
  else if (config->route_add==routeadd_destination)
    config->route_add = routeadd_waypoint;
  else if (config->route_add==routeadd_waypoint)
    config->route_add = routeadd_destination; // routeadd_direction;
  //else if (config->route_add==routeadd_direction)
  //  config->route_add = routeadd_destination;
  set_routing_add(config->route_add);
}

//show
void gui_route_show(Evas *evas, int x, int y, int width, int height)
{
  if (edje_route_visible)
  {
    gui_route_hide();
    return;
  }
  if (edje_route == NULL)
  {
    char buf[255];
    edje_route = edje_object_add(evas);
    snprintf(buf, sizeof(buf), "%s/themes/Default/main.edj", config->datadir);
    edje_object_file_set(edje_route, buf, "menu_route");
    evas_object_move(edje_route, x, y);
    evas_object_resize(edje_route, width, height);
    edje_route_w = width;
    edje_route_h = height;
    edje_object_signal_callback_add(edje_route, "itinerary", "route", gui_route_itinerary, NULL);
    edje_object_signal_callback_add(edje_route, "address", "route", gui_route_address, NULL);
    edje_object_signal_callback_add(edje_route, "hand", "route", gui_route_hand, NULL);
    edje_object_signal_callback_add(edje_route, "vehicle", "route", gui_route_vehicle, NULL);
    edje_object_signal_callback_add(edje_route, "mode", "route", gui_route_mode, NULL);
    edje_object_signal_callback_add(edje_route, "add", "route", gui_route_add, NULL);
    evas_route_parent = evas;
  }
  evas_object_show(edje_route);
  set_routing_vehicle(config->route_vehicle);
  set_routing_add(config->route_add);

  edje_route_visible = 1;
}

void gui_route_resize(int x, int y, int width, int height)
{
  if (edje_route)
  {
    evas_object_move(edje_route, x, y);
    evas_object_resize(edje_route, width, height);
    edje_route_w = width;
    edje_route_h = height;
  }
}

void gui_route_hide(void)
{
  evas_object_hide(edje_route);
  edje_route_visible = 0;
  nav_setvehicle(config->route_vehicle);
}

void gui_route_free(void)
{
  if (edje_route != NULL)
  {
    evas_object_del(edje_route);
    edje_route = NULL;
  }
}
