/* -*- Mode: C; indent-tabs-mode: s; c-basic-offset: 2; tab-width: 2 -*- */
/*
 * gui_nav.c
 * Copyright (C) Mike Crash 2009 <mike@mikecrash.com>
 *
 * main.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * main.c is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdlib.h>
#include <Evas.h>
#include <Ecore.h>
#include <Ecore_Evas.h>
#include <Edje.h>
#include <string.h>
#include "functions.h"
#include "mcconf.h"
#include "mgps.h"
#include "navigation.h"
#include "gui_satellites.h"

Evas *evas_sat = NULL;
Evas_Object *edje_sat = NULL;
Ecore_Timer *sat_timer = NULL;

typedef struct
{
  Evas_Object *img;
  Evas_Object *text;
  int w;
  int h;
  int PRN;
} satimg;

satimg satimgs[maxsats];

//satellite PRN to show detailed info
int satprn = 0;

void satpress(void *data, Evas *e, Evas_Object *obj, void *event_info);

void pol2cart(int azimuth, int elevation, int width, int height, int *x, int *y)
{
  double a = azimuth * pi / 180;
#ifdef PCORRECT
  double e = sin((90 - elevation) * pi / 180);
#else
  double e = (90.0 - elevation) / 90.0;
  //double e = elevation / 90.0;
#endif
  int d = min(width, height) * 8 / 10;
  *x = (int)((width / 2) + sin(a) * e * (d / 2));
  *y = (int)((height / 2) - cos(a) * e * (d / 2));
}

void updatesat()
{
  int sats_num;
  mgpssat *sats;
  char s[50];
  int detailed = 0;

  if (nav_satinfo(&sats_num, &sats) == 0)
  {
    int i;
    int x, y, w, h;
    edje_object_part_geometry_get(edje_sat, "compass", &x, &y ,&w, &h);
    for (i=0; i<sats_num; i++)
    {
      if (satprn == sats[i].PRN)
      {
        snprintf(s, sizeof(s), "%d", sats[i].PRN);
        edje_object_part_text_set(edje_sat, "sat_prn", s);
        snprintf(s, sizeof(s), "%d°", sats[i].elevation);
        edje_object_part_text_set(edje_sat, "sat_elev", s);
        snprintf(s, sizeof(s), "%d°", sats[i].azimuth);
        edje_object_part_text_set(edje_sat, "sat_azim", s);
        if (sats[i].SNR>0)
        {
          snprintf(s, sizeof(s), "%d", sats[i].SNR);
          edje_object_part_text_set(edje_sat, "sat_snr", s);
        }
        else
        {
          edje_object_part_text_set(edje_sat, "sat_snr", "");
        }
        if (sats[i].used)
          edje_object_part_text_set(edje_sat, "sat_used", "Yes");
        else
          edje_object_part_text_set(edje_sat, "sat_used", "No");
        detailed = 1;
      }
    }
    nav_unlock();
  }
  if (detailed == 0)
  {
    edje_object_part_text_set(edje_sat, "sat_prn", "Click to satellite");
    edje_object_part_text_set(edje_sat, "sat_elev", "");
    edje_object_part_text_set(edje_sat, "sat_azim", "");
    edje_object_part_text_set(edje_sat, "sat_snr", "");
    edje_object_part_text_set(edje_sat, "sat_used", "");
  }
}

void updatecompass()
{
  int i;
  int sats_num = 0;
  mgpssat *sats;
  char s[50];

  //update GPS info
  long lat, lon;
  int fix, speed, alt, angle, eph, epv;
  if (nav_gpsinfo(&fix, &lat, &lon, &speed, &alt, &angle, &eph, &epv) == 0)
  {
    if (lat>=0)
      sprintf(s, "%ld.%06ld N", lat/1000000, lat%1000000);
    else
      sprintf(s, "%ld.%06ld S", -lat/1000000, -lat%1000000);
    edje_object_part_text_set(edje_sat, "gps_lat", s);
    if (lon>=0)
      sprintf(s, "%ld.%06ld E", lon/1000000, lon%1000000);
    else
      sprintf(s, "%ld.%06ld W", -lon/1000000, -lon%1000000);
    edje_object_part_text_set(edje_sat, "gps_lon", s);
    nice_dist(s, alt, 1);
    edje_object_part_text_set(edje_sat, "gps_alt", s);
    sprintf(s, "%d", speed);
    edje_object_part_text_set(edje_sat, "gps_speed", s);
    sprintf(s, "%d°", angle);
    edje_object_part_text_set(edje_sat, "gps_track", s);
  }
  else
  {
    edje_object_part_text_set(edje_sat, "gps_lat", "");
    edje_object_part_text_set(edje_sat, "gps_lon", "");
    edje_object_part_text_set(edje_sat, "gps_alt", "");
    edje_object_part_text_set(edje_sat, "gps_speed", "");
    edje_object_part_text_set(edje_sat, "gps_track", "");
  }
  switch (fix)
  {
    case 2:
      edje_object_part_text_set(edje_sat, "gps_fix", "2D FIX");
      break;
    case 3:
      edje_object_part_text_set(edje_sat, "gps_fix", "3D FIX");
      break;
    default:
      edje_object_part_text_set(edje_sat, "gps_fix", "NO FIX");
      break;
  }
  if (fix>1)
  {
    sprintf(s, "%d/%d m", eph, epv);
    edje_object_part_text_set(edje_sat, "gps_eph", s);
  }
  else
  {
    edje_object_part_text_set(edje_sat, "gps_eph", "");
  }

  //update compass
  if (nav_satinfo(&sats_num, &sats) == 0)
  {
    int x, y, w, h;
    edje_object_part_geometry_get(edje_sat, "compass", &x, &y ,&w, &h);
    //update visible satelites
    for (i=0; i<sats_num; i++)
    {
      if (i<sats_num)
      {
        //load image
        if (satimgs[i].img == NULL)
        {
          char buf[255];
          snprintf(buf, sizeof(buf), "%s/themes/Default/satellite.png", config->datadir);
          satimgs[i].img = evas_object_image_add(evas_sat);
          evas_object_image_file_set(satimgs[i].img, buf, 0);
          evas_object_image_size_get(satimgs[i].img, &satimgs[i].w, &satimgs[i].h);
          //printf("%d x %d\n", satimgs[i].w, satimgs[i].h);
          evas_object_resize(satimgs[i].img, satimgs[i].w, satimgs[i].h);
          evas_object_image_fill_set(satimgs[i].img, 0, 0, satimgs[i].w, satimgs[i].h);
        }
        else
        {
          evas_object_event_callback_del(satimgs[i].img, EVAS_CALLBACK_MOUSE_UP, satpress);
        }
        //prn
        if (satimgs[i].text == NULL)
        {
          satimgs[i].text = evas_object_text_add(evas_sat);
          evas_object_text_font_set(satimgs[i].text, "DejaVuSans-Bold", satimgs[i].h*1/3);
          evas_object_text_style_set(satimgs[i].text, EVAS_TEXT_STYLE_GLOW);
          evas_object_text_glow_color_set(satimgs[i].text, 0, 0, 0, 255);
        }
        else
        {
          evas_object_event_callback_del(satimgs[i].text, EVAS_CALLBACK_MOUSE_UP, satpress);
        }
        //set position
        int x0 = 0, y0 = 0;
        pol2cart(sats[i].azimuth, sats[i].elevation, w, h, &x0, &y0);
        //image
        evas_object_move(satimgs[i].img, x + x0 - satimgs[i].w/2, y + y0 - satimgs[i].h/2);
        evas_object_raise(satimgs[i].img);
        evas_object_show(satimgs[i].img);
        //text
        snprintf(s, sizeof(s), "%d", sats[i].PRN);
        evas_object_text_text_set(satimgs[i].text, s);
        if (sats[i].used)
          evas_object_color_set(satimgs[i].text, 0x7d, 0xff, 0x7d, 255);
        else
          evas_object_color_set(satimgs[i].text, 0xff, 0x7d, 0x7d, 255);
        int xt, yt, wt, ht;
        evas_object_geometry_get(satimgs[i].text, &xt, &yt, &wt, &ht);
        evas_object_move(satimgs[i].text, x + x0 - wt/2, y + y0 - ht/2);
        evas_object_raise(satimgs[i].text);
        evas_object_show(satimgs[i].text);
        //detailed info event
        satimgs[i].PRN = sats[i].PRN;
        evas_object_event_callback_add(satimgs[i].img, EVAS_CALLBACK_MOUSE_UP, satpress, &satimgs[i].PRN);
        evas_object_event_callback_add(satimgs[i].text, EVAS_CALLBACK_MOUSE_UP, satpress, &satimgs[i].PRN);
      }
      else
      {
        if (satimgs[i].img)
        {
          evas_object_hide(satimgs[i].img);
        }
      }
    }
    nav_unlock();
  }
  //hide previous satellites, if initialized
  for (i=sats_num; i<maxsats; i++)
  {
    if (satimgs[i].img)
    {
      evas_object_show(satimgs[i].img);
    }
    if (satimgs[i].text)
    {
      evas_object_show(satimgs[i].text);
    }
  }

  
  updatesat();
}

void satpress(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
  if (data)
  {
    int *prn = (int *)data;
    satprn = *prn;
    updatesat();
  }
}

void closepress(void *data, Evas_Object *obj, const char *emission, const char *source)
{
  gui_sat_hide();
}

Eina_Bool gui_sat_timer(void *data)
//int gui_sat_timer(void *data)
{
  updatecompass();
  return 1;
}

void gui_sat_show(Evas *evas, int width, int height)
{
  if (edje_sat == NULL)
  {
    memset(satimgs, 0, sizeof(satimgs));
    evas_sat = evas;
    edje_sat = edje_object_add(evas);
    char *edjefile = (char*)malloc(strlen(config->datadir) + 40);
    sprintf(edjefile, "%s/themes/Default/satellites.edj", config->datadir);
    edje_object_file_set(edje_sat, edjefile, "satellites");
    free(edjefile);
    evas_object_move(edje_sat, 0, 0);
    evas_object_resize(edje_sat, width, height);
    edje_object_signal_callback_add(edje_sat, "*", "sat", closepress, NULL);
  }
  if (sat_timer == NULL)
  {
    sat_timer = ecore_timer_add(1.0, gui_sat_timer, NULL);
  }
  evas_object_show(edje_sat);
  satprn = 0;
  updatecompass();
}

void gui_sat_resize(int width, int height)
{
  if (edje_sat)
  {
    evas_object_resize(edje_sat, width, height);
  }
}

void gui_sat_hide(void)
{
  int i;

  evas_object_hide(edje_sat);
  if (sat_timer)
  {
    ecore_timer_del(sat_timer);
    sat_timer = NULL;
  }
  for (i=0; i<maxsats; i++)
  {
    if (satimgs[i].img)
    {
      evas_object_hide(satimgs[i].img);
    }
    if (satimgs[i].text)
    {
      evas_object_hide(satimgs[i].text);
    }
  }
}

void gui_sat_free(void)
{
  int i;
  if (edje_sat != NULL)
  {
    evas_object_del(edje_sat);
    edje_sat = NULL;
  }
  for (i=0; i<maxsats; i++)
  {
    if (satimgs[i].img)
    {
      evas_object_del(satimgs[i].img);
    }
    if (satimgs[i].text)
    {
      evas_object_del(satimgs[i].text);
    }
  }
}
