/* -*- Mode: C; indent-tabs-mode: s; c-basic-offset: 2; tab-width: 2 -*- */
/*
 * gui_text.c
 * Copyright (C) Mike Crash 2009 <mike@mikecrash.com>
 *
 * main.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * main.c is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdlib.h>
#include <Evas.h>
#include <Ecore.h>
#include <Ecore_Evas.h>
#include <Edje.h>
#include <ctype.h>
#include <string.h>
#include "mcconf.h"
#include "gui_text.h"

Evas_Object *edje_text = NULL;
int (*gui_text_callback)(char *) = NULL;
char gui_text_str[255];
int gui_text_len = 0;
int gui_text_shift = 0;

void gui_text_but(void *data, Evas_Object *obj, const char *emission, const char *source)
{
  printf("%s\n%s\n", emission, source);
  if (strcmp(emission, "ok") == 0)
  {
    gui_text_hide();
    gui_text_callback(gui_text_str);
    return;
  }
  if (strcmp(emission, "cancel") == 0)
  {
    gui_text_hide();
    return;
  }

  if (gui_text_len >= 254)
    return;

  if (strcmp(emission, "shift") == 0)
  {
    if (gui_text_shift)
      gui_text_shift = 0;
    else
      gui_text_shift = 1;
  }
  if (strcmp(emission, "back") == 0)
  {
    if (gui_text_len>0)
    {
      gui_text_len--;
      gui_text_str[gui_text_len] = 0;
    }
  }
  else if (strcmp(emission, "space") == 0)
  {
    gui_text_str[gui_text_len] = ' ';
    gui_text_len++;
    gui_text_str[gui_text_len] = 0;
  }
  else if (strcmp(emission, "comma") == 0)
  {
    gui_text_str[gui_text_len] = ',';
    gui_text_len++;
    gui_text_str[gui_text_len] = 0;
  }
  else if (strcmp(emission, "point") == 0)
  {
    gui_text_str[gui_text_len] = '.';
    gui_text_len++;
    gui_text_str[gui_text_len] = 0;
  }
  else if (strcmp(emission, "semicolon") == 0)
  {
    gui_text_str[gui_text_len] = ';';
    gui_text_len++;
    gui_text_str[gui_text_len] = 0;
  }
  else if (strlen(emission) == 1)
  {
    gui_text_str[gui_text_len] = gui_text_shift ? toupper(emission[0]) : emission[0];
    gui_text_len++;
    gui_text_str[gui_text_len] = 0;
    gui_text_shift = 0;
  }
  edje_object_part_text_set(edje_text, "text", gui_text_str);
}


void gui_text_show(Evas *evas, const char *label, int width, int height, int (*callback)(char *))
{
  if (edje_text == NULL)
  {
    edje_text = edje_object_add(evas);
    char *edjefile = (char*)malloc(strlen(config->datadir) + 30);
    sprintf(edjefile, "%s/themes/Default/text.edj", config->datadir);
    edje_object_file_set(edje_text, edjefile, "menu_text");
    free(edjefile);
    evas_object_move(edje_text, 0, 0);
    evas_object_resize(edje_text, width, height);
    edje_object_signal_callback_add(edje_text, "*", "but", gui_text_but, NULL);
  }
  edje_object_part_text_set(edje_text, "label", label);
  evas_object_show(edje_text);
  gui_text_callback = callback;
  gui_text_len = 0;
  gui_text_str[0] = 0;
  gui_text_shift = 1;
  edje_object_part_text_set(edje_text, "text", gui_text_str);
}

void gui_text_resize(int width, int height)
{
  if (edje_text)
  {
    evas_object_resize(edje_text, width, height);
  }
}

void gui_text_hide(void)
{
  evas_object_hide(edje_text);
}

void gui_text_free(void)
{
  if (edje_text != NULL)
  {
    evas_object_del(edje_text);
    edje_text = NULL;
  }
}
