/* -*- Mode: C; indent-tabs-mode: s; c-basic-offset: 2; tab-width: 2 -*- */
/*
 * gui_track.c
 * Copyright (C) Mike Crash 2009 <mike@mikecrash.com>
 * 
 * main.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * main.c is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdlib.h>
#include <time.h>
#include <Evas.h>
#include <Ecore.h>
#include <Ecore_Evas.h>
#include <Edje.h>
#include <string.h>
#include "mcconf.h"
#include "mctime.h"
#include "gui.h"
#include "gui_track.h"
#include "gui_fileopen.h"
#include "gpx.h"
#include "navigation.h"

//gui
Evas *evas_track_parent = NULL;
Evas_Object *edje_track = NULL;
int edje_track_w, edje_track_h;
int edje_track_visible = 0;

//log track
void gui_track_log(void *data, Evas_Object *obj, const char *emission, const char *source)
{
  if (config->gpx_log)
  {
    config->gpx_log = 0;
    gpx_log_stop();
    edje_object_signal_emit(edje_track, "tracksave,state,off", "mcnavi");
  }
  else
  {
    char ts[50], fs[255];
    time_t t = time (NULL);
    struct tm lt;
    localtime_r (&t, &lt);
    strftime(ts, sizeof(ts), "%Y-%m-%d %H-%M-%S.gpx", &lt);
    snprintf(fs, sizeof(fs), "%s/%s", config->dir_tracks, ts);
    if (gpx_log(fs) == 0)
    {
      config->gpx_log = 1;
      edje_object_signal_emit(edje_track, "tracksave,state,on", "mcnavi");
    }
  }
  //gui_track_hide();
}

int gui_track_callback(char *filename)
{
  gpx_load(filename);
  gpx_lock();
  if (gpxtrack && gpxtrack->first && gpxtrack->first->count)
  {
    map_position pos;
    pos.lat = gpxtrack->first->points[0].lat * 1000000;
    pos.lon = gpxtrack->first->points[0].lon * 1000000;
    nav_setpos(&pos, 0);
  }
  gpx_unlock();
  return 0;
}

//load track
void gui_track_load(void *data, Evas_Object *obj, const char *emission, const char *source)
{
  gui_track_hide();
  gui_fileopen_show(evas_track_parent, win_width, win_height, config->dir_tracks, "*.gpx", gui_track_callback);
}

//delete track
void gui_track_delete(void *data, Evas_Object *obj, const char *emission, const char *source)
{
  gpx_lock();
  if (gpxtrack)
  {
    gpx_free(gpxtrack);
    gpxtrack = NULL;
  }
  gpx_unlock();
  gui_track_hide();
}

//track show
void gui_track_show(Evas *evas, int x, int y, int width, int height)
{
  if (edje_track_visible)
  {
    gui_track_hide();
    return;
  }
  if (edje_track == NULL)
  {
    char buf[255];
    edje_track = edje_object_add(evas);
    snprintf(buf, sizeof(buf), "%s/themes/Default/main.edj", config->datadir);
    edje_object_file_set(edje_track, buf, "menu_track");
    evas_object_move(edje_track, x, y);
    evas_object_resize(edje_track, width, height);
    edje_track_w = width;
    edje_track_h = height;
    edje_object_signal_callback_add(edje_track, "delete", "track", gui_track_delete, NULL);
    edje_object_signal_callback_add(edje_track, "load", "track", gui_track_load, NULL);
    edje_object_signal_callback_add(edje_track, "save", "track", gui_track_log, NULL);
    evas_track_parent = evas;
  }
  if (config->gpx_log)
    edje_object_signal_emit(edje_track, "tracksave,state,on", "mcnavi");
  else
    edje_object_signal_emit(edje_track, "tracksave,state,off", "mcnavi");
  evas_object_show(edje_track);

  edje_track_visible = 1;
}

void gui_track_resize(int x, int y, int width, int height)
{
  if (edje_track)
  {
    evas_object_move(edje_track, x, y);
    evas_object_resize(edje_track, width, height);
    edje_track_w = width;
    edje_track_h = height;
  }
}

void gui_track_hide(void)
{
  evas_object_hide(edje_track);
  edje_track_visible = 0;
}

void gui_track_free(void)
{
  if (edje_track != NULL)
  {
    evas_object_del(edje_track);
    edje_track = NULL;
  }
}
