/* -*- Mode: C; indent-tabs-mode: s; c-basic-offset: 2; tab-width: 2 -*- */
/*
 * map_render.h
 * Copyright (C) Mike Crash 2009 <mike@mikecrash.com>
 * 
 * main.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * main.c is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _MAPRENDER_H
#define _MAPRENDER_H

#include "map_styles.h"
#include "mcthread.h"


typedef struct
{
  //image properties
  int width;
  int height;
  int pixsize;      //size of pixel in bytes
  int rowsize;      //size of row in bytes
  int x;            //center point for lat/lon
  int y;
  long lat;         //position of center point
  long lon;
  int angle;        //rotation of map in degrees (direction of driving)
  int zoom;         //zoom level, 1<<zoom gives pixel size in milimeters, range 0-mapzooms
  int anchor;       //flag if to draw the anchor

  //map style
  map_style *style;

  //image
  void *image1;      //we have two images, one for display, one for rendering
  void *image2;

  //threading
  mcthread thread;
  mcmutex mutex;
  mcsema sema;
  int terminate;

  //this variables needs to me mutexed
  int redraw;
  int rendered;      //new image rendered
  int select;        //what image to display, 0-image1, 1-image2
  int newwidth, newheight;
  int newlat, newlon, newangle;
  int newx, newy;
  int newzoom;
  int newroute;
} mr_image;

int mr_create(mr_image *img);
int mr_resize(mr_image *img, int width, int height, int x, int y);
int mr_free(mr_image *img);
int mr_render(mr_image *img, map_position *pos, int x, int y, int angle, int zoom, int route);

int mr_rendered(mr_image *img);
int *mr_getdata(mr_image *img, int *width, int *height);

#endif
