/* -*- Mode: C; indent-tabs-mode: s; c-basic-offset: 2; tab-width: 2 -*- */
/*
 * mcconf.c
 * Copyright (C) Mike Crash 2009 <mike@mikecrash.com>
 * 
 * main.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * main.c is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "functions.h"
#include "mcconf.h"
#include "map_common.h"

char defaultmap[] = "%s/mcnavi/maps/map.mcm";

mcconf *config = NULL;

char *usehomedir(char *s)
{
  char *ret;
  char *h;
  if ((h = strchr(s, '~')) == NULL)
  {
    ret = (char *)malloc(strlen(s)+1);
    strcpy(ret, s);
  }
  else
  {
    char *homedir=getenv("HOME");
    ret = (char *)malloc(strlen(s)+strlen(homedir)+1);
    strcpy(ret, s);
    char *h2 = strchr(ret, '~');
    strcpy(h2, homedir);
    h  += 1;
    h2 += strlen(homedir);
    strcpy(h2, h);
  }
  return ret;
}

void conf_load(char *filename)
{
  char s[500];
  FILE *f;

  if ((f = fopen(filename, "r"))==NULL)
    return;

  while (!feof(f))
  {
    if (fgets(s, sizeof(s), f) == NULL)
      break;
    if (s[0] == 0)
      continue;
    if (s[0] == '#')
      continue;

    //get cmd and value
    char *cmd, *value, *strip;
    cmd = s;
    value = strchr(s, '=');
    if (value == NULL)
      continue;
    *value = 0;
    value++;
    strip = strchr(value, '\n');
    if (strip)
      *strip = 0;
    strip = strchr(value, '\r');
    if (strip)
      *strip = 0;
    /*cmd = strtok(s, "= \n\r");
    if (cmd == NULL)
      continue;
    value = strtok(NULL, "= \n\r");*/

    //home position
    if (strcmp(cmd, "home_lat") == 0)
    {
      float f = atof(value);
      config->home.lat = f * 1000000.0;
    }
    if (strcmp(cmd, "home_lon") == 0)
    {
      double f = atof(value);
      config->home.lon = f * 1000000.0;
    }

    //home position
    if (strcmp(cmd, "route_sector") == 0)
    {
      float f = atof(value);
      config->route_sector = atoi(value) ? 1 : 0;
    }

    //map cache limits
    if (strcmp(cmd, "limit_nodes") == 0)
    {
      config->limit_nodes = strlong(value);
    }
    if (strcmp(cmd, "limit_text") == 0)
    {
      config->limit_text = strlong(value);
    }
    if (strcmp(cmd, "limit_planes") == 0)
    {
      config->limit_planes = strlong(value) / sizeof(map_position);
    }
    if (strcmp(cmd, "limit_endnodes") == 0)
    {
      config->limit_endnodes = strlong(value);
    }
    if (strcmp(cmd, "limit_visited") == 0)
    {
      config->limit_visited = strlong(value);
    }

    //track logging to gpx
    if (strcmp(cmd, "gpx_decimate") == 0)
    {
      config->gpx_decimate = strlong(value);
    }
    if (strcmp(cmd, "gpx_distance") == 0)
    {
      config->gpx_distance = strlong(value);
    }
    if (strcmp(cmd, "gpx_angle") == 0)
    {
      config->gpx_angle = strlong(value);
    }
    if (strcmp(cmd, "gpx_rise") == 0)
    {
      config->gpx_rise = strlong(value);
    }

    //directories
    if (strcmp(cmd, "dir_tracks") == 0)
    {
      if (config->dir_tracks)
        free(config->dir_tracks);
      config->dir_tracks = usehomedir(value);
    }
    if (strcmp(cmd, "dir_routes") == 0)
    {
      if (config->dir_routes)
        free(config->dir_routes);
      config->dir_routes = usehomedir(value);
    }

    //sounds
    if (strcmp(cmd, "sounds_cmd") == 0)
    {
      if (config->sounds_cmd)
        free(config->sounds_cmd);
      config->sounds_cmd = (char*)malloc(strlen(value)+1);
      strcpy(config->sounds_cmd, value);
    }
    if (strcmp(cmd, "sounds_style") == 0)
    {
      if (config->sounds_style)
        free(config->sounds_style);
      config->sounds_style = (char*)malloc(strlen(value)+1);
      strcpy(config->sounds_style, value);
    }
  }
  fclose(f);
}

int conf_init(int argc, char **argv)
{
  config = (mcconf*)malloc(sizeof(mcconf));
  memset(config, 0, sizeof(mcconf));
  
  config->datadir = (char*)malloc(strlen(PACKAGE_DATA_DIR)+8);
  sprintf(config->datadir, "%s/mcnavi", PACKAGE_DATA_DIR);

  //parse arguments
  int i = 1;
  while (i<argc)
  {
    if (strcmp(argv[i], "-m") == 0)
    {
      i++;
      if ((i<argc) && (config->map == NULL))
      {
        config->map = (char*)malloc(strlen(argv[i])+1);
        strcpy(config->map, argv[i]);
      }
    }
    if (strcmp(argv[i], "-h") == 0)
    {
      printf("mcnavi version %s\n", "0.2.2");
      printf("Usage: mcnavi [-m mapfile]\n");
      return 1;
    }
    i++;
  }

  if (config->map == NULL)
  {
    config->map = (char*)malloc(strlen(defaultmap)+strlen(PACKAGE_DATA_DIR)-1);
    sprintf(config->map, defaultmap, PACKAGE_DATA_DIR);
  }

  //load default values
  config->home.lat = 50073744;
  config->home.lon = 14857791;

  config->limit_nodes = 5000000;
  config->limit_planes = 3000000;
  config->limit_pois = 1000000;
  config->limit_locs = 30000;
  config->limit_text = 100000;
  config->limit_endnodes = 150;
  config->limit_visited = 10000;

  config->gpx_log = 0;
  config->gpx_decimate = 1;
  config->gpx_distance = 1500;
  config->gpx_angle = 5000;
  config->gpx_rise = 5000;

  config->route_vehicle = RV_CAR;
  config->route_mode = routemode_toll;
  config->route_add = routeadd_destination;
  config->route_sector = 0;

  char *homedir=getenv("HOME");
  if (homedir)
  {
    config->dir_tracks = (char *)malloc(strlen(homedir)+1);
    strcpy(config->dir_tracks, homedir);
    config->dir_routes = (char *)malloc(strlen(homedir)+1);
    strcpy(config->dir_routes, homedir);
  }

  //load default configuration file
  char s[255];
  snprintf(s, sizeof(s), "%s/mcnavi.conf", config->datadir);
  conf_load(s);

  //load user configuration file
  snprintf(s, sizeof(s), "%s/.mcnavi/mcnavi.conf", homedir);
  conf_load(s);
  return 0;
}

int conf_deinit(void)
{
  if (config == NULL)
    return -1;
  if (config->datadir)
    free(config->datadir);
  if (config->dir_tracks)
    free(config->dir_tracks);
  if (config->dir_routes)
    free(config->dir_routes);
  free(config);
  config = NULL;
  return 0;
}
