/* -*- Mode: C; indent-tabs-mode: s; c-basic-offset: 2; tab-width: 2 -*- */
/*
 * gps_gpsd.c
 * Copyright (C) Mike Crash 2009 <mike@mikecrash.com>
 * 
 * main.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * main.c is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "projection.h"
#include "mctime.h"
#include "mgps.h"
#include "mcthread.h"
#include "mcconf.h"
#include "gpx.h"

#if GPSD_API_MAJOR_VERSION == 4
#define GPSD_API_19 1
#endif

mgpsdata *mgps;

int mgps_lock(mgpsdata *mgps)
{
  return mcmutex_lock(&mgps->mutex);
}

int mgps_unlock(mgpsdata *mgps)
{
  return mcmutex_unlock(&mgps->mutex);
}

#if GPSD_API_MAJOR_VERSION >= 5
static void mgps_process(struct gps_data_t *gpsdata)
#elif GPSD_API_MAJOR_VERSION == 4
static void mgps_process(struct gps_data_t *gpsdata, char *buf UNUSED, size_t len UNUSED)
#else
static void mgps_process(struct gps_data_t *gpsdata, char *buf UNUSED, size_t len UNUSED, int value UNUSED)
#endif
{
  //printf("GPS %f - %d - %d\n", mgps->data->online, mgps->data->status, mgps->data->satellites_visible);
  if (mgps->data->fix.time!=0)
  {
    mgps->fix = mgps->data->fix.mode;
    if ((mgps->data->status!=0) && (mgps->data->fix.mode > 1))
    {
      mgps_lock(mgps);
      if (config->chrono_ena && !((mgps->lat==0) && (mgps->lon==0)))
      {
        config->chrono_len += wgs84_distance(mgps->lat, mgps->lon, mgps->data->fix.latitude*1000000, mgps->data->fix.longitude*1000000);
      }
      timer_start(&mgps->time);
      mgps->lat = mgps->data->fix.latitude*1000000.0;
      mgps->lon = mgps->data->fix.longitude*1000000.0;
      mgps->alt = mgps->data->fix.altitude;
      mgps->speed = mgps->data->fix.speed * 3.6;
      mgps->angle = mgps->data->fix.track>180 ? mgps->data->fix.track-360 : mgps->data->fix.track;
#if GPSD_API_MAJOR_VERSION >= 4
      mgps->eph = mgps->data->fix.epx > mgps->data->fix.epy ? mgps->data->fix.epx : mgps->data->fix.epy;
  #else
      mgps->eph = mgps->data->fix.eph;
  #endif
      mgps->epv = mgps->data->fix.epv;
      mgps->online = 1;
      mgps_unlock(mgps);
      if (config->gpx_log)
      {
        gpx_point gp;
        //printf("G %f\n", mgps->data->fix.time);
        gp.time = mgps->data->fix.time;
        gp.lat = mgps->data->fix.latitude;
        gp.lon = mgps->data->fix.longitude;
        gp.altitude = mgps->data->fix.altitude;
        gp.speed = mgps->data->fix.speed;
        gp.course = mgps->data->fix.track;
  #if GPSD_API_MAJOR_VERSION >= 4
        gp.eph = mgps->data->fix.epx > mgps->data->fix.epy ? mgps->data->fix.epx : mgps->data->fix.epy;
  #else
        gp.eph = mgps->data->fix.eph;
  #endif
        gp.epv = mgps->data->fix.epv;
        gpx_logpoint(&gp);
      }
    }
    else
    {
      mgps_lock(mgps);
      if (mgps->online && config->gpx_log)
      {
        gpx_log_split();
      }
      mgps->online = 0;
      mgps_unlock(mgps);
    }
  }

  //satellites
#if GPSD_API_MAJOR_VERSION >= 4
  if (mgps->data->satellites_visible>0)
#else
  if (mgps->data->satellites>0)
#endif
  {
    mgps_lock(mgps);
    int i,j;
#if GPSD_API_MAJOR_VERSION >= 4
    j = mgps->data->satellites_visible > maxsats ? maxsats : mgps->data->satellites_visible;
#else
    j = mgps->data->satellites > maxsats ? maxsats : mgps->data->satellites;
#endif
    mgps->sats_num = j;
    for (i=0; i<j; i++)
    {
      mgps->sats[i].PRN = mgps->data->PRN[i];
      mgps->sats[i].elevation = mgps->data->elevation[i];
      mgps->sats[i].azimuth = mgps->data->azimuth[i];
      mgps->sats[i].SNR = mgps->data->ss[i];
      mgps->sats[i].used = 0;
#if GPSD_API_MAJOR_VERSION >= 4
      int k;
      for (k=0; k<mgps->data->satellites_used; k++)
      {
        if (mgps->sats[i].PRN == mgps->data->used[k])
          mgps->sats[i].used = 1;
      }
#else
      if (mgps->data->satellites_used != 0)
        mgps->sats[i].used = mgps->data->used[i] ? 1 : 0;
#endif
    }
    mgps_unlock(mgps);
  }
  else
  {
    mgps_lock(mgps);
    mgps->sats_num = 0;
    mgps_unlock(mgps);
  }
}


void *mgps_thread(void *p)
{
  mgps = (mgpsdata*)p;
  fd_set fds;
  int data;
  struct timeval tv;

#if GPSD_API_MAJOR_VERSION <= 4
  gps_set_raw_hook(mgps->data, mgps_process);
#endif
#if GPSD_API_MAJOR_VERSION >= 4
  if (gps_stream(mgps->data, WATCH_ENABLE | WATCH_NEWSTYLE, NULL) != 0)
    return NULL;
#else
  if (gps_query(mgps->data, "w+x\n") != 0)
    return NULL;
#endif
  
  while (mgps->terminate==0)
  {
#if GPSD_API_MAJOR_VERSION >= 5
    if (!gps_waiting(mgps->data, 5000000))
	    break;
    if (gps_read(mgps->data) == -1)
	    break;
    else
    {
      mgps_process(mgps->data);
    }
#else
    FD_ZERO(&fds);
    FD_SET(mgps->data->gps_fd, &fds);
    tv.tv_usec = 100000;
    tv.tv_sec = 0;
    data = select(mgps->data->gps_fd + 1, &fds, NULL, NULL, &tv);

    if (data)
    {
      gps_poll(mgps->data);
    }
#endif
  }
  printf("GPS end\n");
  return NULL;
}

mgpsdata *mgps_init(void)
{
  mgpsdata *mgps;
  mgps = (mgpsdata*)malloc(sizeof(mgpsdata));
  memset(mgps, 0, sizeof(mgpsdata));
  mcmutex_init(&mgps->mutex);

#if GPSD_API_MAJOR_VERSION >= 5
  mgps->data = (gps_data_t*)malloc(sizeof(gps_data_t));
  if ((gps_open("127.0.0.1", DEFAULT_GPSD_PORT, mgps->data)) != 0)
#else
  if ((mgps->data = gps_open("127.0.0.1", DEFAULT_GPSD_PORT)) == NULL)
#endif
  {
    printf("Can't connect to gpsd\n");
#if GPSD_API_MAJOR_VERSION >= 5
    free(mgps->data);
#endif
    free(mgps);
    return NULL;
  }
  mcthread_create(&mgps->thread, mgps_thread, mgps, 0);

  return mgps;
}

int mgps_deinit(mgpsdata *mgps)
{
  if (mgps == NULL)
    return -1;
  mgps->terminate = 1;
  mcthread_join(&mgps->thread);
  gps_close(mgps->data);
  //printf("GPS: %p %p %lx\n", mgps, mgps->data, mgps->thread);
  mcmutex_destroy(&mgps->mutex);
#if GPSD_API_MAJOR_VERSION >= 5
  free(mgps->data);
#endif
  free(mgps);
  printf("GPS gone\n");
  return 0;
}

long mgps_timer(mgpsdata *mgps)
{
  long time;
  mgps_lock(mgps);
  if (mgps->time.tv_sec==0)
    time = -1;
  else
    time = timer_get(&mgps->time);
  mgps_unlock(mgps);
  return time;
}
