/* -*- Mode: C; indent-tabs-mode: s; c-basic-offset: 2; tab-width: 2 -*- */
/*
 * gps_gpsd.h
 * Copyright (C) Mike Crash 2009 <mike@mikecrash.com>
 * 
 * main.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * main.c is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef _MGPS_H
#define _MGPS_H

#include <gps.h>
#include <pthread.h>
#include <sys/time.h> 
#include "mcthread.h"

#define maxsats 20

typedef struct
{
  int used;
  int PRN;
  int elevation;
  int azimuth;
  int SNR;
} mgpssat;

typedef struct
{
  //gps data
  struct timeval time;
  int fix;        //0 - not seen, 1 - no fix, 2 - 2D, 3 - 3D
  long lat;       //latitude
  long lon;       //longitude
  long alt;       //altitude in meters
  long speed;     //speed *10
  long angle;     //angle course
  long eph;       //horizontal error
  long epv;       //vertical error
  int online;     //signal present

  //satellites
  int sats_num;   //satellites in next structure
  mgpssat sats[maxsats];

  //internal
  int terminate;
  mcmutex mutex;
  pthread_t thread;
  struct gps_data_t *data;
} mgpsdata;


int mgps_lock(mgpsdata *mgps);
int mgps_unlock(mgpsdata *mgps);

mgpsdata *mgps_init(void);
int mgps_deinit(mgpsdata *mgps);
long mgps_timer(mgpsdata *mgps);


#endif
