/* -*- Mode: C; indent-tabs-mode: s; c-basic-offset: 2; tab-width: 2 -*- */
/*
 * route.h
 * Copyright (C) Mike Crash 2009 <mike@mikecrash.com>
 * 
 * main.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * main.c is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef _NAVIGATION_H
#define _NAVIGATION_H

#include "route.h"
#include "map_render.h"
#include "mgps.h"


typedef struct
{
  mcthread thread;
  mcmutex mutex;
  mcsema sema;
  int terminate;

  //map
  mr_image map_img;
  int map_tracking;
  int map_tracking_disable;
  struct timeval map_dragtime;
  map_position map_pos;
  int map_angle;
  int map_zoom;
  int map_width;
  int map_height;
  int map_x;
  int map_y;

  //routing
  int route;                        //indicator if we should begin routing
  routing_add route_add;            //destination or waypoint
  int route_delete;
  char *route_name;
  route_progress route_prgs;        //progress indicator
  map_position route_pos;           //new destination/waypoint position

  //navigation
  route_dir route_nextturn;
  long route_toturn;
  long route_toturn_last;
  long route_togo;
  long route_totake;
  int route_lastsound;
  maptypes minway, maxway;
  
  //gps
  mgpsdata *gps;
  route_node *routepos;

} navdata;

extern route_route *route;
extern navdata *nav;

int nav_init();
int nav_deinit();

int nav_lock();
int nav_unlock();

int nav_route(map_position *pos, routing_add add, char *name);
int nav_delete(int itinerary);
int nav_stop();
int nav_save(char *filename);
int nav_load(char *filename);

int nav_progress(route_progress *progress);
int nav_getpos(map_position *pos, int *angle);
int nav_setpos(map_position *pos, int angle);
int nav_getzoom(void);
int nav_setzoom(int zoom);
int nav_setsize(int width, int height);
int nav_getanchor(int *x, int *y);
int nav_gpsinfo(int *fix, long *lat, long *lon, int *speed, int *alt, int *angle, int *eph, int *epv);
int nav_satinfo(int *sats_num, mgpssat **sats);  //call nav_unlock after use
int nav_routeinfo(route_dir *nextturn, long *toturn, long *togo, long *timerest);
int nav_tracking(int tracking);
int nav_setvehicle(routing_vehicle vehicle);

#endif
