/* -*- Mode: C; indent-tabs-mode: s; c-basic-offset: 2; tab-width: 2 -*- */
/*
 * route.h
 * Copyright (C) Mike Crash 2009 <mike@mikecrash.com>
 * 
 * main.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * main.c is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef _ROUTE_H
#define _ROUTE_H

#include "map_read.h"
#include "route_common.h"

typedef struct
{
  int progress;                    //progress in percents
  int nodes;                       //nodes passed
  long length;                     //distance
} route_progress;

typedef struct
{
  int count;
  map_position pos[];
} route_directions;

typedef enum 
{ 
  routeadd_direction,
  routeadd_waypoint,
  routeadd_destination
} routing_add;

//itinerary
typedef struct route_itinerary
{
  route_nodetype nodetype;
  char *name;
  map_position pos;
  route_node *node;
  struct route_itinerary *next;
  struct route_itinerary *prev;
} route_itinerary;

//route
typedef struct
{
  routing_vehicle vehicle;
  route_itinerary *itinerary;
  route_node *route;
} route_route;

//common
route_node *route_lastnode(route_node *route);

//routing
routingresult route_find_pos2pos(map_position *source, map_position *dest, route_directions *directions, int angle, routing_vehicle route_vehicle, routing_mode route_mode, route_node **route, route_node *prevroute, route_progress *progress);
routingresult route_find_node2pos(route_node *source, map_position *dest, route_directions *directions, int angle, routing_vehicle vehicle, routing_mode route_mode, route_node **route, route_node *prevroute, route_progress *progress);
routingresult route_find_pos2node( map_position *source, route_node *dest, route_directions *directions, int angle, routing_vehicle vehicle, routing_mode route_mode, route_node **route, route_node *prevroute, route_progress *progress);
routingresult route_find_node2node( route_node *source, route_node *dest, route_directions *directions, int angle, routing_vehicle vehicle, routing_mode route_mode, route_node **route, route_node *prevroute, route_progress *progress);
route_node *route_combine(routing_vehicle vehicle, route_node *route, route_node *pass, route_node *node);
route_node *route_append(route_node *route, route_node *append);
int route_maketurn(routing_vehicle vehicle, route_node *route, route_node *end);

//control
int route_stop();
int route_lock();
int route_unlock();
route_route *route_init();
int route_deinit(route_route *route);

//itinerary
route_itinerary *route_itinerary_new();
void route_itinerary_free(route_itinerary *itinerary);
route_itinerary *route_lastitinerary(route_itinerary *itinerary);
route_directions *route_createdirections(route_itinerary *ri1, route_itinerary *ri2);


#endif
