/* -*- Mode: C; indent-tabs-mode: s; c-basic-offset: 2; tab-width: 2 -*- */
/*
 * route_common.h
 * Copyright (C) Mike Crash 2011 <mike@mikecrash.com>
 * 
 * main.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * main.c is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _ROUTECOMMON_H
#define _ROUTECOMMON_H

#include "map_read.h"

#define visited_max 1000

typedef enum
{
  RTS_ROUTING,       //we still routing
  RTS_DEST,          //we have reached destination
  RTS_ROUTE,         //we have reached the route
  RTS_ERROR=-1       //error while routing
} routingresult;

typedef enum
{
  RV_TRAIN,
  RV_TRUCK,
  RV_CAR,
  RV_CYCLE,
  RV_MTB,
  RV_FOOT
} routing_vehicle;

typedef enum 
{ 
  routemode_length,
  routemode_speed,
  routemode_toll
} routing_mode;

typedef enum
{
  nt_node,
  nt_start,
  nt_direction,
  nt_destination,
  nt_waypoint,
  nt_goal
} route_nodetype;

typedef enum
{
  dir_none,
  dir_straight,
  dir_r_uturn,
  dir_l_uturn,
  dir_hardleft, 
  dir_left,
  dir_softleft,
  dir_softright,
  dir_right,
  dir_hardright,
  dir_rr_straight,
  dir_rr_uturn,
  dir_rr_hardleft,
  dir_rr_left,
  dir_rr_softleft,
  dir_rr_softright,
  dir_rr_right,
  dir_rr_hardright,
  dir_rl_straight,
  dir_rl_uturn,
  dir_rl_hardleft,
  dir_rl_left,
  dir_rl_softleft,
  dir_rl_softright,
  dir_rl_right,
  dir_rl_hardright,
  dir_direction,
  dir_waypoint,
  dir_finish
} route_dir;

#define cntroutedir (dir_finish+1)

//visited
typedef struct
{
  long fromid;
  long toid;
} route_turn;

typedef struct route_visited_block
{
  int count;
  route_turn turn[visited_max];
  struct route_visited_block *next;
} route_visited_block;

typedef struct route_visited
{
  int count;
  route_visited_block *blocks;
} route_visited;

//route node
typedef struct route_node
{
  long id;                         //id of this node
  route_nodetype nodetype;         //type of this node
  int reached;                     //if we reached the node, used for rerouting to waypoints and destinations
  int cross;                       //total number of roads from this node
  map_position pos;                //position of node
  maptypes waytype;                //way type from previous node
  long length;                     //length rided from source
  int speed;                       //speed allowed from previous node
  unsigned int flags;              //way flags - direction, oneway, ...
  route_dir direction;             //actual direction to turn
  long toturn;                     //distance to next turn other then dir_front
  long togo;                       //distance to destination
  long totake;                     //time to reach destination in 0.1 seconds
  long cost_src;                   //cost of way rided from source - this is length or time
  long cost;                       //total cost - sum of cost_src and cost to destination
  struct route_node *prev_node;    //previous node we ride
  struct route_node *next_node;    //next node to ride
  struct route_node *prev_list;    //next node in the same level
  struct route_node *next_list;    //next node in the same level
} route_node;

//end nodes
typedef struct route_endnode
{
  route_node *node;
  struct route_endnode *next;
  struct route_endnode *prev;
} route_endnode;

typedef struct
{
  int count;
  route_endnode *first;         //list of end nodes sorted by cost
  route_endnode *last;          //last end node
} route_endnodes;

//common
int vehiclespeed(routing_vehicle vehicle, maptypes waytype, int *speed);
maptypes way_minallowed(routing_vehicle routetype);
maptypes way_maxallowed(routing_vehicle routetype);
int way_allowed(routing_vehicle vehicle, maptypes waytype);
int aerospeed(routing_vehicle vehicle);
long cost(routing_vehicle vehicle, routing_mode mode, maptypes waytype, int length, int speed);

//nodes and routes
route_node *new_node(void);
void free_node(route_node *node);
void clear_node(route_node *node);
int route_freeroute(route_node *route);

//visited
void add_visited(route_visited *visited, long fromid, long toid);
int is_visited(route_visited *visited, long fromid, long toid);
int clear_visited(route_visited *visited);

//end nodes
void add_endnode(route_endnodes *endnodes, route_node *node);
void remove_endnode(route_endnodes *endnodes, route_node *node);

//other
void printf_nodeinfo(route_node *node);


#endif
