/* -*- Mode: C; indent-tabs-mode: s; c-basic-offset: 2; tab-width: 2 -*- */
/*
 * route_sector.cc
 * Copyright (C) Mike Crash 2011 <mike@mikecrash.com>
 * 
 * main.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * main.c is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "route_sector.h"
#include "projection.h"
#include "mctime.h"
#include "mcconf.h"

//TODO
//problem: zakazano odboceni na krajnim node?

//boundary node for sector with routes to other boundary nodes
typedef struct
{
  long id;               //OSM ID
  map_position pos;      //position
  int crosscount;
  routecross *cross;     //cross
  int routescount;       //count of routes
  route_node *routes[];  //array of all routes from node to node
} route_sector_node;

//list of all routes between border nodes
typedef struct route_sector
{
  struct route_sector *next;
  struct route_sector *prev;
  long lat;                     //sector coordinates modulo MAPSECTRECT
  long lon;
  int size;
  int count;                    //count of all border nodes
  route_sector_node *nodes[];   //array of all border nodes
} route_sector;

typedef struct
{
  routing_vehicle vehicle;
  routing_mode mode;
  int count;
  route_sector *first;
  route_sector *last;
} route_sectors;

typedef struct
{
  node_sector *sector;             //sector for routing
  route_node *start;               //start node
  route_node *goal;                //goal end node
  route_endnodes endnodes;         //list of end nodes sorted by cost
  route_visited visited;           //list of visited node turns
  routing_vehicle vehicle;
  routing_mode mode;
  int nodes;                       //nodes analyzed
  routingresult status;
} route_sector_data;

route_sectors *routesectors = NULL;

int route_sector_step(route_sector_data *routing, route_node *node)
{
  int i;

  if (node==NULL)
  {
    routing->status = RTS_ERROR;
    return -1;
  }

  //printf("Node %ld OSM %10ld Way %ld Cost %ld\n", node->id, node->osmid, node->wayid, node->cost);
  remove_endnode(&routing->endnodes, node);

  //find all ways from this node
  mcross *cross;
  int crosscount;
  if (map_nodecross(node->id, routing->sector, &crosscount, &cross)<0)
  {
    clear_node(node);
    //printf("ERROR: cross for node %ld not found\n", node->id);
    return 0;
  }

  node->cross = crosscount;
  node->direction = dir_none;
  //if (node->id == 2851673)
  //  printf("Stop\n");

  //add as visited
  //add_visited(&routing->visited, node->prev_node->wayid, node->wayid);
  if (node->prev_node)
  {
    add_visited(&routing->visited, node->prev_node->id, node->id);
  }

  //create nodes for it and update the end list
  route_node *node3 = NULL;
  for (i=0; i<crosscount; i++)
  {
    route_node *node2;
    map_position endpos;
    int length;
    int speed = cross[i].speed;
    int waydir = cross[i].flags & WF_DIR;
    unsigned int flags = cross[i].flags;
    maptypes waytype = cross[i].waytype;

    //do not go back
    //TODO allow turn back on crossroads without semaphore
    if ((node->prev_node) && (cross[i].id == node->prev_node->id))
    {
      continue;
    }
    
    //disallow forbidden routing
    if ((routing->vehicle!=RV_FOOT) && (flags & WF_ONEWAY) && waydir)
      continue;
    if ((routing->vehicle==RV_CAR) && (flags & WF_NOMOTORCAR))
      continue;
    if ((routing->vehicle==RV_TRUCK) && (flags & WF_NOTRUCKS))
      continue;
    if ((routing->vehicle==RV_CYCLE) && (flags & WF_NOBICYCLE))
      continue;
    if ((routing->vehicle<=RV_FOOT) && (flags & WF_NOFOOT))
      continue;
    if ((flags & WF_TOLL) && (routing->mode!=routemode_toll) && ((routing->vehicle==RV_TRUCK)||(routing->vehicle==RV_CAR)))
    //if (((waytype == WT_MOTORWAY) || (waytype == WT_TRUNK)) && (routing->mode!=routemode_toll) && ((routing->vehicle==RV_TRUCK)||(routing->vehicle==RV_CAR)))
      continue;
    if (way_allowed(routing->vehicle, waytype) == 0)
      continue;
    if (((routing->vehicle==RV_TRUCK)||(routing->vehicle==RV_CAR)) && (node->length>3000) && (node->togo>3000) && (waytype>WT_TERTIARY))
      continue;
    if (map_nodeposition(cross[i].id, routing->sector, &endpos)!=0)
      continue;

    length = wgs84_distance(node->pos.lat, node->pos.lon, endpos.lat, endpos.lon);
    //printf("C %i %ld %ld (%ld,%ld) -> %ld (%ld,%ld)\n", i, cross->segments[i], startnode, startpos.lat, startpos.lon, endnode, endpos.lat, endpos.lon);

    //check if we have found the destination, update data, link it and exit
    if (routing->goal->id == cross[i].id)
    {
      //printf("We have reached goal node! (%ld)\n", cross[i].node);

      //update goal node
      routing->goal->cost_src = node->cost_src + cost(routing->vehicle, routing->mode, routing->goal->waytype, length, speed);
      routing->goal->cost= routing->goal->cost_src;
      routing->goal->length = node->length + length;
      routing->goal->prev_node = node;
      routing->goal->speed = speed;
      routing->goal->waytype = waytype;
      if (node->next_node==NULL)
      {
        node->next_node = routing->goal;
      }
      else
      {
        node3->next_list = routing->goal;
      }
      routing->nodes++;
      printf_nodeinfo(routing->goal);
      routing->status = RTS_DEST;
      break;
    }

    //route to next node
    //if (is_visited(routing, node->wayid, cross->segments[i]) == 0)
    if (is_visited(&routing->visited, node->id, cross[i].id) == 0)
    {
      node2 = new_node();
      node2->id = cross[i].id;
      node2->pos = endpos;
      node2->cross = 0;
      node2->direction = dir_none;
      
      node2->length = node->length + length;
      node2->speed = speed;
      node2->waytype = waytype;
      node2->cost_src = node->cost_src + cost(routing->vehicle, routing->mode, waytype, length, speed);
      node2->togo = wgs84_distance(node2->pos.lat, node2->pos.lon, routing->goal->pos.lat, routing->goal->pos.lon);
      node2->cost = node2->cost_src + cost(routing->vehicle, routing->mode, (maptypes)-1, node2->togo, aerospeed(routing->vehicle))/* * dirindexlen / dirindex*/;
      node2->prev_node = node;
      if (node->next_node==NULL)
      {
        node->next_node = node2;
      }
      else
      {
        node3->next_list = node2;
        node2->prev_list = node3;
      }

      add_endnode(&routing->endnodes, node2);
      node3 = node2;
      routing->nodes++;
      printf_nodeinfo(node2);
    }
  }
  map_nodeunlock(node->id, &node->pos);

  //if there is no way, this is closed street, so clear the current path
  if (node->next_node==NULL)
  {
    remove_endnode(&routing->endnodes, node);
    clear_node(node);
  }

  //may be we can simply free very far end nodes to save memory
  while (routing->endnodes.count > config->limit_endnodes)
  {
    route_endnode *curendnode = routing->endnodes.last;
    curendnode->prev->next = NULL;
    routing->endnodes.last = curendnode->prev;
    clear_node(curendnode->node);
    free(curendnode);
    routing->endnodes.count--;
  }

  return 0;
}

route_node *route_sector_node2node(node_sector *sector, routing_vehicle vehicle, routing_mode route_mode, long start, long goal)
{
  route_sector_data routing;
  route_node *node;

  //init
  memset(&routing, 0, sizeof(routing));
  routing.sector = sector;
  routing.vehicle = vehicle;
  routing.mode = route_mode;

  //start node
  node = new_node();
  node->id = start;
  if (map_nodeposition(start, sector, &node->pos) != 0)
  {
    free(node);
    return NULL;
  }
  node->nodetype = nt_start;
  routing.start = node;
  add_endnode(&routing.endnodes, node);

  //goal node
  node = new_node();
  node->id = goal;
  if (map_nodeposition(goal, sector, &node->pos) != 0)
  {
    free(routing.start);
    free(node);
    return NULL;
  }
  node->nodetype = nt_goal;
  routing.goal = node;

  //let's go
  routing.status = RTS_ROUTING;
  while ((routing.status==RTS_ROUTING) && (routing.endnodes.first!= NULL) && (route_sector_step(&routing, routing.endnodes.first->node) == 0))
  {
    
  }

  //free all unused nodes
  clear_visited(&routing.visited);
  while (routing.endnodes.first)
  {
    node = routing.endnodes.first->node;
    //printf_nodeinfo(node);
    remove_endnode(&routing.endnodes, routing.endnodes.first->node);
    clear_node(node);
  }

  //free goal if route not found
  if (routing.goal->prev_node == NULL)
  {
    free(routing.goal);
    routing.goal = NULL;
  }

  //print the result and return
  if (routing.status == RTS_DEST)
  {
    //update togo and totake information
    node = routing.goal;
    long togo = 0;
    long length = routing.goal->length;
    long totake = 0;
    int speed = node->speed;
    maptypes waytype = node->waytype;
    node->togo = togo;
    node->totake = totake;
    node = node->prev_node;
    while (node)
    {
      long d = length - node->length;
      vehiclespeed(vehicle, waytype, &speed);
      togo += d;
      node->togo = togo;
      totake += d * 36 / speed;
      node->totake = totake;
      length -= d;
      speed = node->speed;
      waytype = node->waytype;
      node = node->prev_node;
    }

    printf("Found the route from %ld to %ld: %ldm %ld:%02ld, %d nodes analyzed\n", routing.start->id, routing.goal->id, routing.start->togo, routing.start->totake / 600, routing.start->totake/10 % 60, routing.nodes);
/*    node = routing.start;
    while (node)
    {
      //printf("Node %10ld OSM %10ld Way %10ld Length %d Cost %ld Turn %d %ld\n", node->id, node->osmid, node->wayid, node->length, node->cost_src, node->direction, node->toturn);
      printf("Node %ld Len %ld Togo %ld Totake %ld Cost %ld/%ld Turn %d %ld\n", node->id, node->length, node->togo, node->totake, node->cost, node->cost_src, node->direction, node->toturn);
      if (node->next_list)
      {
        printf("NEXT LIST %ld !!!!!!!!!!!\n", node->next_list->id);
      }
      if (node->prev_list)
        printf("PREV LIST !!!!!!!!!!!\n");
      node = node->next_node;
    }*/
    return routing.start;
  }

  return NULL;
}

route_sector *route_sector_parse(node_sector *sector, routing_vehicle vehicle, routing_mode route_mode)
{
  int i, j;
  struct timeval timer;
  timer_start(&timer);

  //init
  if (sector == NULL)
    return NULL;

  //create list of nodes with cross nodes not in this sector
  int count = 0;
  mnode **nodes = (mnode **)malloc(sizeof(mnode *)*sector->count);
  
  for (i=0; i<sector->count; i++)
  {
    mnode *node1 = sector->nodes[i];
    for (j=0; j<node1->crosscount; j++)
    {
      long node2 = node1->cross[j].id;
      if ((node1->cross[j].waytype<=WT_TERTIARY) && (map_nodeinsector(node2, sector) == 0))
      {
        nodes[count] = node1;
        count++;
        break;
      }
    }
  }

  //allocate routes table
  //printf("%d - %d\n", count, sizeof(route_sector)+count*count*sizeof(route_node*));
  route_sector *rs = (route_sector *)malloc(sizeof(route_sector)+count*sizeof(route_sector_node*));
  rs->lat = MAPSECTOR(sector->lat);
  rs->lon = MAPSECTOR(sector->lon);
  rs->size = sizeof(route_sector)+count*sizeof(route_sector_node*);
  rs->count = count;

  //route each node to node
  for (i=0; i<count; i++)
  {
    route_sector_node *node = (route_sector_node *)malloc(sizeof(route_sector_node) + sizeof(route_node *) * (count-1));

    //copy cross for nodes, but only if routes to other sector
    //TODO optimize - do not call map_nodeposition twice
    int crosscount = 0;
    map_position pos;
    for (j=0; j<nodes[i]->crosscount; j++)
    {
      if (map_nodeposition(nodes[i]->cross[j].id, sector, &pos) != 0)
        crosscount++;
    }
    node->id = nodes[i]->id;
    node->pos = nodes[i]->pos;
    node->crosscount = nodes[i]->crosscount;
    node->cross = (routecross *)malloc(crosscount * sizeof(routecross));
    int c = 0;
    for (j=0; j<nodes[i]->crosscount; j++)
    {
      if (map_nodeposition(nodes[i]->cross[j].id, sector, &pos) != 0)
      {
        node->cross[c].id = nodes[i]->cross[j].id;
        node->cross[c].waytype = nodes[i]->cross[j].waytype;
        node->cross[c].speed = nodes[i]->cross[j].speed;
        node->cross[c].flags = nodes[i]->cross[j].flags;
        c++;
      }
    }
    rs->size += node->crosscount * sizeof(routecross);

    //route nodes
    for (j=0; j<count; j++)
    {
      if (i!=j)
      {
        int k = j<i ? j : j-1;
        node->routes[k] = route_sector_node2node(sector, vehicle, route_mode, nodes[i]->id, nodes[j]->id);
        rs->size += 0;  //TODO
      }
    }
    rs->nodes[i] = node;
  }
  timer_print(&timer);

  //deinit
  free(nodes);
  return rs;
}

route_sector *route_sector_get(long lat, long lon)
{
  lat = MAPSECTOR(lat);
  lon = MAPSECTOR(lon);
  printf("Route sector %ld,%ld\n", lat, lon);

  //check if sector is fetched
  route_sector *rsector = routesectors->first;
  while (rsector)
  {
    if ((rsector->lat == lat) && (rsector->lon == lon))
    {
      //bring to top and return
      if (rsector != routesectors->first)
      {
        if (routesectors->last == rsector)
          routesectors->last = rsector->prev;
        if (rsector->prev)
          rsector->prev->next = rsector->next;
        if (rsector->next)
          rsector->next->prev = rsector->prev;
        routesectors->first->prev = rsector;
        rsector->next = routesectors->first;
        rsector->prev = NULL;
        routesectors->first = rsector;
      }
      return rsector;
    }
    rsector = rsector->next;
  }

  //parse sector and find routes between border nodes
  node_sector *nsector = map_locknodesector(lat, lon);
  rsector = route_sector_parse(nsector, routesectors->vehicle, routesectors->mode);
  map_unlocknodesector(nsector);
  if (rsector)
  {
    if (routesectors->first)
     routesectors->first->prev = rsector;
    rsector->prev = NULL;
    rsector->next = routesectors->first;
    routesectors->first = rsector;
    if (routesectors->last == NULL)
      routesectors->last = rsector;
  }

  //limit size
  //TODO

  return rsector;
}

int route_sector_clean(route_sector *sector)
{
  while (sector)
  {
    for (int i=0; i<sector->count; i++)
    {
      route_sector_node *node = sector->nodes[i];
      if (node->cross)
        free(node->cross);
      for (int j=0; j<node->routescount; j++)
      {
        if (node->routes[j])
          route_freeroute(node->routes[j]);
      }
      free(node);
    }
  }
  return 0;
}

void route_sector_init(routing_vehicle vehicle, routing_mode mode)
{
  if (routesectors)
  {
    if ((routesectors->vehicle != vehicle) || (routesectors->mode != mode))
    {
      route_sector_clean(routesectors->first);
      routesectors->first = NULL;
    }
  }
  
  if (routesectors == NULL)
  {
    routesectors = (route_sectors *)malloc(sizeof(route_sectors));
  }

  memset(routesectors, 0, sizeof(route_sectors));
  routesectors->vehicle = vehicle;
  routesectors->mode = mode;
}

void route_sector_deinit()
{
  if (routesectors)
  {
    route_sector_clean(routesectors->first);
    free(routesectors);
    routesectors = NULL;
  }
}

//node position
int nodeposfromroute(long id, route_sector *sector, map_position *pos)
{
  for (int i=0; i<sector->count; i++)
  {
    if (sector->nodes[i]->id == id)
    {
      *pos = sector->nodes[i]->pos;
      return 0;
    }
  }
  return -1;
}

int route_nodeposition(long id, map_position *loc, map_position *pos)
{
  long lat = MAPSECTOR(loc->lat);
  long lon = MAPSECTOR(loc->lon);
  int i,j;
  route_sector *sector;

  for (i=-1; i<=1; i++)
  {
    for (j=-1; j<=1; j++)
    {
      sector = route_sector_get(lat+i*MAPSECTRECT, lon+j*MAPSECTRECT);
      if (sector && (nodeposfromroute(id, sector, pos)==0))
          return 0;
    }
  }
  return -1;
}

//get all cross to other sectors for node
int route_nodecross(long id, map_position *loc, int *count, routecross **cross)
{
  route_sector *sector = route_sector_get(loc->lat, loc->lon);
  if (sector == NULL)
    return -1;

  for (int i=0; i<sector->count; i++)
  {
    route_sector_node *node = sector->nodes[i];
    if (node->id == id)
    {
      *count = node->crosscount;
      *cross = node->cross;
      return 0;
    }
  }
  return -1;
}

//append sector routes to node
int route_sector_addroute(route_node *node, route_node *append, route_endnodes *endnodes)
{
  if ((node == NULL) || (append == NULL))
    return -1;
  if (node->id != append->id)
    return -1;
  append = append->next_node;
  route_node *first = node;

  while (append)
  {
    route_node *next = new_node();
    *next = *append;
    next->length += first->length;
    next->cost_src += first->cost_src;
    next->prev_node = node;
    if (node->next_node == NULL)
    {
      node->next_node = next;
    }
    else
    {
      node = node->next_node;
      while (node->next_list)
        node = node->next_list;
      node->next_list = next;
      next->prev_list = node;
    }
    if (append->next_node == NULL)
    {
      add_endnode(endnodes, next);
    }
    append = append->next_node;
    node = next;
  }
  return 0;
}

int route_sector_append(route_node *node, route_endnodes *endnodes)
{
  int ret = -1;

  //get node
  route_sector *sector = route_sector_get(node->pos.lat, node->pos.lon);
  if (sector == NULL)
    return -1;

  //find node
  route_sector_node *snode = NULL;
  for (int i=0; i<sector->count; i++)
  {
    if (sector->nodes[i]->id == node->id)
    {
      snode = sector->nodes[i];
      break;
    }
  }

  //append segments
  if (snode)
  {
    for (int i=0; i<snode->routescount; i++)
    {
      if (route_sector_addroute(node, snode->routes[i], endnodes) == 0)
        ret = 0;
    }
  }

  //at least one added segment to be successfull
  return ret;
}
