/* -*- Mode: C; indent-tabs-mode: s; c-basic-offset: 2; tab-width: 2 -*- */
/*
 * map_city.c
 * Copyright (C) Mike Crash 2009 <mike@mikecrash.com>
 * 
 * main.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * main.c is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdlib.h>
#include <string.h>
#include "typeio.h"
#include "diacritics.h"
#include "functions.h"
#include "map_common.h"
#include "map_text.h"
#include "map_city.h"

mapaddrsearch *map_city_start(char *name, long parent, long filepos)
{
  mapaddrsearch *as = (mapaddrsearch *)malloc(sizeof(mapaddrsearch)+strlen(name)+1);
  as->type = MAS_CITY;
  as->filepos = filepos;
  as->parent = parent;
  strcpy(as->name, name);
  as->index = 0;
  return as;
}

tcity *map_city_get(mapaddrsearch *as)
{
  tcity ttc;

  if (as->filepos == 0xffffffff)
    return NULL;

  do
  {
    map_lock(map);
    long filepos = map->info.cities + map_part_size + as->filepos;
    fseek(map->f, filepos, SEEK_SET);
    fread_long(&ttc.id, 1, map->f);
    //printf("City %lx %lx\n", map->info.cities, filepos);
    if (ttc.id != 0)
    {
      fread_long(&ttc.country, 1, map->f);
      //printf("Country %ld\n", ttc.country);
      if (ttc.country != as->parent)
      {
        map_unlock(map);
        break;
      }
      fread_long(&ttc.pos.lat, 1, map->f);
      fread_long(&ttc.pos.lon, 1, map->f);
      fread_long(&ttc.namefilepos, 1, map->f);
      fread_long(&ttc.streetsfilepos, 1, map->f);
      map_unlock(map);
      if (ttc.namefilepos == -1)
      {
        break;
      }
      char *name = map_gettext(ttc.namefilepos);
      if (name == NULL)
        break;
      //printf("%ld from %ld, streetsfilepos %ld - %ld,%ld \"%s\"\n", ttc.id, ttc.country, ttc.streetsfilepos, ttc.pos.lat, ttc.pos.lon, name);

      ttc.filepos = as->filepos;
      as->filepos += map_city_size;
      
      int l = strlen(name) + 1;
      char *cname = (char *)malloc(l);
      utf8_to_ascii(name, cname);
      as->index++;
      
      if ((strlen(as->name) == 0) || (strcasepos(cname, as->name) == 0))
      {
        free(cname);
        tcity *tc = (tcity *)malloc(sizeof(tcity) + l);
        memcpy(tc, &ttc, sizeof(ttc));
        strcpy(tc->name, name);
        map_leavetext(ttc.namefilepos);
        return tc;
      }
      map_leavetext(ttc.namefilepos);
      free(cname);
    }
    else
    {
      map_unlock(map);
    }
  }
  while (ttc.id!=0);
  return NULL;
}

void map_city_stop(mapaddrsearch *as)
{
  if (as)
  {
    free(as);
  }
}


