/* -*- Mode: C; indent-tabs-mode: s; c-basic-offset: 2; tab-width: 2 -*- */
/*
 * map_common.h
 * Copyright (C) Mike Crash 2009 <mike@mikecrash.com>
 *
 * main.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * main.c is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef _MCMAPCOMMON_H
#define _MCMAPCOMMON_H

#include <stdio.h>
#include <map.h>
#include "mcthread.h"
//#include "map_segindex.h"
//#include "map_planes.h"

#define segmentscachesize 15

// nodes
struct mnode;

struct mcross
{
  long id;              //id of connected node
  mnode *node;          //pointer to connected node structure if known - will speed up next look up
  maptypes waytype;
  int speed;
  unsigned int flags;
  long namepos;
  char *name;
  char *note;
};

struct mturn
{
  unsigned int flags;
  int count;
  long *nodes;
};

struct mnode
{
  long id;               //OSM ID
  map_position pos;
  unsigned int flags;
  int crosscount;
  mcross *cross;
  int turncount;
  mturn *turn;
};

struct msegment
{
  int node;   //index of node in sector
  int cross;  //index of cross
};

struct msegments
{
  int count;
  msegment segments[];
};

struct node_sector
{
  //sector info
  long lat;
  long lon;
  long count;                 //count of nodes
  long filepos;               //position of nodes in nodes section
  long size;                  //size of nodes in nodes section

  //cache info
  int used;                   //is sector used? do not free the cache!
  int mem;                    //memory size used
  int readcount;              //count of reading of this sector from file

  mnode **nodes;              //pointer to array of pointers to nodes
  msegments **segments;       //pointer to array of segments by way type (size is cntwaytypes)

  struct node_sector *next;
  struct node_sector *prev;
};

struct nodescache
{
  int count;                  //cached sectors
  int mem;                    //size in memory
  int limit;                  //memory limit
  int readcount;              //maximum sector readcount
  node_sector *sectors;
  mcmutex mutex;
};

//planes
struct mplane
{
  maptypes planetype;
  map_multipolygon *mp;
};

struct mplanecache
{
  mplane plane;
  long lat1, lon1, lat2, lon2;
  struct mplanecache *next;
};

struct plane_sector
{
  //sector info
  long lat;
  long lon;
  char zoom;
  long count;
  long filepos;
  long size;

  //cache info
  int used;                   //is sector used? do not free the cache!
  int mem;                    //memory size used

  mplanecache **planes;       //pointer to array of planes by type

  struct plane_sector *next;
  struct plane_sector *prev;
};

struct planescache
{
  int count;                  //cached sectors
  int mem;                    //size in memory
  plane_sector *sectors;
};

//Locations
struct mloc
{
  maptypes loctype;
  map_position pos;
  long namepos;
  char *name;
};

struct mloccache
{
  struct mloccache *next;
  mloc loc;
};

struct loc_sector
{
  //sector info
  long lat;
  long lon;
  long count;
  long filepos;
  long size;

  //cache
  int used;                   //is sector used? do not free the cache!
  int mem;                    //memory used

  mloccache **locs;           //pointer to array of Locs by type

  struct loc_sector *next;
  struct loc_sector *prev;
};

struct locscache
{
  int count;                  //cached sectors
  int mem;                    //size in memory
  loc_sector *sectors;
};

//POIs
typedef struct
{
  maptypes poitype;
  map_position pos;
  int flags; 
  long namepos;
  char *name;
} mpoi;

typedef struct mpoicache
{
  struct mpoicache *next;
  mpoi poi;
} mpoicache;

typedef struct poi_sector
{
  //sector info
  long lat;
  long lon;
  long count;
  long filepos;
  long size;

  //cache
  int used;                   //is sector used? do not free the cache!
  int mem;                    //memory used

  mpoicache **pois;           //pointer to array of POIs by type

  struct poi_sector *next;
  struct poi_sector *prev;
} poi_sector;

typedef struct
{
  int count;                  //cached sectors
  int mem;                    //size in memory
  poi_sector *sectors;
} poiscache;

//address
typedef struct
{
  long countries_count;
} addresscache;


//text
typedef struct mtext
{
  struct mtext *next;
  struct mtext *prev;
  long filepos;
  int used;
  char text[];
} mtext;

typedef struct
{
  long count;            //count of text in cache
  mtext *first;          //first cached text
  mtext *last;           //last cached text
  mcmutex mutex;
} textcache;

//main structure
typedef struct
{
  //file info
  FILE *f;
  mcmutex mutex;
  map_info info;

  //cache
  nodescache nodes;
  planescache planes;
  locscache locs;
  poiscache pois;
  addresscache address;
  textcache text;

  //various
  int error;
} mcmap;

int map_lock(mcmap *map);
int map_unlock(mcmap *map);
int map_readpart(map_part *part);

extern mcmap *map;

#endif
