/* -*- Mode: C; indent-tabs-mode: s; c-basic-offset: 2; tab-width: 2 -*- */
/*
 * map_index.c
 * Copyright (C) Mike Crash 2009 <mike@mikecrash.com>
 * 
 * main.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * main.c is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <map.h>
#include <typeio.h>
#include <stdlib.h>
#include "map_common.h"
#include "map_index.h"

mapindexcache *map_indexread(char partid, long id, mapindexcache *cache)
{
  map_part part;

  //currently we do not support append of index
  if (cache)
    return cache;

  map_lock(map);

  //check index type
  map_readpart(&part);
  if (partid != part.partid)
  {
    map_unlock(map);
    return cache;
  }

  //TODO check the count

  //alloc new index
  mapindexcache *ic;
  ic = (mapindexcache *)malloc(sizeof(mapindexcache));
  ic->index = (mapindex *)malloc(sizeof(mapindex)*part.count);
  ic->next = NULL;

  //read the index
  long i = 0;
  mapindex *index = ic->index;
  while ((i<part.count) && !feof(map->f))
  {
    fread_long(&index->id, 1, map->f);
    fread_long(&index->filepos, 1, map->f);
    index++;
    i++;
  }

   ic->count = i;
   map_unlock(map);
   return ic;
}

mapindexcache * map_indexclear(mapindexcache *cache)
{
  if (cache==NULL)
    return NULL;
  if (cache->index)
    free(cache->index);
  free(cache);
  return NULL;
}

long map_indexfilepos(mapindexcache *cache, long id)
{
  if (cache == NULL)
    return 0;

  mapindex *index = cache->index;
  long i = 0;
  while ((i<cache->count) && (id < index->id))
  {
    index++;
    i++;
  }

  if (i>=cache->count)
    return 0;
  else
    return index->filepos;
}
