/* -*- Mode: C; indent-tabs-mode: s; c-basic-offset: 2; tab-width: 2 -*- */
/*
 * map_locations.cc
 * Copyright (C) Mike Crash 2009 <mike@mikecrash.com>
 * 
 * main.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * main.c is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdlib.h>
#include <typeio.h>
#include <string.h>
#include "map_common.h"
#include "map_locations.h"
#include "map_text.h"
#include "mcconf.h"


// *****************************
// free cache functions
// *****************************
void loccache_free(mloccache *cache)
{
  mloccache *next;
  while (cache)
  {
    next = cache->next;
    free(cache);
    cache = next;
  }
}

// *****************************
// read all plane sectors from file
// *****************************
int map_locindexread(void)
{
  map_part part;
  long cnt;

  map_lock(map);
  fseek(map->f, map->info.locsindex, SEEK_SET);
  //printf("Planes: %ld\n",map->info.planes);

  //check index type
  map_readpart(&part);
  //printf("Part: %d , %ld\n", part.partid, part.count);
  if (part.partid != MPID_LOCSINDEX)
  {
    map_unlock(map);
    printf("Bad Loc part\n");
    return -1;
  }

  //alloc new index
  cnt = part.count;

  //read the index
  long i = 0;
  loc_sector *prev = NULL;
  while ((i<part.count) && !feof(map->f))
  {
    loc_sector *sector = (loc_sector *)malloc(sizeof(loc_sector));
    fread_long(&sector->lat, 1, map->f);
    fread_long(&sector->lon, 1, map->f);
    fread_long(&sector->count, 1, map->f);
    fread_long(&sector->filepos, 1, map->f);
    fread_long(&sector->size, 1, map->f);
    sector->mem = 0;
    sector->used = 0;
    sector->locs = NULL;
    sector->next = NULL;
    sector->prev = prev;
    if (i==0)
      map->locs.sectors = sector;
    if (prev)
      prev->next = sector;
    prev = sector;
    //printf("Loc Sector %ld,%ld: %ld %ld\n", pi->pos.lat, pi->pos.lon, pi->count, pi->filepos);
    i++;
  }
  map->locs.count = 0;
  map->locs.mem = 0;
  map_unlock(map);
  return 0;
}

//read sector from file
int readlocsect(loc_sector *sector)
{
  //printf("Loc %ld,%ld Count: %ld Filepos: %ld Size: %ld Pos: %lx\n", sector->lat, sector->lon, sector->count, sector->filepos, sector->size, 
  //       map->info.locs + map_part_size + sector->filepos);
  //read it
  char *buf = (char*)malloc(sector->size);
  map_lock(map);
  fseek(map->f, map->info.locs + map_part_size + sector->filepos, SEEK_SET);
  fread_char(buf, sector->size, map->f);
  char *p = buf;

  sector->locs = (mloccache **)malloc(sizeof(mloccache*) * cntloctypes);
  memset(sector->locs, 0, sizeof(mloccache*) * cntloctypes);

  int i = 0;
  while (i<sector->count)
  {
    mloccache *pc = (mloccache *)malloc(sizeof(mloccache));
    pc->loc.loctype = (maptypes)read_uchar(&p);
    pc->loc.pos.lat = read_long(&p);
    pc->loc.pos.lon = read_long(&p);
    pc->loc.namepos = read_long(&p);
    pc->loc.name = NULL;

    int typ = (int)pc->loc.loctype - firstloctype;
    pc->next = sector->locs[typ];
    sector->locs[typ] = pc;
    sector->mem += sizeof(mloccache);
    i++;
  }
  map->locs.mem += sector->mem;
  map->locs.count++;
  free(buf);
  map_unlock(map);
  return 0;
}

int leavelocsector(loc_sector *sector)
{
  int i;
  for (i=0; i<cntloctypes; i++)
  {
    mloccache *mc = sector->locs[i];
    while (mc)
    {
      mloccache *mc2 = mc->next;
      if (mc->loc.namepos && mc->loc.name)
        map_leavetext(mc->loc.namepos);
      free(mc);
      mc = mc2;
    }
  }
  free(sector->locs);
  map->locs.mem -= sector->mem;
  map->locs.count--;
  sector->locs = NULL;
  sector->mem = 0;
  return 0;
}

//bring to top
int locsector2top(loc_sector *sector)
{
  if (sector == map->locs.sectors)
    return 0;
  if (sector->next)
    sector->next->prev = sector->prev;
  if (sector->prev)
    sector->prev->next = sector->next;
  map->locs.sectors->prev = sector;
  sector->next = map->locs.sectors;
  map->locs.sectors = sector;
  sector->prev = NULL;
  return 0;
}

//find all required sectors and update it
int update_loccache(long lat1, long lon1, long lat2, long lon2)
{
  lat1 = MAPSECTOR(lat1);
  lon1 = MAPSECTOR(lon1);
  lat2 = MAPSECTOR(lat2);
  lon2 = MAPSECTOR(lon2);
  loc_sector *sector = map->locs.sectors;
  int checked = 0;
  int check = ((lat2-lat1) / MAPSECTRECT + 1) * ((lon2-lon1) / MAPSECTRECT + 1);
  int updated = 0;

  while (sector && (checked<check))
  {
    loc_sector *next = sector->next;
    if ((sector->lat>=lat1) && (sector->lat<=lat2) && (sector->lon>=lon1) && (sector->lon<=lon2))
    {
      if ((sector->locs==NULL))
      {
        readlocsect(sector);
        updated++;
      }
      locsector2top(sector);
      checked++;
    }
    sector = next;
  }
  if (updated)
  {
    printf("Cached Locs: %d memory: %d kB\n", map->locs.count, map->locs.mem/1024);
  }
  return checked;
}


//free cache for maximum count
int freeloccache()
{
  int updated = 0;
  //printf("%ld - %ld\n", map->nodes.mem, config->limit_nodes);
  //check limit
  if (map->locs.mem < config->limit_locs)
    return 0;

  //fins last cached sector
  loc_sector *sector = map->locs.sectors;
  while (sector && sector->next && sector->next->locs)
  {
    sector = sector->next;
  }

  //free sectors
  while (sector && (map->locs.mem > config->limit_locs))
  {
    if (sector->locs)
    {
      leavelocsector(sector);
      updated = 1;
    }
    sector = sector->prev;
  }

  if (updated)
  {
    printf("Cached Locs: %d memory: %d kB\n", map->locs.count, map->locs.mem/1024);
  }
  return 0;
}


// *****************************
// init and deinit
// *****************************

int map_locs_init(void)
{
  if (map_locindexread() != 0)
    return -1;

  //check POIs part
  map_part part; 
  map_lock(map);
  fseek(map->f, map->info.locs, SEEK_SET);
  map_readpart(&part);
  map_unlock(map);
  if (part.partid!=MPID_LOCS)
    return -1;

  return 0;
}

int map_locs_clear(void)
{
  loc_sector *sector = map->locs.sectors;
  while (sector)
  {
    loc_sector *sector2 = sector->next;
    if (sector->locs)
    {
      leavelocsector(sector);
    }
    free(sector);
    sector = sector2;;
  }

  map->locs.sectors = NULL;
  map->locs.count = 0;
  map->locs.mem = 0;
  return 0;
}

// *******************
// plane search
// *******************

maplocsearch *map_locs_start(long lat1, long lon1, long lat2, long lon2)
{
  maplocsearch *ps;
  
  ps = (maplocsearch *)malloc(sizeof(maplocsearch));
  ps->lat1 = lat1;
  ps->lon1 = lon1;
  ps->lat2 = lat2;
  ps->lon2 = lon2;
  ps->checked = 0;
  ps->check = update_loccache(lat1, lon1, lat2, lon2);

  ps->sector = map->locs.sectors;
  ps->loc = NULL;
  //printf("Find POI %ld,%ld->%ld,%ld %d (%d-%d)\n", ps->lat1, ps->lon1, ps->lat2, ps->lon2, ps->loctype, firstloctype, maxloctype);
  return ps;
}

void map_locs_type(maplocsearch *ps, maptypes loctype)
{
  ps->loctype = loctype;
  ps->checked = 0;
  ps->sector = map->locs.sectors;
  ps->loc = NULL;
}

mloc *map_locs_get(maplocsearch *ps)
{
  loc_sector *sector = (loc_sector *)ps->sector;
  mloccache *loc = (mloccache *)ps->loc;

  while (sector && (ps->checked<ps->check))
  {
    //printf("Find loc %ld,%ld->%ld,%ld\n", plane->lat1, plane->lon1, plane->lat2, plane->lon2);
    //is cache area part of requested area?
    if (sector->locs)
    {
      if (loc==NULL)
        loc = sector->locs[ps->loctype - firstloctype];
      while (loc)
      {
        if ((loc->loc.pos.lat>=ps->lat1) &&
            (loc->loc.pos.lat<=ps->lat2) &&
            (loc->loc.pos.lon>=ps->lon1) &&
            (loc->loc.pos.lon<=ps->lon2))
        {
          ps->loc = loc->next;
          if (ps->loc == NULL)
          {
            ps->sector = sector->next;
            ps->checked++;
          }
          else
            ps->sector = sector;
          if (loc->loc.name == NULL)
          {
            loc->loc.name = map_gettext(loc->loc.namepos);
          }
          return &loc->loc;
        }
        loc = loc->next;
      }
    }
    sector = sector->next;
    ps->checked++;
    loc = NULL;
  }
  ps->sector = sector;
  ps->loc = loc;
  return NULL;
}

int map_locs_stop(maplocsearch *ps)
{
  free(ps);
  freeloccache();
  return 0;
}
