/* -*- Mode: C; indent-tabs-mode: s; c-basic-offset: 2; tab-width: 2 -*- */
/*
 * map_nodes.c
 * Copyright (C) Mike Crash 2009 <mike@mikecrash.com>
 * 
 * main.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * main.c is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <string.h>
#include <stdlib.h>
#include <typeio.h>
#include "mcconf.h"
#include "mctime.h"
#include "map_common.h"
#include "projection.h"

int readnodesectorindex()
{
  map_part part; 
  fseek(map->f, map->info.nodesindex, SEEK_SET);
  map_readpart(&part);
  if (part.partid!=MPID_NODESINDEX)
    return -1;

  int i = 0;
  node_sector *prev = NULL;
  while (i<part.count)
  {
    //alloc sector
    node_sector *sector = (node_sector*)malloc(sizeof(node_sector));
    sector->next = NULL;
    sector->prev = prev;
    if (i==0)
      map->nodes.sectors = sector;
    if (prev)
      prev->next = sector;

    //read
    fread_long(&sector->lat, 1, map->f);
    fread_long(&sector->lon, 1, map->f);
    fread_long(&sector->count, 1, map->f);
    fread_long(&sector->filepos, 1, map->f);
    fread_long(&sector->size, 1, map->f);

    //init
    sector->used = 0;
    sector->mem = 0;
    sector->nodes = NULL;

    //printf("Node sector: %ld,%ld nodes %ld pos %ld size %ld\n", sector->lat, sector->lon, sector->count, sector->filepos, sector->size);
    prev = sector;
    i++;
  }

  return 0;
}

//segments
int update_segments(node_sector *sector)
{
  if (sector == NULL)
    return -1;
  if (sector->nodes == NULL)
    return -1;
  if (sector->segments)
    return 0;

  //get count of segment types
  int count[cntwaytypes];
  int wcnt = 0;
  int total = 0;
  memset(count, 0, sizeof(count));
  for (int i=0; i<sector->count; i++)
  {
    mnode *node = sector->nodes[i];
    for (int j=0; j<node->crosscount; j++)
    {
      int wt = (int)node->cross[j].waytype;
      if ((wt<cntwaytypes) && ((node->cross[j].flags & WF_DIR) == 0))
      {
        if (count[wt] == 0)
          wcnt++;
        count[wt]++;
        total++;
      }
    }
  }

  //create and fill array of segments by waytype
  sector->segments = (msegments**)malloc(sizeof(msegments*) * cntwaytypes);
  memset(sector->segments, 0, sizeof(msegments*) * cntwaytypes);
  for (int i=0; i<sector->count; i++)
  {
    mnode *node = sector->nodes[i];
    for (int j=0; j<node->crosscount; j++)
    {
      int wt = (int)node->cross[j].waytype;
      if ((wt<cntwaytypes) && ((node->cross[j].flags & WF_DIR) == 0))
      {
        if (sector->segments[wt] == NULL)
        {
          sector->segments[wt] = (msegments*)malloc(sizeof(msegments)+sizeof(msegment)*count[wt]);
          sector->segments[wt]->count = 0;
        }
        sector->segments[wt]->segments[sector->segments[wt]->count].node = i;
        sector->segments[wt]->segments[sector->segments[wt]->count].cross = j;
        sector->segments[wt]->count++;
      }
    }
  }

  //update size
  long mem = sizeof(msegments*) * cntwaytypes + sizeof(msegment) * wcnt + sizeof(msegment) * total;
  sector->mem += mem;
  map->nodes.mem += mem;
  return 0;
}

int clear_segments(node_sector *sector)
{
  if (sector == NULL)
    return -1;
  if (sector->segments == NULL)
    return -1;
  int wcnt = 0;
  int total = 0;
  for (int i=0; i<cntwaytypes; i++)
  {
    if (sector->segments[i])
    {
      wcnt++;
      total += sector->segments[i]->count;
      free(sector->segments[i]);
    }
  }
  free(sector->segments);
  sector->segments = NULL;
  long mem = sizeof(msegments*) * cntwaytypes + sizeof(msegment) * wcnt + sizeof(msegment) * total;
  sector->mem -= mem;
  map->nodes.mem -= mem;
  return 0;
}

int readnodesector(node_sector *sector)
{
  //printf("Read node sector: %ld,%ld nodes %ld pos %ld size %ld\n", sector->lat, sector->lon, sector->count, sector->filepos, sector->size);
  int size = 0;
  char *buf = (char*)malloc(sector->size);
  map_lock(map);
  fseek(map->f, map->info.nodes + map_part_size + sector->filepos, SEEK_SET);
  fread_char(buf, sector->size, map->f);
  map_unlock(map);

  sector->nodes = (mnode **)malloc(sizeof(mnode *) * sector->count);

  int i;
  char *p = buf;
  for (i=0; i<sector->count; i++)
  {
    sector->nodes[i] = (mnode *)malloc(sizeof(mnode));
    sector->nodes[i]->id = read_long(&p);
    sector->nodes[i]->pos.lat = read_long(&p);
    sector->nodes[i]->pos.lon = read_long(&p);
    sector->nodes[i]->flags = read_uint(&p);
    sector->nodes[i]->crosscount = read_uchar(&p);
    sector->nodes[i]->cross = (mcross*)malloc(sizeof(mcross)*sector->nodes[i]->crosscount);
    //printf("  %d/%d %ld %d,%d flags %u crosscount %d\n", i, sector->count, sector->nodes[i]->id, sector->nodes[i]->pos.lat, sector->nodes[i]->pos.lon, sector->nodes[i]->flags, sector->nodes[i]->crosscount);
    for (int j=0; j<sector->nodes[i]->crosscount; j++)
    {
      sector->nodes[i]->cross[j].id = read_long(&p);
      sector->nodes[i]->cross[j].node = NULL;
      sector->nodes[i]->cross[j].waytype = (maptypes)read_uchar(&p);
      sector->nodes[i]->cross[j].speed = read_uchar(&p);
      sector->nodes[i]->cross[j].flags = read_uint(&p);
      sector->nodes[i]->cross[j].namepos = read_long(&p);
      sector->nodes[i]->cross[j].name = NULL;
      sector->nodes[i]->cross[j].note = NULL;
      //printf("    %d: %ld - %d,%d,%u,%ld\n", j, sector->nodes[i]->cross[j].node, sector->nodes[i]->cross[j].waytype, sector->nodes[i]->cross[j].speed, sector->nodes[i]->cross[j].flags, sector->nodes[i]->cross[j].namepos);
    }
		//turn restrictions
    int turnsize = 0;
		if (sector->nodes[i]->flags & NF_TURNRESTRICTION)
		{
			sector->nodes[i]->turncount = read_uchar(&p);
      //printf("Turncount %d\n", sector->nodes[i]->turncount);
			sector->nodes[i]->turn = (mturn*)malloc(sector->nodes[i]->turncount * sizeof(mturn));
      for (int j=0; j<sector->nodes[i]->turncount; j++)
      {
        sector->nodes[i]->turn[j].flags = read_uint(&p);
        sector->nodes[i]->turn[j].count = read_uchar(&p);
        sector->nodes[i]->turn[j].nodes = (long*)malloc(sector->nodes[i]->turn[j].count * sizeof(long));
        for (int k=0; k<sector->nodes[i]->turn[j].count; k++)
        {
          sector->nodes[i]->turn[j].nodes[k] = read_long(&p);
        }
        turnsize += sizeof(long)*sector->nodes[i]->turn[j].count;
      }
      turnsize += sizeof(mturn)*sector->nodes[i]->turncount;
		}
		else
		{
			sector->nodes[i]->turncount = 0;
			sector->nodes[i]->turn = NULL;
		}
    size += sizeof(mnode) + sizeof(mcross)*sector->nodes[i]->crosscount + turnsize;
  }
  free(buf);
  sector->mem = size + sizeof(map_node *) * sector->count;
  sector->readcount++;
  if (sector->readcount > map->nodes.readcount)
    map->nodes.readcount = sector->readcount;
  map->nodes.mem += sector->mem;
  map->nodes.count++;
  return 0;
}

int leavenodesector(node_sector *sector)
{
  if (sector->nodes == NULL)
    return -1;

  clear_segments(sector);

  //printf("Free node sector: %ld,%ld nodes %ld pos %ld size %ld\n", sector->lat, sector->lon, sector->count, sector->filepos, sector->size);
  for (int i=0; i<sector->count; i++)
  {
    for (int j=0; j<sector->nodes[i]->crosscount; j++)
     {
       if (sector->nodes[i]->cross[j].name)
       {
         map_leavetext(sector->nodes[i]->cross[j].namepos);
       }
       if (sector->nodes[i]->cross[j].node)
       {
         mnode *link = sector->nodes[i]->cross[j].node;
         for (int k=0; k<link->crosscount; k++)
         {
           if (link->cross[k].id == sector->nodes[i]->id)
             link->cross[k].node = NULL;
         }
         sector->nodes[i]->cross[j].node = NULL;
       }
     }
		if (sector->nodes[i]->cross)
			free(sector->nodes[i]->cross);
		if (sector->nodes[i]->turn)
    {
      for (int k=0; k<sector->nodes[i]->turncount; k++)
        free(sector->nodes[i]->turn[k].nodes);
			free(sector->nodes[i]->turn);
    }
    free(sector->nodes[i]);
  }
  free(sector->nodes);
  map->nodes.mem -= sector->mem;
  map->nodes.count--;
  sector->nodes = NULL;
  sector->mem = 0;
  return 0;
}


// init and deinit
int map_nodes_init()
{
  map_part part; 
  mcmutex_init(&map->nodes.mutex);
  map_lock(map);
  //check correct structure
  fseek(map->f, map->info.nodes, SEEK_SET);
  map_readpart(&part);
  if (part.partid!=MPID_NODES)
  {
    map_unlock(map);
    return -1;
  }
  //read sectors to memory
  if (readnodesectorindex()!=0)
  {
    map_unlock(map);
    return -1;
  }
  map->nodes.count = 0;
  map->nodes.mem = 0;
  map_unlock(map);
  //readnodesector(map->nodes.sectors);
  //readnodesector(map->nodes.sectors->next);
  return 0;
}

int map_nodes_clear()
{
  node_sector *nc = map->nodes.sectors;
  while (nc)
  {
    node_sector *nc2 = nc->next;
    if (nc->nodes)
    {
      leavenodesector(nc);
    }
    free(nc);
    nc = nc2;
  }
  printf("Cached nodes: %d memory: %d kB\n", map->nodes.count, map->nodes.mem/1024);
  printf("Maximum sector read count: %d\n", map->nodes.readcount);
  map->nodes.count = 0;
  map->nodes.mem = 0;
  map->nodes.sectors = NULL;
  mcmutex_destroy(&map->nodes.mutex);
  return 0;
}

//free cache for maximum count
int freenodecache()
{
  int updated = 0;
  //printf("%ld - %ld\n", map->nodes.mem, config->limit_nodes);
  //check limit
  if (map->nodes.mem < config->limit_nodes)
    return 0;

  //fins last cached sector
  node_sector *sector = map->nodes.sectors;
  while (sector && sector->next && sector->next->nodes)
  {
    sector = sector->next;
  }

  //free sectors
  while (sector && (map->nodes.mem > config->limit_nodes))
  {
    if ((sector->nodes) && (sector->used==0))
    {
      leavenodesector(sector);
      updated = 1;
    }
    sector = sector->prev;
  }

  if (updated)
  {
    printf("Cached nodes: %d memory: %d kB\n", map->nodes.count, map->nodes.mem/1024);
  }
  return 0;
}

//bring to top
int nodesector2top(node_sector *sector)
{
  if (sector == map->nodes.sectors)
    return 0;
  if (sector->next)
    sector->next->prev = sector->prev;
  if (sector->prev)
    sector->prev->next = sector->next;
  map->nodes.sectors->prev = sector;
  sector->next = map->nodes.sectors;
  map->nodes.sectors = sector;
  sector->prev = NULL;
  return 0;
}

//bring to back
int nodesector2back(node_sector *sector)
{
  node_sector *last = map->nodes.sectors;
  while (last->next && last->next->nodes)
  {
    if (last == sector)
      return 0;
    last = last->next;
  }

  //not required
  if (last == sector)
    return 0;

  //remove sector
  if (sector->prev)
    sector->prev->next = sector->next;
  if (sector->next)
    sector->next->prev = sector->prev;

  //first sector
  if (map->nodes.sectors == last)
  {
    map->nodes.sectors->prev = sector;
    sector->next = map->nodes.sectors;
    map->nodes.sectors = sector;
    return 0;
  }

  //swap
  if (last->next)
    last->next->prev = sector;
  sector->next = last->next;
  last->next = sector;
  sector->prev = last;
  return 0;
}

//insert sector after another sector in order
int nodesector2after(node_sector *sector, int after)
{
  //check order
  node_sector *last = map->nodes.sectors;
  int i = 0;
  while (last && last->next->nodes && (i<after))
  {
    if (last == sector)
      return 0;
    last = last->next;
    i++;
  }

  //not required
  if ((last==NULL) || (last == sector))
    return 0;

  //remove sector
  if (sector->prev)
    sector->prev->next = sector->next;
  if (sector->next)
    sector->next->prev = sector->prev;

  //swap
  if (last->next)
    last->next->prev = sector;
  sector->next = last->next;
  last->next = sector;
  sector->prev = last;
  return 0;
}

node_sector *get_nodesector(long lat, long lon)
{
  node_sector *sector = map->nodes.sectors;
  while (sector)
  {
    if ((sector->lat==lat) && (sector->lon==lon))
    {
      //read from file
      if (sector->nodes == NULL)
      {
        if (readnodesector(sector) != 0)
        {
          return NULL;
        }
        //nodesector2back(sector);
        nodesector2top(sector);
      }
      return sector;
    }
    sector = sector->next;
  }
  return NULL;
}

mnode *getnode(long id, node_sector *sector, int lock = 0)
{
  if ((sector->nodes[0]->id<=id) && (sector->nodes[sector->count-1]->id>=id))
  {
    //binary search in sorted list
    int k1 = 0;
    int k2 = sector->count-1;
    while (k1<=k2)
    {
      int k = (k1+k2)/2;
      if (sector->nodes[k]->id == id)
      {
        //lock sector if requested
        if (lock)
        {
          sector->used++;
        }
        return sector->nodes[k];
      }
      if (id<sector->nodes[k]->id)
        k2 = k - 1;
      else
        k1 = k + 1;
    }
  }
  //printf("Node %ld not found\n", id);
  return NULL;
}

mnode *getnode(long id, map_position *loc, int lock = 0, int after = -1)
{
  int updated = 0;
  long latangle = wgs84_latangle(MAPMAXNODEDIST);
  long lonangle = wgs84_lonangle(loc->lat, MAPMAXNODEDIST);
  long lat1 = MAPSECTOR(loc->lat-latangle);
  long lon1 = MAPSECTOR(loc->lon-lonangle);
  long lat2 = MAPSECTOR(loc->lat+latangle);
  long lon2 = MAPSECTOR(loc->lon+lonangle);
  int checked = 0;
  int check = ((lat2-lat1) / MAPSECTRECT + 1) * ((lon2-lon1) / MAPSECTRECT + 1);

  node_sector *sector = map->nodes.sectors;
  while (sector && (checked<check))
  {
    node_sector *next = sector->next;
    if (!((sector->lat<lat1) || (sector->lat>lat2) || (sector->lon<lon1) || (sector->lon>lon2)))
    {
      if (sector->nodes==NULL)
      {
        if (readnodesector(sector) != 0)
        {
          return NULL;
        }
        if (after>=0)
          nodesector2after(sector, after);
        else
          nodesector2top(sector);
        updated = 1;
      }
      checked++;
      mnode *node = getnode(id, sector, lock);
      if (node)
        return node;
    }
    sector = next;
  }

  if (updated)
  {
    printf("Cached nodes: %d memory: %d kB\n", map->nodes.count, map->nodes.mem/1024);
  }

  return NULL;
}

//find all required sectors and update it, bring sectors to top
int update_nodecache(long lat1, long lon1, long lat2, long lon2)
{
  int updated = 0;
  node_sector *sector = map->nodes.sectors;
  lat1 = MAPSECTOR(lat1);// - MAPSECTRECT;
  lon1 = MAPSECTOR(lon1);// - MAPSECTRECT;
  lat2 = MAPSECTOR(lat2);// + MAPSECTRECT;
  lon2 = MAPSECTOR(lon2);// + MAPSECTRECT;
  //printf("Update %ld,%ld->%ld,%ld\n", lat1, lon1, lat2, lon2);
  int checked = 0;
  int check = ((lat2-lat1) / MAPSECTRECT + 1) * ((lon2-lon1) / MAPSECTRECT + 1);

  while (sector && (checked<check))
  {
    node_sector *next = sector->next;
    if (!((sector->lat<lat1) || (sector->lat>lat2) || (sector->lon<lon1) || (sector->lon>lon2)))
    {
      if (sector->nodes==NULL)
      {
        readnodesector(sector);
        updated = 1;
      }
      //printf("  Check %ld,%ld\n", sector->lat, sector->lon); 
      nodesector2top(sector);
      checked++;
    }
    sector = next;
  }

  if (updated)
  {
    printf("Cached nodes: %d memory: %d kB\n", map->nodes.count, map->nodes.mem/1024);
  }

  return checked;
}

//nodes
int nodeposition(long nodeid, map_position *loc, map_position *pos)
{
  mnode *node = getnode(nodeid, loc, 0);
  if (node == NULL)
  {
    return -1;
  }
  *pos = node->pos;
  return 0;
}

int map_nodeposition(long nodeid, node_sector *sector, map_position *pos)
{
  if (sector == NULL)
    return -1;

  mnode *node = getnode(nodeid, sector);
  if (node == NULL)
  {
    return -1;
  }
  *pos = node->pos;
  return 0;
}

int map_nodeposition(long nodeid, map_position *loc, map_position *pos)
{
  if (map == NULL)
    return -1;

  mcmutex_lock(&map->nodes.mutex);

  if (nodeposition(nodeid, loc, pos) != 0)
  {
    mcmutex_unlock(&map->nodes.mutex);
    return -1;
  }

  mcmutex_unlock(&map->nodes.mutex);
  return 0;
}

int crossposition(mnode *node, int cross, map_position *loc, map_position *pos, int after = -1)
{
  if ((node == NULL) || (cross>=node->crosscount))
    return -1;
  
  if (node->cross[cross].node == NULL)
  {
    mnode *crossnode = getnode(node->cross[cross].id, &node->pos, 0, after);
    if (crossnode == NULL)
    {
      return -1;
    }
    node->cross[cross].node = crossnode;
    for (int i=0; i<crossnode->crosscount; i++)
    {
      if (crossnode->cross[i].id == node->id)
      {
        crossnode->cross[i].node = node;
        break;
      }
    }
    *pos = crossnode->pos;
  }
  else
  {
    *pos = node->cross[cross].node->pos;
  }
  return 0;
}

//these is not thread safe, lock mutex
int map_nodeinsector(long id, node_sector *sector)
{
  if (sector == NULL)
    return -1;

  mnode *node = getnode(id, sector);
  if (node)
    return 1;
  else
    return 0;
}


//return nearest way segment
int map_nearestwayseg(map_position *pos, maptypes waytype_min, maptypes waytype_max, long *node1, long *node2, maptypes *waytype, int *speed, unsigned int *flags)
{
  mcmutex_lock(&map->nodes.mutex);

  //first update nodes near required point
  long flat1 = pos->lat - 4000;
  long flat2 = pos->lat + 4000;
  long flon1 = pos->lon - 4000;
  long flon2 = pos->lon + 4000;
  update_nodecache(flat1, flon1, flat2, flon2);

  //search
  int dist = -1;
  node_sector *sector = map->nodes.sectors;
  while (sector && sector->nodes)
  {
    //is cache area part of requested area?
    if (!((flat2<sector->lat) || (flat1>sector->lat+MAPSECTRECT) || (flon2<sector->lon) || (flon1>sector->lon+MAPSECTRECT)))
    {
      for (int i=0; i<sector->count; i++)
      {
        mnode *mn = sector->nodes[i];
        int cross = 0;
        while (cross<mn->crosscount)
        {
          if ((mn->cross[cross].flags & WF_DIR) == 0)
          {
            //check distance
            map_position pos2;
            if (nodeposition(mn->cross[cross].id, &mn->pos, &pos2) == 0)
            {
              int d = wgs84_waydistance(pos->lat, pos->lon, mn->pos.lat, mn->pos.lon, pos2.lat, pos2.lon);
              if ((dist==-1) || (d<dist))
              {
                //check way type
                if ((mn->cross[cross].waytype>=waytype_min) && (mn->cross[cross].waytype<=waytype_max))
                {
                  *flags = mn->cross[cross].flags;
                  *waytype = mn->cross[cross].waytype;
                  *speed = mn->cross[cross].speed;
                  *node1 = mn->id;
                  *node2 = mn->cross[cross].id;
                  dist = d;
                }
              }
            }
          }
          cross++;
        }
      }
    }
    sector = sector->next;
  }

  mcmutex_unlock(&map->nodes.mutex);

  if (dist==-1)
    return -1;

  //printf("Nearest way %ld-%ld, way %ld\n", *node1, *node2, *way);
  return 0;
}

int map_nearestwaysnap(map_position *pos, maptypes waytype_min, maptypes waytype_max, map_position *snap, int *dist)
{
  long node1, node2;
  unsigned int flags;
  maptypes waytype;
  int speed;
  
  if (map_nearestwayseg(pos, waytype_min, waytype_max, &node1, &node2,  &waytype, &speed, &flags) != 0)
    return -1;

  mcmutex_lock(&map->nodes.mutex);

  map_position pos1, pos2;

  if (nodeposition(node1, pos, &pos1) != 0)
  {
    mcmutex_unlock(&map->nodes.mutex);
    return -1;
  }

  if (nodeposition(node2, pos, &pos2) != 0)
  {
    mcmutex_unlock(&map->nodes.mutex);
    return -1;
  }

  int section1, section2;
  wgs84_waysnap(pos->lat, pos->lon, pos1.lat, pos1.lon, pos2.lat, pos2.lon, &snap->lat, &snap->lon, dist, &section1, &section2);

  mcmutex_unlock(&map->nodes.mutex);

  return 0;
}

node_sector *map_locknodesector(long lat, long lon)
{
  if (map == NULL)
    return NULL;

  lat = MAPSECTOR(lat);
  lon = MAPSECTOR(lon);

  mcmutex_lock(&map->nodes.mutex);
  node_sector *sector = get_nodesector(lat, lon);
  if (sector)
  {
    sector->used++;
  }
  mcmutex_unlock(&map->nodes.mutex);

  return sector;
}

int map_unlocknodesector(node_sector *sector)
{
  mcmutex_lock(&map->nodes.mutex);
  sector->used--;
  mcmutex_unlock(&map->nodes.mutex);
  return 0;
}

//return cross, lock to avoid freeing this node, call map_nodeunlock to unlock
int map_nodecross(long nodeid, node_sector *sector, int *count, mcross **cross)
{
  if (map == NULL)
    return -1;

  mcmutex_lock(&map->nodes.mutex);

  mnode *node = getnode(nodeid, sector, 1);
  if (node == NULL)
  {
    mcmutex_unlock(&map->nodes.mutex);
    return -1;
  }

  *cross = node->cross;
  *count = node->crosscount;
  mcmutex_unlock(&map->nodes.mutex);
  return 0;
}

int map_nodecross(long nodeid, map_position *loc, int *count, mcross **cross)
{
  if (map == NULL)
    return -1;

  mcmutex_lock(&map->nodes.mutex);

  mnode *node = getnode(nodeid, loc, 1);
  if (node == NULL)
  {
    mcmutex_unlock(&map->nodes.mutex);
    return -1;
  }

  *cross = node->cross;
  *count = node->crosscount;
  mcmutex_unlock(&map->nodes.mutex);
  return 0;
}

int map_nodecrossturn(long nodeid, map_position *loc, int *count, mcross **cross, int *turncount, mturn **turn)
{
  if (map == NULL)
    return -1;

  mcmutex_lock(&map->nodes.mutex);

  mnode *node = getnode(nodeid, loc, 1);
  if (node == NULL)
  {
    mcmutex_unlock(&map->nodes.mutex);
    return -1;
  }

  *cross = node->cross;
  *count = node->crosscount;
  *turncount = node->turncount;
  *turn = node->turn;
  mcmutex_unlock(&map->nodes.mutex);
  return 0;
}

int map_nodeunlock(long nodeid, map_position *loc)
{
  mcmutex_lock(&map->nodes.mutex);

  //find sector
  node_sector *sector = get_nodesector(MAPSECTOR(loc->lat), MAPSECTOR(loc->lon));
  //don't check node is present in sector - we trust the caller that the node exists
  if (sector && (sector->used>0))
  {
    sector->used--;
  }

  freenodecache();
  mcmutex_unlock(&map->nodes.mutex);
  return 0;
}

//segments for render
mapsegmentsearch *map_segment_start(long lat1, long lon1, long lat2, long lon2)
{
  mapsegmentsearch *cs;

  //we must search larger rectangle to get ways with outside nodes but crossing the area
  //long latsearch = MAPSECTRECT;
  //long lonsearch = MAPSECTRECT;
  long latsearch = wgs84_latangle(MAPMAXNODEDIST/2);
  long lonsearch = wgs84_lonangle(lat1, MAPMAXNODEDIST/2);
  cs = (mapsegmentsearch *)malloc(sizeof(mapsegmentsearch));
  cs->lat1 = lat1-latsearch;
  cs->lon1 = lon1-lonsearch;
  cs->lat2 = lat2+latsearch;
  cs->lon2 = lon2+lonsearch;

  //printf("Segments %ld,%ld->%ld,%ld\n", lat1, lon1, lat2, lon2);

  mcmutex_lock(&map->nodes.mutex);
  cs->check = update_nodecache(cs->lat1, cs->lon1, cs->lat2, cs->lon2);

  return cs;
}

void map_segment_type(mapsegmentsearch *cs, maptypes waytype, int names)
{
  cs->waytype = waytype;
  cs->sector = map->nodes.sectors;
  cs->index = 0;
  cs->node = 0;
  cs->cross = 0;
  cs->checked = 0;
  cs->names = names;

  //printf(" Search type %d\n", waytype);
  //printf("  Look sector %ld,%ld\n", map->nodes.sectors->lat, map->nodes.sectors->lon);
}

map_segment *map_segment_get(mapsegmentsearch *cs)
{
  node_sector *sector = (node_sector *)cs->sector;

  //printf("Look sector %ld,%ld\n", sector->lat, sector->lon);
  while (cs->sector && (cs->checked<cs->check))
  {
    if (sector->segments == NULL)
      update_segments(sector);
    //is cache area part of requested area?
    if (!((cs->lat2<sector->lat) || (cs->lat1>sector->lat+MAPSECTRECT) || (cs->lon2<sector->lon) || (cs->lon1>sector->lon+MAPSECTRECT)))
    {
      //printf("  Search sector %ld,%ld\n", sector->lat, sector->lon);
      if (sector->segments && sector->segments[cs->waytype])
      {
        while (cs->index < sector->segments[cs->waytype]->count)
        {
          int ni = sector->segments[cs->waytype]->segments[cs->index].node;
          int nc = sector->segments[cs->waytype]->segments[cs->index].cross;
          //printf("   Segment %d/%d\n", cs->index, sector->segments[cs->waytype]->count);
          cs->index++;
          mnode *mn = sector->nodes[ni];
          map_position pos;
          if (crossposition(mn, nc, &mn->pos, &pos, cs->check-1) == 0)
          {
            if ( ((pos.lat>=cs->lat1) && (pos.lat<=cs->lat2) && (pos.lon>=cs->lon1) && (pos.lon<=cs->lon2)) ||
                ((mn->pos.lat>=cs->lat1) && (mn->pos.lat<=cs->lat2) && (mn->pos.lon>=cs->lon1) && (mn->pos.lon<=cs->lon2)) )
            {
              cs->segment.start = mn->pos;
              cs->segment.end = pos;
              cs->segment.waytype = mn->cross[nc].waytype;
              cs->segment.flags = mn->cross[nc].flags;
              if ((mn->cross[nc].name == NULL) && (mn->cross[nc].namepos))
              {
                mn->cross[nc].name = map_gettext(mn->cross[nc].namepos);
              }
              cs->segment.name = mn->cross[nc].name;
              return &cs->segment;
            }
          }
        }
      }
      cs->checked++;
    }
    sector = sector->next;
    cs->sector = sector;
    cs->index = 0;
    //printf("Look sector %ld,%ld\n", sector->lat, sector->lon);
  }
  cs->sector = NULL;
  return NULL;
}  

/*map_segment *map_segment_get(mapsegmentsearch *cs)
{
  node_sector *sector = (node_sector *)cs->sector;
  long node = cs->node;
  int cross = cs->cross;

  //printf("Look sector %ld,%ld\n", sector->lat, sector->lon);
  while (sector && (cs->checked<cs->check))
  {
    //is cache area part of requested area?
    if (!((cs->lat2<sector->lat) || (cs->lat1>sector->lat+MAPSECTRECT) || (cs->lon2<sector->lon) || (cs->lon1>sector->lon+MAPSECTRECT)))
    {
      //printf("  Search sector %ld,%ld\n", sector->lat, sector->lon);
      while (node<sector->count)
      {
        mnode *mn = sector->nodes[node];
        while (cross<mn->crosscount)
        {
          if ((mn->cross[cross].flags & WF_DIR) == 0)
          {
            if (mn->cross[cross].waytype == cs->waytype)
            {
              map_position pos;
              if (crossposition(mn, cross, &mn->pos, &pos, cs->check-1) == 0)
              {
                if ( ((pos.lat>=cs->lat1) && (pos.lat<=cs->lat2) && (pos.lon>=cs->lon1) && (pos.lon<=cs->lon2)) ||
                    ((mn->pos.lat>=cs->lat1) && (mn->pos.lat<=cs->lat2) && (mn->pos.lon>=cs->lon1) && (mn->pos.lon<=cs->lon2)) )
                {
                  cs->segment.start = mn->pos;
                  cs->segment.end = pos;
                  cs->segment.waytype = mn->cross[cross].waytype;
                  cs->segment.flags = mn->cross[cross].flags;
                  if ((mn->cross[cross].name == NULL) && (mn->cross[cross].namepos))
                  {
                    mn->cross[cross].name = map_gettext(mn->cross[cross].namepos);
                  }
                  cs->segment.name = mn->cross[cross].name;
                  //cs->segment.note = mn->cross[cross].note;
                  //printf("  Found %ld-%ld [%ld,%ld]-[%ld,%ld]\n", mn->id, mn->cross[cross].id, mn->pos.lat, mn->pos.lon, mn->cross[cross].node->pos.lat, mn->cross[cross].node->pos.lon);

                  //save search status
                  if (cross == (mn->crosscount-1))
                  {
                    if (node == (sector->count - 1))
                    {
                      cs->sector = sector->next;
                      cs->checked++;
                      cs->node = 0;
                    }
                    else
                    {
                      cs->sector = sector;
                      cs->node = node + 1;
                    }
                    cs->cross = 0;
                  }
                  else
                  {
                    cs->sector = sector;
                    cs->node = node;
                    cs->cross = cross + 1;
                  }
                  return &cs->segment;
                }
              }
            }
          }
          cross++;
        }
        node++;
        cross = 0;
      }
      cs->checked++;
    }
    sector = sector->next;
    node = 0;
  }
  cs->sector = NULL;
  return NULL;
}*/

void map_segment_stop(mapsegmentsearch *cs)
{
  freenodecache();
  //printf("Done\n");
  free(cs);
  mcmutex_unlock(&map->nodes.mutex);
}

char *map_segment_name(long node1, long node2)
{
  //TODO
  return NULL;
}
