/* -*- Mode: C; indent-tabs-mode: s; c-basic-offset: 2; tab-width: 2 -*- */
/*
 * map_nodes.h
 * Copyright (C) Mike Crash 2009 <mike@mikecrash.com>
 * 
 * main.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * main.c is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef _MCMAPNODES_H
#define _MCMAPNODES_H

typedef struct
{
  map_position start;
  map_position end;
  maptypes waytype;
  unsigned int flags;
  char *name;
  //char *note;
} map_segment;

typedef struct
{
  long lat1;
  long lon1;
  long lat2;
  long lon2;
  maptypes waytype;
  int names;
  int check;
  int checked;
  map_segment segment;
  void *sector;
  int index;
  long node;
  int cross;
} mapsegmentsearch;

int map_nodes_init();
int map_nodes_clear();

node_sector *map_locknodesector(long lat, long lon);
int map_unlocknodesector(node_sector *sector);

int map_nodeposition(long nodeid, map_position *loc, map_position *pos);
int map_nodeposition(long nodeid, node_sector *sector, map_position *pos); //this is not thread safe, lock sector first
int map_nodeinsector(long id, node_sector *sector); //this is not thread safe, lock sector first

int map_nearestwayseg(map_position *pos, maptypes waytype_min, maptypes waytype_max, long *node1, long *node2, maptypes *waytype, int *speed, unsigned int *flags);
int map_nearestwaysnap(map_position *pos, maptypes waytype_min, maptypes waytype_max, map_position *snap, int *dist);

int map_nodecross(long nodeid, node_sector *sector, int *count, mcross **cross);
int map_nodecross(long nodeid, map_position *loc, int *count, mcross **cross);
int map_nodecrossturn(long nodeid, map_position *loc, int *count, mcross **cross, int *turncount, mturn **turn);
int map_nodeunlock(long nodeid, map_position *loc);

mapsegmentsearch *map_segment_start(long lat1, long lon1, long lat2, long lon2);
void map_segment_type(mapsegmentsearch *cs, maptypes waytype, int names);
map_segment *map_segment_get(mapsegmentsearch *cs);
void map_segment_stop(mapsegmentsearch *cs);

char *map_segment_name(long node1, long node2);

#endif