/* -*- Mode: C; indent-tabs-mode: s; c-basic-offset: 2; tab-width: 2 -*- */
/*
 * map_read.c
 * Copyright (C) Mike Crash 2009 <mike@mikecrash.com>
 * 
 * main.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * main.c is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdlib.h>
#include <typeio.h>
#include <string.h>
#include "map_common.h"
#include "map_read.h"

// map variables
mcmap *map = NULL;

// *****************************
// map read
// *****************************
int map_readinfo(mcmap *map)
{
  map_lock(map);
  fseek(map->f, 0, SEEK_SET);
  fread_char(map->info.header, sizeof(map->info.header), map->f);
  if ((map->info.header[0] != 'M') ||
      (map->info.header[1] != 'C') ||
      (map->info.header[2] != 'M') ||
      (map->info.header[3] != 'A') ||
      (map->info.header[4] != 'P') ||
      (map->info.header[5] != 0))
  {
    map->error = MERR_HEADER;
    map_unlock(map);
    return -1;
  }
  fread_int(&map->info.mapversion, 1, map->f);
  if (map->info.mapversion != MAP_VERSION)
  {
    map->error = MERR_VERSION;
    map_unlock(map);
    printf("Wrong map version\n");
    return -1;
  }
  fread_long(&map->info.sectorsize, 1, map->f);
  if (map->info.sectorsize != MAPSECTRECT)
  {
    map->error = MERR_VERSION;
    map_unlock(map);
    printf("Wrong map sector size\n");
    return -1;
  }
  fread_long(&map->info.flags, 1, map->f);
  fread_long(&map->info.nodesindex, 1, map->f);
  fread_long(&map->info.nodes, 1, map->f);
  fread_long(&map->info.planesindex, 1, map->f);
  fread_long(&map->info.planes, 1, map->f);
  fread_long(&map->info.locsindex, 1, map->f);
  fread_long(&map->info.locs, 1, map->f);
  fread_long(&map->info.poisindex, 1, map->f);
  fread_long(&map->info.pois, 1, map->f);
  fread_long(&map->info.routesindex, 1, map->f);
  fread_long(&map->info.routes, 1, map->f);
  fread_long(&map->info.contoursindex, 1, map->f);
  fread_long(&map->info.contours, 1, map->f);
  fread_long(&map->info.zoomindex, 1, map->f);
  fread_long(&map->info.zoom, 1, map->f);
  fread_long(&map->info.countries, 1, map->f);
  fread_long(&map->info.cities, 1, map->f);
  fread_long(&map->info.streets, 1, map->f);
  fread_long(&map->info.addresses, 1, map->f);
  fread_long(&map->info.text, 1, map->f);
  map_unlock(map);
  //TODO check pointers
  return 0;
}

// *****************************
// map open and close
// *****************************

int map_open(char *mapname)
{
  map_close();
  map = (mcmap*)malloc(sizeof(mcmap));
  memset(map, 0, sizeof(mcmap));
  // open map file
  map->f = fopen(mapname, "r");
  mcmutex_init(&map->mutex);
  if (map->f == NULL)
    return -1;
  //read header
  if (map_readinfo(map) != 0)
  {
    fclose(map->f);
    free(map);
    map = NULL;
    return -1;
  }
  if (map_text_init() != 0)
  {
    fclose(map->f);
    free(map);
    map = NULL;
    return -1;
  }
  if (map_nodes_init() != 0)
  {
    map_text_clear();
    fclose(map->f);
    free(map);
    map = NULL;
    return -1;
  }
  if (map_planes_init() != 0)
  {
    map_nodes_clear();
    map_text_clear();
    fclose(map->f);
    free(map);
    map = NULL;
    return -1;
  }
  if (map_pois_init() != 0)
  {
    map_planes_clear();
    map_nodes_clear();
    map_text_clear();
    fclose(map->f);
    free(map);
    map = NULL;
    return -1;
  }
  if (map_locs_init() != 0)
  {
    map_pois_clear();
    map_planes_clear();
    map_nodes_clear();
    map_text_clear();
    fclose(map->f);
    free(map);
    map = NULL;
    return -1;
  }
  if (map_address_init() != 0)
  {
    map_locs_clear();
    map_pois_clear();
    map_planes_clear();
    map_nodes_clear();
    map_text_clear();
    fclose(map->f);
    free(map);
    map = NULL;
    return -1;
  }
  return 0;
}

int map_close(void)
{
  if (map == NULL)
    return -1;
  map_address_clear();
  map_pois_clear();
  map_planes_clear();
  map_nodes_clear();
  map_text_clear();
  if (map->f)
    fclose(map->f);
  mcmutex_destroy(&map->mutex);
  free(map);
  map = NULL;
  return 0;
}
