/* -*- Mode: C; indent-tabs-mode: s; c-basic-offset: 2; tab-width: 2 -*- */
/*
 * map_city.c
 * Copyright (C) Mike Crash 2009 <mike@mikecrash.com>
 * 
 * main.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * main.c is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdlib.h>
#include <string.h>
#include "typeio.h"
#include "diacritics.h"
#include "functions.h"
#include "map_common.h"
#include "map_text.h"
#include "map_street.h"

mapaddrsearch *map_street_start(char *name, long parent, long filepos)
{
  mapaddrsearch *as = (mapaddrsearch *)malloc(sizeof(mapaddrsearch)+strlen(name)+1);
  as->type = MAS_STREET;
  as->parent = parent;
  as->filepos = filepos;
  strcpy(as->name, name);
  as->index = 0;
  return as;
}

tstreet *map_street_get(mapaddrsearch *as)
{
  tstreet tts;

  if (as->filepos == 0xffffffff)
    return NULL;

  do
  {
    map_lock(map);
    long filepos = map->info.streets + map_part_size + as->filepos;
    //printf("Street %lx %lx\n", map->info.streets, filepos);
    fseek(map->f, filepos, SEEK_SET);
    fread_long(&tts.id, 1, map->f);
    if (tts.id != 0)
    {
      fread_long(&tts.city, 1, map->f);
      fread_long(&tts.pos.lat, 1, map->f);
      fread_long(&tts.pos.lon, 1, map->f);
      fread_long(&tts.namefilepos, 1, map->f);
      fread_long(&tts.addressesfilepos, 1, map->f);
      map_unlock(map);
      if (tts.city!=as->parent)
        break;
      char *name = map_gettext(tts.namefilepos);
      if (name == NULL)
        break;
      //printf("%ld from %ld, filepos %ld - %ld,%ld \"%s\"\n", tts.id, tts.city, tts.addressesfilepos, tts.pos.lat, tts.pos.lon, tmp);

      tts.filepos = as->filepos;
      as->filepos += map_street_size;
      
      int l = strlen(name) + 1;
      char *cname = (char *)malloc(l);
      utf8_to_ascii(name, cname);
      as->index++;
      
      if ((strlen(as->name) == 0) || (strcasepos(cname, as->name) == 0))
      {
        free(cname);
        tstreet *ts = (tstreet *)malloc(sizeof(tstreet) + l);
        memcpy(ts, &tts, sizeof(tts));
        strcpy(ts->name, name);
        map_leavetext(tts.namefilepos);
        return ts;
      }
      map_leavetext(tts.namefilepos);
      free(cname);
    }
    else
    {
      map_unlock(map);
    }
  }
  while (tts.id!=0);
  return NULL;
}

void map_street_stop(mapaddrsearch *as)
{
  if (as)
  {
    free(as);
  }
}


