/* -*- Mode: C; indent-tabs-mode: s; c-basic-offset: 2; tab-width: 2 -*- */
/*
 * map_ways.c
 * Copyright (C) Mike Crash 2009 <mike@mikecrash.com>
 * 
 * main.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * main.c is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdlib.h>
#include <typeio.h>
#include <string.h>
#include "mcconf.h"
#include "map_text.h"
#include "map_common.h"

mtext *readtext(long filepos)
{
  map_lock(map);
  long pos = map->info.text + map_part_size + filepos;
  //printf("%ld %ld %lx\n", way->id, way->streetfilepos, filepos);
  fseek(map->f, pos, SEEK_SET);
  int size;
  fread_int(&size, 1, map->f);
  //TODO check wrong size
  mtext *mc = (mtext*)malloc(sizeof(mtext) + size);
  mc->prev = NULL;
  mc->next = NULL;
  mc->filepos = filepos;
  mc->used = 0;
  fread_char(mc->text, size, map->f);
  map_unlock(map);
  return mc;
}

mtext *gettext(long filepos)
{
  //check cache
  //printf("Get text %ld\n", filepos);
  mtext *mc = map->text.first;
  while (mc)
  {
    //printf("Find way %ld\n",wc->way.id);
    if (mc->filepos == filepos)
    {
      //bring it to top and return
      if (mc != map->text.first)
      {
        if (map->text.last == mc)
          map->text.last = mc->prev;
        if (mc->prev)
          mc->prev->next = mc->next;
        if (mc->next)
          mc->next->prev = mc->prev;
        map->text.first->prev = mc;
        mc->next = map->text.first;
        mc->prev = NULL;
        map->text.first = mc;
      }
      return mc;
    }
    mc = mc->next;
  }

  //not found, read it from file
  mc = readtext(filepos);
  if (mc == NULL)
    return NULL;

  if (map->text.first)
  {
    map->text.first->prev = mc;
  }
  map->text.first = mc;
  if (map->text.last == NULL)
    map->text.last = mc;
  map->text.count++;

  //free cache for maximum count
  mtext *mc2 = map->text.last;
  while (mc2 && (mc2 != map->text.first) && (map->text.count>config->limit_text))
  {
    if (mc2->used == 0)
    {
      printf("Free text %ld\n", mc2->filepos);
      mtext *mc3 = mc2->prev;
      if (mc2->next == NULL)
      {
        map->text.last = mc3;
      }
      else
      {
        mc2->next->prev = mc3;
      }
      if (mc3)
      {
        mc3->next = mc2->next;
      }
      else
      {
        map->text.first = mc2->next;
      }
      free(mc2);
      map->text.count--;
      mc2 = mc3;
    }
    else
    {
      mc2 = mc2->prev;
    }
  }

  return mc;
}

//**************

int map_text_init()
{
  map_part part; 
  mcmutex_init(&map->text.mutex);
  //check correct structure
  map_lock(map);
  fseek(map->f, map->info.text, SEEK_SET);
  map_readpart(&part);
  if (part.partid!=MPID_TEXT)
  {
    map_unlock(map);
    return -1;
  }
  map_unlock(map);
  return 0;
}

int map_text_clear()
{
  mtext *tc = map->text.first;
  while (tc)
  {
    mtext *tc2 = tc->next;
    free(tc);
    tc = tc2;
  }
  map->text.count = 0;
  map->text.first = NULL;
  map->text.last = NULL;
  mcmutex_destroy(&map->text.mutex);
  return 0;
}

//**************

char *map_gettext(long filepos)
{
  mtext *mc = gettext(filepos);
  if (mc == NULL)
    return NULL;
  mc->used++;
  return mc->text;
}

void map_leavetext(long filepos)
{
  mtext *mc = gettext(filepos);
  if (mc == NULL)
    return;
  if (mc->used>0)
    mc->used--;
}

