/* -*- Mode: C; indent-tabs-mode: s; c-basic-offset: 2; tab-width: 2 -*- */
/*
 * db_mysql.cpp
 * Copyright (C) Mike Crash 2009 <mike@mikecrash.com>
 *
 * main.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * main.c is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "db_mysql.h"

MYSQL *mysql;
MYSQL_RES *results;
MYSQL_ROW record;

int sql_connect(const char *server, const char *user, const char *password, const char *db)
{
   if (mysql_library_init(0, NULL, NULL))
   {
     printf("Cannot init MySQL library\n");
     return -1;
   }

   mysql = mysql_init(NULL);
   if (mysql==NULL)
   {
     printf("Cannot init MySQL\n");
     return -1;
   }

   if (!mysql_real_connect(mysql, server, user, password, db, 3306, NULL, 0))
   {
     printf("Cannot connect to MySQL\n");
     return -1;
   }

   if (mysql_query(mysql, "SET CHARACTER SET utf8") != 0)
   {
     printf("Cannot set character set\n");
     return -1;
   }

   if (mysql_query(mysql, "SET character_set_connection=utf8") != 0)
   {
     printf("Cannot set connection character set\n");
     return -1;
   }

   if (mysql_query(mysql, "SET collation_connection=utf8_bin") != 0)
   {
     printf("Cannot set connection collation\n");
     return -1;
   }

   return 0;
}

//create schema
int sql_init(const char *schema)
{
  char *buf;
  FILE *f;
  int size, len;

  //load the file to memory
  if ((f = fopen(schema, "r")) == NULL)
  {
    return -1;
  }
  fseek (f , 0 , SEEK_END);
  size = (int)ftell(f);
  rewind (f);
  buf = (char *)malloc(size+1);
  len = (int)fread(buf, 1, size, f);
  if (len != size)
  {
    fclose(f);
    free(buf);
    return -1;
  }
  buf[size] = 0;
  fclose(f);

  //the query
  char *buf2 = buf;
  char *sub;
  while ((sub = strstr(buf2, ";")) != NULL)
  {
    *sub = 0;
    if (mysql_query(mysql, buf2) != 0)
    {
      const char *err;
      if ((err = mysql_error(mysql)) != NULL)
      {
        printf("%s\n", err);
      }

      free(buf);
      return -1;
    }
    buf2 = sub;
    buf2++;
  }

  //free
  free(buf);
  return 0;
}

int sql_query(const char *sql)
{
   if (mysql_query(mysql, sql) != 0)
   {
      const char *err;
      if((err = mysql_error(mysql)) != NULL)
      {
        printf("%s\n", err);
      }
      return -1;
   }
   return 0;
}

sql_result *sql_storeresult()
{
  return mysql_store_result(mysql);
}

sql_result *sql_useresult()
{
  return mysql_use_result(mysql);
}

sql_row sql_fetch_row(sql_result *result)
{
  return mysql_fetch_row(result);
}

unsigned long *sql_fetch_lengths(sql_result *result)
{
  return mysql_fetch_lengths(result);
}

int sql_numrows(sql_result *result)
{
  return (int)mysql_num_rows(result);
}

int sql_affectedrows(void)
{
  return (int)mysql_affected_rows(mysql);
}

unsigned long long sql_insertid()
{
  return mysql_insert_id(mysql);
}

char *sql_record_str(sql_row record, int index)
{
  return record[index];
}

char sql_record_char(sql_row record, int index)
{
  char ch;
  ch = (char)atoi(record[index]);
  return ch;
}

int sql_record_int(sql_row record, int index)
{
  return atoi(record[index]);
}

long sql_record_long(sql_row record, int index)
{
  return atol(record[index]);
}

long long sql_record_longlong(sql_row record, int index)
{
  //return strtoll(record[index], NULL, 10);
  long long value;
  sscanf(record[index], "%lld", &value);
  return value;
}

int sql_freeresult(sql_result *result)
{
   mysql_free_result(result);
   return 0;
}

int sql_close()
{
  mysql_close(mysql);
  mysql_library_end();
  return 0;
}

int sql_escape(const char *from, char *to, unsigned long len)
{
  mysql_real_escape_string(mysql, to, from, len);
  return 0;
}

