/* -*- Mode: C; indent-tabs-mode: s; c-basic-offset: 2; tab-width: 2 -*- */
/*
 * map.c
 * Copyright (C) Mike Crash 2009 <mike@mikecrash.com>
 * 
 * main.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * main.c is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef _MCMAPCREATEADDRESS_H
#define _MCMAPCREATEADDRESS_H

#include <map.h>

typedef struct countrycache
{
  long id;
  struct countrycache *next;
} countrycache;

typedef struct citycache
{
  long id;   //city id
  long bid;  //boundary id from relation
  long country;
  int renamed;
  long lat;
  long lon;
  citytypes citytype;
  long filepos;
  struct citycache *next;
  char name[];
} citycache;

typedef struct streetcache
{
  long id;
  long city;
  long lat;
  long lon;
  long filepos;
  struct streetcache *next;
  char name[];
} streetcache;

typedef struct addresscache
{
  long id;
  long lat;
  long lon;
  long street;
  struct addresscache *next;
  long filepos;
  char name[];
} addresscache;

citycache *map_nearestcity(long lat, long lon, citytypes type, int country);
unsigned long map_streetpos(int id);
int free_address(void);

int map_addcity(long bid, char *name, int country, citytypes citytype, long lat, long lon);
int map_addstreet(int country, char *city, char *street, long lat, long lon);
int map_addaddress(int country, char *city, char *street, char* housenumber, long lat, long lon);

int map_sort_countries(void);
int map_check_cities(void);
long map_sort_cities(void);
long map_sort_streets(void);
long map_sort_addresses(void);
int map_write_countries(FILE *f);
int map_write_cities(FILE *f, long citiessize);
int map_write_streets(FILE *f, long streetssize);
int map_write_addresses(FILE *f, long addressessize);


#endif
