/* -*- Mode: C; indent-tabs-mode: s; c-basic-offset: 2; tab-width: 2 -*- */
/*
 * map_boundary.c
 * Copyright (C) Mike Crash 2009 <mike@mikecrash.com>
 *
 * main.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * main.c is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdlib.h>
#include <string.h>
#include "typeio.h"
#include "db_mysql.h"
#include "countries.h"
#include "polygon.h"
#include "map_boundary.h"
#include "map_nodes.h"
#include "map_common.h"
#include "map_relation.h"
#include "map_ways.h"

//
//boundary file
//

typedef struct
{
  char header[6];   //string MCBND
  int  version;
  long count;
} boundary_fileheader;

char BND_HEADER[] = "MCBND";
#define BND_VERSION 1

//structure of one item in file
/*typedef struct
{
  char bid;         //=a4 control byte
  long id;
  int  type;
  int  name_len[];  //with ending zero
  long count;
  char name[];
  map_position points[];
} boundary_fileboundary;
*/

#define BND_BID 0xa4

//cross table for admin_level

typedef struct
{
  int country;
  int admin_level_state;
  int admin_level_region;
  int admin_level_district;
  int admin_level_city;
} admin_level_btype;

admin_level_btype admin_levels[] = {
  {0,   4, 6, 7, 8},
  {250, 4, 6, 7, 8},  //czech
  {784, 4, 6, 8, 10}, //slovakia
  {826, 4, 5, 8, 10}  //slovenia
};


//
//cache
//

typedef struct boundary_cache
{
  struct boundary_cache *next;
  boundary bnd;
} boundary_cache;


long n_boundaries = 0;

boundary_type map_boundarytype(int country, int admin_level)
{
  //check country
  if (admin_level == 2)
    return BT_COUNTRY;
  //check cross table
  int i;
  int cnt = sizeof(admin_levels) / sizeof(admin_level_btype);
  for (i=1; i<cnt; i++)
  {
    if (country == admin_levels[i].country)
    {
      if (admin_level == admin_levels[i].admin_level_state)
        return BT_STATE;
      if (admin_level == admin_levels[i].admin_level_region)
        return BT_REGION;
      if (admin_level == admin_levels[i].admin_level_district)
        return BT_DISTRICT;
      if (admin_level == admin_levels[i].admin_level_city)
        return BT_CITY;
      return BT_ERROR;
    }
  }

  //default values
  if (admin_level == admin_levels[0].admin_level_state)
    return BT_STATE;
  if (admin_level == admin_levels[0].admin_level_region)
    return BT_REGION;
  if (admin_level == admin_levels[0].admin_level_district)
    return BT_DISTRICT;
  if (admin_level == admin_levels[0].admin_level_city)
    return BT_CITY;

  return BT_ERROR;
}

int map_addboundary(boundary *boundary)
{
  int i, j;
  int res;
  long namelen = boundary->name==NULL ? 0 : (long)strlen(boundary->name);
  char *name = (char*)malloc(namelen*2+1);
  if (namelen)
  {
    sql_escape(boundary->name, name, namelen);
  }
  else
  {
    name[0] = 0;
  }
  char *arr = multipolygon2ascii(boundary->mp);
  if (arr == NULL)
    return -1;
  char *sql = (char*)malloc(strlen(arr)+namelen+200);
  
  sprintf(sql, "INSERT INTO mcm_boundaries (typ, country, name, lat1, lon1, lat2, lon2, polygon) VALUES (%d, %d, '%s', %ld, %ld, %ld, %ld, %s)",
    boundary->type, boundary->country, name, boundary->lat1, boundary->lon1, boundary->lat2, boundary->lon2, arr);
  free(name);
  free(arr);
  res = sql_query(sql);
  free(sql);
  return res;
}

boundary* map_boundaryinside(map_position *pos, boundary_type type)
{
  char sql[500];
  sprintf(sql, "SELECT id, typ, country, name, lat1, lon1, lat2, lon2, polygon FROM `mcm_boundaries` WHERE (lat1<=%ld) AND (lon1<=%ld) AND (lat2>=%ld) AND (lon2>=%ld) AND (typ=%d)",
                pos->lat, pos->lon, pos->lat, pos->lon, (int)type);
  if (sql_query(sql) != 0)
    return NULL;
  sql_result *result = sql_storeresult();
  if (result == NULL)
    return NULL;
  sql_row row;
  while ((row = sql_fetch_row(result)) != NULL)
  {
    unsigned long *lengths = sql_fetch_lengths(result);
    long id = sql_record_long(row, 0);
    char *arr = sql_record_str(row, 8);
    map_multipolygon *mp = ascii2multipolygon(arr);

    if (map_pointinmultipolygon(pos, mp) == 1)
    {
      boundary *bnd = (boundary *)malloc(sizeof(boundary));
      bnd->id = id;
      bnd->type = (boundary_type)sql_record_int(row, 1);
      bnd->country = sql_record_int(row, 2);
      bnd->lat1 = sql_record_long(row, 4);
      bnd->lon1 = sql_record_long(row, 5);
      bnd->lat2 = sql_record_long(row, 6);
      bnd->lon2 = sql_record_long(row, 7);
      bnd->name = (char*)malloc(lengths[3]+1);
      strcpy(bnd->name, sql_record_str(row, 3));
      bnd->mp = mp;
      sql_freeresult(result);
      return bnd;
    }

    map_freemultipolygon(mp);
  }
  sql_freeresult(result);
  return NULL;
}

int map_saveboundary(char *filename, boundary_type type)
{
  FILE *f;
  char bid = BND_BID;
  int i,j;
  char sql[500];

  if ((f = fopen(filename, "wb")) == NULL)
  {
    printf("Error opening file %s\n", filename);
    return -1;
  }
  printf("Save boundaries\n");

  if (type==(boundary_type)-1)
    sprintf(sql, "SELECT id, typ, country, name, polygon FROM `mcm_boundaries`");
  else
    sprintf(sql, "SELECT id, typ, country, name, polygon FROM `mcm_boundaries` WHERE (typ=%d)",
            (int)type);
  if (sql_query(sql) != 0)
    return -1;
  sql_result *result = sql_storeresult();
  if (result == NULL)
    return -1;
  long count = sql_numrows (result);

  //write header
  fwrite_char(BND_HEADER, 6, f);
  i = BND_VERSION;
  fwrite_int(&i, 1, f);
  fwrite_long(&count, 1, f);

  sql_row row;
  while ((row = sql_fetch_row(result)) != NULL)
  {
    unsigned long *lengths = sql_fetch_lengths(result);
    long id = sql_record_long(row, 0);
    int typ = (boundary_type)sql_record_int(row, 1);
    int country = sql_record_long(row, 2);
    char *name = sql_record_str(row, 3);
    char *arr = sql_record_str(row, 4);
    map_multipolygon *mp = ascii2multipolygon(arr);

    if ((type==-1) || ((boundary_type)typ == type))
    {
      char s[1000];
      map_log(LOG_INFO, "Write boundary %s\n", name);
      //bid test sync
      fwrite_char(&bid, 1, f);
      //id
      fwrite_long(&id, 1, f);
      //type
      fwrite_int(&typ, 1, f);
      //country
      fwrite_int(&country, 1, f);
      //name
      i = strlen(name) + 1;
      fwrite_int(&i, 1, f);
      fwrite_char(name, i, f);
      //multipolygon
      fwrite_int(&(mp->count), 1, f);
      //polygons
      for (i=0; i<mp->count; i++)
      {
        fwrite_int(&mp->polygon[i]->count, 1, f);
        for (j=0; j<mp->polygon[i]->count; j++)
        {
          fwrite_long(&mp->polygon[i]->points[j].lat, 1, f);
          fwrite_long(&mp->polygon[i]->points[j].lon, 1, f);
        }
      }
    }
  }
  sql_freeresult(result);

  fclose(f);
  return 0;
}


int map_loadboundaries(char *filename)
{
  FILE *f;
  int i,j;
  long fcount;
  char s[200];

  if ((f = fopen(filename, "rb")) == NULL)
  {
    printf("Error opening file %s\n", filename);
    return -1;
  }

  //check header
  fread_char(s, 6, f);
  if (strcmp(BND_HEADER, s) != 0)
  {
    fclose(f);
    printf("Not a boundary file: %s\n", filename);
    return -1;
  }
  fread_int(&i, 1, f);
  if (i!=BND_VERSION)
  {
    fclose(f);
    printf("Wrong version of boundary file: %s\n", filename);
    return -1;
  }
  fread_long(&fcount, 1, f);

  int bcount = 0;

  //read data
  while (!feof(f) && (bcount<fcount))
  {
    char bid;
    boundary boundary;

    fread_char(&bid, 1, f);
    if (feof(f))
    {
      break;
    }
    if ((bid&0xff) != BND_BID)
    {
      fclose(f);
      printf("Error in boundary file %2x: %s\n", bid, filename);
      return -1;
    }
    fread_long(&boundary.id, 1, f);
    fread_int(&i, 1, f);
    boundary.type = (boundary_type)i;
    fread_int(&i, 1, f);
    boundary.country = i;
    fread_int(&i, 1, f);
    boundary.name = (char*)malloc(i);
    fread_char(boundary.name, i, f);
    //sprintf(s,"%ld Name: %s\n", n_boundaries, bc->boundary.name);
    //map_log(s);
    fread_int(&i, 1, f);
    boundary.mp = (map_multipolygon*)malloc(sizeof(map_multipolygon)+i*sizeof(map_polygon*));
    boundary.mp->count = i;
    for (i=0; i<boundary.mp->count; i++)
    {
      int cnt;
      fread_int(&cnt, 1, f);
      map_polygon *p = (map_polygon *)malloc(sizeof(map_polygon)+sizeof(map_position)*cnt);
      boundary.mp->polygon[i] = p;
      p->count = cnt;
      for (j=0; j<cnt; j++)
      {
        fread_long(&p->points[j].lat, 1, f);
        fread_long(&p->points[j].lon, 1, f);
        //printf("%d: %ld,%ld\n", j, p->points[j].lat, p->points[j].lon);
      }
    }
    map_polygonarea(boundary.mp->polygon[0], &boundary.lat1, &boundary.lon1, &boundary.lat2, &boundary.lon2);
    map_log(LOG_DEBUG, "Boundary '%s' %d  country %d (%ld,%ld -> %ld,%ld)\n", boundary.name, boundary.type, boundary.country, boundary.lat1, boundary.lon1, boundary.lat2, boundary.lon2);

    //store to database
    map_addboundary(&boundary);

    //free
    free(boundary.name);
    map_freemultipolygon(boundary.mp);

    n_boundaries++;
    bcount++;
  }

  map_log(LOG_INFO, "Imported %d boundaries\n", bcount);

  fclose(f);
  return 0;
}

int map_process_boundaries()
{
  map_log(LOG_INFO, "Process boundaries\n");
  printf("Process boundaries 0%%\r");
  fflush(stdout);

  long progress = -1;
  char sql[200];
  sprintf(sql, "SELECT COUNT(*) FROM `osm_relations_tags` WHERE name='boundary' AND val='administrative'");
  if (sql_query(sql) != 0)
    return -1;
  sql_result *result1 = sql_storeresult();
  if (result1 == NULL)
    return -1;
  sql_row row1;
  if ((row1 = sql_fetch_row(result1)) == NULL)
    return -1;
  long size = sql_record_long(row1, 0);
  sql_freeresult(result1);

  //find all boundaries in relations
  long numrows = SQLCHUNK;
  long start = 0;
  while (numrows == SQLCHUNK)
  {
    long index = 0;
    sprintf(sql, "SELECT id FROM `osm_relations_tags` WHERE name='boundary' AND val='administrative' LIMIT %ld,%d", start, SQLCHUNK);
    if (sql_query(sql) != 0)
      return -1;
    result1 = sql_storeresult();
    if (result1 == NULL)
      return -1;
    while ((row1 = sql_fetch_row(result1)) != NULL)
    {
      long long id = sql_record_longlong(row1, 0);
      int admin_level;
      boundary_type bt;
      char *name;
      index++;
      if (map_getrelationtag_int(id, "admin_level", &admin_level) != 0)
        continue;
      if (map_getrelationtag_str(id, "name", &name) != 0)
        continue;
      osm_relation *relation;
      if (map_getrelation(id, &relation) == 0)
      {
        map_multipolygon *mp;
        if (relation_multipolygon(relation, &mp) == 0)
        {
          map_multipolygon **mps;
          int polycnt;
          if (map_multipolygons_split(mp, &polycnt, &mps) == 0)
          {
            for (int i=0; i<polycnt; i++)
            {
              boundary bnd;
              bnd.id = 0;
              bnd.country = 0;
              bnd.name = name;
              bnd.mp = mps[i];
              map_polygonarea(mps[i]->polygon[0], &bnd.lat1, &bnd.lon1, &bnd.lat2, &bnd.lon2);
              if (admin_level == 2)
              {
                bnd.type = BT_COUNTRY;
                char *iso;
                
                if (map_getrelationtag_str(id, "country_code_iso3166_1_alpha_2", &iso) == 0)
                {
                  bnd.country = country_iso2code(iso);
                  free(iso);
                }
                if (map_getrelationtag_str(id, "iso3166-1:alpha2", &iso) == 0)
                {
                  bnd.country = country_iso2code(iso);
                  free(iso);
                }
                if ((bnd.country == 0) && (map_getrelationtag_str(id, "country_code_fips", &iso) == 0))
                {
                  bnd.country = country_iso2code(iso);
                  free(iso);
                }
                if (map_getrelationtag_str(id, "ISO3166-1", &iso) == 0)
                {
                  bnd.country = country_iso2code(iso);
                  free(iso);
                }
                if ((bnd.country == 0) && (map_getrelationtag_str(id, "int_name", &iso) == 0))
                {
                  bnd.country = country_osmname2code(iso);
                  free(iso);
                }
                if ((bnd.country == 0) && (map_getrelationtag_str(id, "name:en", &iso) == 0))
                {
                  bnd.country = country_osmname2code(iso);
                  free(iso);
                }
                if ((bnd.country == 0) && (map_getrelationtag_str(id, "short_name", &iso) == 0))
                {
                  bnd.country = country_osmname2code(iso);
                  free(iso);
                }
                if (bnd.country == 0)
                {
                  bnd.country = country_osmname2code(name);
                }
                if (bnd.country == 0)
                  map_log(LOG_WARNING, "Cannot get country code for %s, relation %ld, please contact author\n", name, id);
                else
                  map_addboundary(&bnd);
              }
              else
              {
                map_position pos;
                pos.lat = (bnd.lat1+bnd.lat2)/2;
                pos.lon = (bnd.lon1+bnd.lon2)/2;
                boundary *b = map_boundaryinside(&pos, BT_COUNTRY);
                if (b)
                {
                  bt = map_boundarytype(b->country, admin_level);
                  bnd.type = bt;
                  bnd.country = b->country;
                  map_freemultipolygon(b->mp);
                  free(b);
                  if (bt != BT_ERROR)
                  {
                    map_addboundary(&bnd);
                  }
                }
              }
              map_freemultipolygon(mps[i]);
            }
            free(mps);
          }
        }
        free(relation);
      }
      free(name);
      long prg = (start+index) * 100 / size;
      if (prg!=progress)
      {
        progress = prg;
        printf("Process boundaries %ld%%\r", prg);
        fflush(stdout);
      }
    }

    sql_freeresult(result1);
    start += index;
    numrows = index;
  }
  printf("\n");
  map_log(LOG_INFO, "Process boundaries done\n");
  return 0;
}

int map_freeboundary(boundary *bnd)
{
  if (bnd==NULL)
    return -1;
  map_freemultipolygon(bnd->mp);
  free(bnd);
  return 0;
}

int map_process_citylimit()
{
  map_log(LOG_INFO, "Process city limit\n");
  printf("Process citylimit 0%%\r");
  fflush(stdout);

  long progress = -1;
  char sql[200];
  sprintf(sql, "SELECT COUNT(*) FROM `osm_ways_tags` WHERE name='landuse' AND val='residential'");
  if (sql_query(sql) != 0)
    return -1;
  sql_result *result1 = sql_storeresult();
  if (result1 == NULL)
    return -1;
  sql_row row1;
  if ((row1 = sql_fetch_row(result1)) == NULL)
    return -1;
  long size = sql_record_long(row1, 0);
  sql_freeresult(result1);

  //find all residential as boundaries
  long numrows = SQLCHUNK;
  long start = 0;
  while (numrows == SQLCHUNK)
  {
    long index = 0;
    sprintf(sql, "SELECT id FROM `osm_ways_tags` WHERE name='landuse' AND val='residential' LIMIT %ld,%d", start, SQLCHUNK);
    if (sql_query(sql) != 0)
      return -1;
    result1 = sql_storeresult();
    if (result1 == NULL)
      return -1;
    while ((row1 = sql_fetch_row(result1)) != NULL)
    {
      long long id = sql_record_longlong(row1, 0);
      index++;
      map_multipolygon *mp = map_waymultipolygon(id);
      if (mp)
      {
        boundary bnd;
        bnd.id = 0;
        bnd.type = BT_TRAFFIC_CITYLIMIT;
        bnd.country = 0;
        bnd.name = NULL;
        bnd.mp = mp;
        map_polygonarea(mp->polygon[0], &bnd.lat1, &bnd.lon1, &bnd.lat2, &bnd.lon2);
        map_addboundary(&bnd);
        map_freemultipolygon(mp);
      }
      long prg = (start+index) * 100 / size;
      if (prg!=progress)
      {
        progress = prg;
        printf("Process citylimit %ld%%\r", prg);
        fflush(stdout);
      }
    }

    sql_freeresult(result1);
    start += index;
    numrows = index;
  }
  printf("\n");
  map_log(LOG_INFO, "Process city limit done\n");
  return 0;
}
