/* -*- Mode: C; indent-tabs-mode: s; c-basic-offset: 2; tab-width: 2 -*- */
/*
 * map_nodes.c
 * Copyright (C) Mike Crash 2009 <mike@mikecrash.com>
 *
 * main.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * main.c is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdlib.h>
#include <string.h>
#include <typeio.h>
#include "db_mysql.h"
#include "diacritics.h"
#include "map_common.h"
#include "map_text.h"
#include "map_nodes.h"
#include "map_tags.h"
#include "map_convert.h"
#include "map_boundary.h"
//#include "map_address.h"
//#include "map_poi.h"

long node_min;
long node_max;

long node_progress = 0;

/*
long get_nodeindex(long id)
{
  long idx = 0;
  nodecache *nc = map_nodes;
  nodecache *nc2 = NULL;
  //find segment of sorted nodes
  while (nc)
  {
    if ((nc->count>0) && (nc->nodes[0].id>id))
    {
      break;
    }
    //skip increasing for the first time
    if (nc2)
      idx += NODECACHEBLOCK;
    nc2 = nc;
    nc = nc->next;
  }

  if (nc2 == NULL)
    return -1;

  //find inside the block
  int l = 0;
  int r = nc2->count - 1;
  while (l <= r)
  {
    int c = (l+r)/2;
    if (nc2->nodes[c].id == id)
      return (idx+c);

    if (id < nc2->nodes[c].id)
    {
      r = c - 1;
    }
    else
    {
      l = c + 1;
    }
  }

  return -1;
}
*/

int map_nodeposition(long long nodeid, map_position *pos)
{
  char sql[100];
  sprintf(sql, "SELECT lat, lon FROM osm_nodes WHERE id='%lld'", nodeid);
  if (sql_query(sql) != 0)
    return -1;

  sql_result *result;
  if ((result = sql_storeresult()) == NULL)
    return -1;
  sql_row row;
  if ((row = sql_fetch_row(result)) == NULL)
  {
    sql_freeresult(result);
    return -1;
  }
  pos->lat = sql_record_long(row, 0);
  pos->lon = sql_record_long(row, 1);
  sql_freeresult(result);
  return 0;
}

map_tags *map_getnodetags(long long id)
{
  return get_tags("osm_nodes_tags", id);
}

int node_used(long long nodeid)
{
  int res = 0;
  char sql[100];
  sprintf(sql, "SELECT used FROM osm_nodes WHERE id='%lld'", nodeid);

  if (sql_query(sql) != 0)
    return -1;

  sql_result *result;
  if ((result = sql_storeresult()) == NULL)
    return -1;
  sql_row row;
  if ((row = sql_fetch_row(result)) == NULL)
  {
    sql_freeresult(result);
    return -1;
  }

  res = sql_record_int(row, 0);
  sql_freeresult(result);
  return res;
}

int add_country(int country)
{
  char sql[100];

  //check it exists in database
  sprintf(sql, "SELECT id FROM mcm_country WHERE id='%d'", country);

  if (sql_query(sql) != 0)
    return -1;

  sql_result *result;
  if ((result = sql_storeresult()) == NULL)
    return -1;
  sql_row row;
  if ((row = sql_fetch_row(result)) != NULL)
  {
    sql_freeresult(result);
    return 0;
  }
  sql_freeresult(result);

  //insert new country
  sprintf(sql, "INSERT INTO mcm_country (id) VALUES (%d)", country);
  if (sql_query(sql) != 0)
    return -1;

  return 0;
}

int add_city(int country, int boundary, citytypes citytype, map_position *pos, char *name)
{
  int res = 0;
  long len = (long)strlen(name);
  char *sql = (char *)malloc(len*2 + 500);

  //update size in country
  sprintf(sql, "UPDATE mcm_country SET size=size+%d WHERE id=%d", map_city_size, country);
  if (sql_query(sql) != 0)
  {
    free(sql);
    return -1;
  }

  //add city
  long tid = gettextid(name);
  char *namec = (char*)malloc(len+1);
  utf8_to_ascii(name, namec, 1);
  sprintf(sql, "INSERT INTO mcm_city (country, boundary, typ, lat, lon, name, namec) VALUES (%d, %d, %d, %ld, %ld, %ld, '%s')",
    country, boundary, citytype, pos->lat, pos->lon, tid, namec);
  if (sql_query(sql) != 0)
    res = -1;
  free(sql);
  return res;
}

int map_process_cities()
{
  long long progress = 0;
  if (sql_query("SELECT COUNT(*) FROM `osm_nodes_tags` WHERE name='place' AND (val='city' OR val='village' OR val='town')") != 0)
    return -1;
  sql_result *result = sql_storeresult();
  if (result == NULL)
    return -1;
  sql_row row;
  if ((row = sql_fetch_row(result)) == NULL)
    return -1;
  long long count = sql_record_longlong(row, 0);
  sql_freeresult(result);
  map_log(LOG_INFO, "Process cities\n");
  printf("Process cities 0%%\r");

  int numrows = SQLCHUNK;
  int start = 0;
  while (numrows == SQLCHUNK)
  {
    char sql[200];
    sprintf(sql, "SELECT id, val FROM `osm_nodes_tags` WHERE name='place' AND (val='city' OR val='village' OR val='town') LIMIT %d,%d", start, SQLCHUNK);
    if (sql_query(sql) != 0)
      return -1;
    sql_result *result1 = sql_storeresult();
    if (result1 == NULL)
      return -1;
    sql_row row1;
    numrows = 0;
    while ((row1 = sql_fetch_row(result1)) != NULL)
    {
      citytypes citytype;
      long long id = sql_record_longlong(row1, 0);
      char *val = sql_record_str(row1, 1);
      if (strcmp(val, "city") == 0)
        citytype = CT_CITY;
      else if (strcmp(val, "town") == 0)
        citytype = CT_TOWN;
      else
        citytype = CT_VILLAGE;

      map_position pos;
      if (map_nodeposition(id, &pos) == 0)
      {
        char *name;
        if (get_tag_str("osm_nodes_tags", id, "name", &name) == 0)
        {
          boundary *bnd;
          int country = 0;
          if ((bnd = map_boundaryinside(&pos, BT_COUNTRY)) != NULL)
          {
            int citybnd = 0;
            boundary *bnd2;
            if ((bnd2 = map_boundaryinside(&pos, BT_CITY)) != NULL)
            {
              citybnd = bnd2->id;
              map_freeboundary(bnd2);
            }
            country = bnd->country;
            add_country(country);
            add_city(country, citybnd, citytype, &pos, name);
            map_freeboundary(bnd);
          }
          free(name);
        }
      }

      numrows++;
      long long prg = (start+numrows) * 100 / count;
      if (prg!=progress)
      {
        progress = prg;
        printf("Process cities %lld%%\r", prg);
        fflush(stdout);
      }
    }
    start += numrows;

    sql_freeresult(result1);
  }
  printf("\n");
  map_log(LOG_INFO, "Process cities done\n");
  return 0;
}


int map_process_nodes()
{
  long long progress = 0;
  if (sql_query("SELECT COUNT(*) FROM `osm_nodes`") != 0)
    return -1;
  sql_result *result = sql_storeresult();
  if (result == NULL)
    return -1;
  sql_row row;
  if ((row = sql_fetch_row(result)) == NULL)
    return -1;
  long long count = sql_record_longlong(row, 0);
  sql_freeresult(result);
  map_log(LOG_INFO, "Process nodes\n");
  printf("Process nodes 0%%\r");

  long long numrows = SQLCHUNK;
  long long start = 0;
  while (numrows == SQLCHUNK)
  {
    char sql[200];
    sprintf(sql, "SELECT id, lat, lon FROM `osm_nodes` LIMIT %lld,%d", start, SQLCHUNK);
    if (sql_query(sql) != 0)
      return -1;
    result = sql_storeresult();
    if (result == NULL)
      return -1;
    long long index = 0;
    while ((row = sql_fetch_row(result)) != NULL)
    {
      osm_node node;
      node.id = sql_record_longlong(row, 0);
      node.pos.lat = sql_record_long(row, 1);
      node.pos.lon = sql_record_long(row, 2);
      map_tags *tags = map_getnodetags(node.id);
      if (tags)
      {
        map_convert(tags, &node, NULL, NULL);
        free_tags(tags);
      }

      index++;
      long long prg = (start+index) * 100 / count;
      if (prg!=progress)
      {
        progress = prg;
        printf("Process nodes %lld%%\r", prg);
        fflush(stdout);
      }
    }

    numrows = index;
    start += numrows;
    sql_freeresult(result);
  }
  printf("\n");
  map_log(LOG_INFO, "Process nodes done\n");
  return 0;
}
