/* -*- Mode: C; indent-tabs-mode: s; c-basic-offset: 2; tab-width: 2 -*- */
/*
 * map_relation.c
 * Copyright (C) Mike Crash 2009 <mike@mikecrash.com>
 *
 * main.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * main.c is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdlib.h>
#include <typeio.h>
#include <string.h>
#include "db_mysql.h"
#include "polygon.h"
#include "map_common.h"
#include "map_ways.h"
#include "map_relation.h"
#include "map_tags.h"
#include "map_convert.h"
/*#include "map_osmway.h"
#include "map_boundary.h"
#include "map_nodes.h"
#include "map_planes.h"

//nodes cache
osm_relation *map_relation = NULL;
osm_relation *map_relationcur = NULL;

long n_relations = 0;
*/
char map_relation_type(const char *s)
{
  if (strcmp(s, "multipolygon") == 0) return RELT_MULTIPOLYGON;
  if (strcmp(s, "boundary") == 0) return RELT_BOUNDARY;
  if (strcmp(s, "restriction") == 0) return RELT_RESTRICTION;
  if (strcmp(s, "route") == 0) return RELT_ROUTE;
  if (strcmp(s, "enforcement") == 0) return RELT_ENFORCEMENT;
  return 0;
}

char map_relation_member(const char *s)
{
  if (strcmp(s, "node") == 0) return RIT_NODE;
  if (strcmp(s, "way") == 0) return RIT_WAY;
  if (strcmp(s, "relation") == 0) return RIT_RELATION;
  return 0;
}

char map_relation_role(const char *s)
{
  if (strcmp(s, "subarea") == 0) return RIR_SUBAREA;
  if (strcmp(s, "inner") == 0) return RIR_INNER;
  if (strcmp(s, "outer") == 0) return RIR_OUTER;
  if (strcmp(s, "admin_centre") == 0) return RIR_ADMINCENTER;
  if (strcmp(s, "from") == 0) return RIR_FROM;
  if (strcmp(s, "to") == 0) return RIR_TO;
  if (strcmp(s, "via") == 0) return RIR_VIA;
  return 0;
}
/*
unsigned int osmc_flags(char *s)
{
  if (strstr(s, "red") == s) return WF_RED;
  if (strstr(s, "green") == s) return WF_GREEN;
  if (strstr(s, "blue") == s) return WF_BLUE;
  if (strstr(s, "yellow") == s) return WF_YELLOW;
  if (strstr(s, "brown") == s) return WF_BROWN;
  if (strstr(s, "orange") == s) return WF_ORANGE;
  if (strstr(s, "purple") == s) return WF_PURPLE;
  return 0;
}
*/
int parse_relation_members(long cnt, unsigned char *arr, relation_member *ri)
{
  int i;

  for (i=0; i<cnt; i++)
  {
    ri->type = *arr;
    arr++;
    ri->role = *arr;
    arr++;
    int j;
    ri->id = 0;
    for (j=0; j<8; j++)
    {
      ri->id = (ri->id << 8) + *arr;
      arr++;
    }
    //printf("%d - %d - %lld\n", ri[i].type, ri[i].role, ri[i].id);
    ri++;
  }
  return 0;
}

int map_getrelation(long long id, osm_relation **relation)
{
  int res = -1;
  char sql[200];
  osm_relation *rel = NULL;

  sprintf(sql, "SELECT members FROM `osm_relations` WHERE id=%lld", id);
  if (sql_query(sql) != 0)
    return -1;

  sql_result *result = sql_storeresult();
  if (result == NULL)
    return -1;

  sql_row row = sql_fetch_row(result);
  unsigned long *lengths = sql_fetch_lengths(result);
  if (row)
  {
    long cnt = lengths[0] / 10;
    rel = (osm_relation *)malloc(sizeof(osm_relation)+sizeof(relation_member)*cnt);
    rel->id = id;
    rel->count = cnt;
    relation_member *ri = rel->members;

    unsigned char *arr = (unsigned char*)sql_record_str(row, 0);
    res = parse_relation_members(cnt, arr, ri);
  }

  sql_freeresult(result);
  *relation = rel;
  return res;
}

int map_getrelationtag_str(long long id, const char *name, char **value)
{
  return get_tag_str("osm_relations_tags", id, name, value);
}

int map_getrelationtag_longlong(long long id, const char *name, long long *value)
{
  return get_tag_longlong("osm_relations_tags", id, name, value);
}

int map_getrelationtag_int(long long id, const char *name, int *value)
{
  return get_tag_int("osm_relations_tags", id, name, value);
}

map_tags *map_getrelationtags(long long id)
{
  return get_tags("osm_relations_tags", id);
}

//convert way nodes to points
int ways2points(osm_way **ways, int wayscount, map_position *points)
{
  int first, step;
  long long lastnode;
  int cnt;

  if (wayscount==0)
    return -1;

  //copy first way - reverse?
  if ( (wayscount==1) ||
       (ways[0]->nodes[ways[0]->count-1] == ways[1]->nodes[0]) ||
       (ways[0]->nodes[ways[0]->count-1] == ways[1]->nodes[ways[1]->count-1]) )
  {
    first = 0;
    step = 1;
    lastnode = ways[0]->nodes[ways[0]->count-1];
  }
  else
  {
    first = ways[0]->count-1;
    step = -1;
    lastnode = ways[0]->nodes[0];
  }
  //sprintf(s, " 0: %ld nodes %ld -> %ld\n", ways[0]->count, ways[0]->nodes[first], ways[0]->nodes[first+step*(ways[0]->count-1)]);
  //map_log(s);
  //copy the nodes
  int i;
  for (i=0; i<ways[0]->count; i++)
  {
    if (map_nodeposition(ways[0]->nodes[first+i*step], &points[i])!=0)
    {
      map_log(LOG_ERROR, "Node %lld not found!\n", ways[0]->nodes[first+i*step]);
      return -1;
    }
  }
  cnt = ways[0]->count;

  //copy the rest, omit first node
  for (i=1; i<wayscount; i++)
  {
    if (lastnode == ways[i]->nodes[0])
    {
      first = 1;
      step = 1;
      lastnode = ways[i]->nodes[ways[i]->count-1];
    }
    else
    {
      first = ways[i]->count-2;
      step = -1;
      lastnode = ways[i]->nodes[0];
    }
    //sprintf(s, "%2d: %ld nodes %ld -> %ld\n", i, ways[i]->count, ways[i]->nodes[first], ways[i]->nodes[first+step*(ways[i]->count-2)]);
    //map_log(s);
    //copy the nodes
    int j;
    //printf("Nodes: %ld\n", ways[i]->count);
    for (j=0; j<ways[i]->count-1; j++)
    {
      if (map_nodeposition(ways[i]->nodes[first+j*step], &points[j+cnt])!=0)
      {
        map_log(LOG_ERROR, "Node %lld not found!\n", ways[i]->nodes[first+j*step]);
        return -1;
      }
    }
    cnt += ways[i]->count-1;
  }
  //sprintf(s, "Nodes: %d\n", cnt);
  //map_log(s);
  return 0;
}

//returns array of independent polygons
int relation_multipolygon(osm_relation *rel, map_multipolygon **multipolygon)
{
  int wayscount = 0;
  int index = 0;
  int i, k;
  osm_way **ways = (osm_way **)malloc(sizeof(ways)*rel->count);
  memset(ways, 0, sizeof(ways)*rel->count);

  //sprintf(s, "Relation %ld %d %s\n", rel->id, rel->type, get_tag_str(rel->tags, osmtag_name));
  //map_log(s);
  //if (rel->id == 52411)
  //  printf("AAA\n");

  //get all ways and check if it exists
  for (i=0; i<rel->count; i++)
  {
    relation_member *ri = &rel->members[i];
    //printf("RI %ld %d %d\n", ri->id, ri->type, ri->role);
    //sprintf(s, "RI %ld %d %d\n", ri->id, ri->type, ri->role);
    //map_log(s);
    if (ri->type == RIT_WAY)
    {
      //check for duplicates in boundary
      int k;
      int duplicate = 0;
      for (k=0; k<index; k++)
      {
        if (ways[k]->id == ri->id)
        {
          duplicate = 1;
          map_log(LOG_WARNING, "Duplicate way %lld in relation %lld\n", ri->id, rel->id);
          break;
        }
      }

      if (duplicate==0)
      {
        if (map_getway(ri->id, &ways[index]) != 0)
        {
          map_log(LOG_ERROR, "Way %lld not exists for relation %lld\n", ri->id, rel->id);
          for (k=0; k<index; k++)
            if (ways[k])
              free(ways[k]);
          free(ways);
          return -1;
        }
        //sprintf(s, "Way %ld nodes %ld->%ld\n", ri->id, ways[index]->nodes[0], ways[index]->nodes[ways[index]->count-1]);
        //map_log(s);
        if (ways[index]->count>1)
        {
          index++;
        }
      }
    }
  }
  wayscount = index;

  if (wayscount==0)
  {
    for (k=0; k<i; k++)
      if (ways[k])
        free(ways[k]);
    free(ways);
    return -1;
  }

  //resolve ways to segments
  map_polygon **mp = (map_polygon **)malloc(sizeof(map_polygon*)*wayscount);  //alloc maximum number - ways
  int *waysused = (int *)malloc(sizeof(int) * wayscount);
  memset(waysused, 0, sizeof(int) * wayscount);
  osm_way **wayssort = (osm_way **)malloc(sizeof(osm_way*) * (wayscount * 2 + 1));
  int polys = 0;

  for (k=0; k<wayscount; k++)
  {
    int nodes=0;
    if (waysused[k]==0)
    {
      int w1 = wayscount;
      int w2 = w1;
      int found = 1;
      wayssort[w1] = ways[k];
      waysused[k] = 1;
      nodes = ways[k]->count;
      while (found)
      {
        found = 0;
        for (i=k+1; i<wayscount; i++)
        {
          if (waysused[i]==0)
          {
            if ((ways[i]->nodes[0] == wayssort[w1]->nodes[0]) ||
                (ways[i]->nodes[0] == wayssort[w1]->nodes[wayssort[w1]->count-1]) ||
                (ways[i]->nodes[ways[i]->count-1] == wayssort[w1]->nodes[0]) ||
                (ways[i]->nodes[ways[i]->count-1] == wayssort[w1]->nodes[wayssort[w1]->count-1]))
            {
              waysused[i] = 1;
              w1--;
              wayssort[w1] = ways[i];
              nodes += ways[i]->count - 1;
              found = 1;
            }
            else
            if ((ways[i]->nodes[0] == wayssort[w2]->nodes[0]) ||
                (ways[i]->nodes[0] == wayssort[w2]->nodes[wayssort[w2]->count-1]) ||
                (ways[i]->nodes[ways[i]->count-1] == wayssort[w2]->nodes[0]) ||
                (ways[i]->nodes[ways[i]->count-1] == wayssort[w2]->nodes[wayssort[w2]->count-1]))
            {
              waysused[i] = 1;
              w2++;
              wayssort[w2] = ways[i];
              nodes += ways[i]->count - 1;
              found = 1;
            }
          }
        }
      }
      mp[polys] = map_newpolygon(nodes);
      //sprintf(s, "Sorted polygon %d with %d nodes\n", polys, nodes);
      //map_log(s);
      //for (i=w1; i<=w2; i++)
      //{
      //  sprintf(s, "Way %ld nodes %ld->%ld\n", wayssort[i]->id, wayssort[i]->nodes[0], wayssort[i]->nodes[wayssort[i]->count-1]);
      //  map_log(s);
      //}
      //create polygon
      if (ways2points(&wayssort[w1], w2-w1+1, mp[polys]->points) == 0)
      {
        polys++;
      }
      else
      {
        free(mp[polys]);
      }
    }
  }

  //check all polygons are closed
  int bad = 0;
  for (i=0; i<polys; i++)
  {
    //sprintf(s, "  Polygon %p with %d points\n", mp[i], mp[i]->count);
    //map_log(s);
    if ((mp[i]->points[0].lat != mp[i]->points[mp[i]->count-1].lat) || 
        (mp[i]->points[0].lon != mp[i]->points[mp[i]->count-1].lon))
    {
      bad = 1;
      break;
    }
    else
    {
      mp[i]->count--;
    }
  }

  //
  if ((polys == 0) || bad)
  {
    for (i=0; i<polys; i++)
    {
      free(mp[i]);
    }
    free(mp);
    free(waysused);
    free(wayssort);
    for (k=0; k<rel->count; k++)
      if (ways[k])
        free(ways[k]);
    free(ways);
    return -1;
  }

  //create multipolygon
  map_multipolygon *mmp = map_newmultipolygon(polys);
  //sprintf(s, "Multipolygon %p with %d polygons\n", mmp, polys);
  //map_log(s);
  for (i=0; i<polys; i++)
  {
    //sprintf(s, "  Polygon %p with %d points\n", mp[i], mp[i]->count);
    //map_log(s);
    mmp->polygon[i] = mp[i];
  }
  *multipolygon = mmp;
  free(mp);
  free(waysused);
  free(wayssort);
  for (k=0; k<rel->count; k++)
    if (ways[k])
      free(ways[k]);
  free(ways);
  return 0;
}

int process_relation_multipolygon(osm_relation *relation, map_tags *tags)
{
  map_multipolygon *mp;
  if (relation_multipolygon(relation, &mp) == 0)
  {
    map_multipolygon **mps;
    int polycnt;
    if (map_multipolygons_split(mp, &polycnt, &mps) == 0)
    {
      for (int i=0; i<polycnt; i++)
      {
        map_convert(tags, NULL, NULL, mps[i]);
        map_freemultipolygon(mps[i]);
      }
      free(mps);
    }
  }
  return 0;
}

int save_restriction(osm_restriction *restriction)
{
	char sql[200];
  //get node
  sprintf(sql, "SELECT sector, turn FROM mcm_nodes WHERE id=%lld", restriction->id);
  if (sql_query(sql) != 0)
    return -1;
  sql_result *result = sql_storeresult();
  if (result == NULL)
    return -1;
  sql_row row  = sql_fetch_row(result);
	
	char *newturn;
	long sector;
  long turnsize;
  if (row)
  {
    //node exists, update turns
		sector = sql_record_long(row, 0);
    char *oldturn = sql_record_str(row, 1);
		newturn = (char *)malloc(strlen(oldturn) + restriction->count*20 + 20);
		sprintf(newturn, "%s,%u,%d", oldturn, restriction->flags, restriction->count);
    turnsize = 0;
	}
	else
	{
    //node not exists
		sector = create_node(restriction->id);
		if (sector == -1)
		{
			sql_freeresult(result);
			return -1;
		}
		newturn = (char *)malloc(restriction->count*20 + 20);
		sprintf(newturn, "%u,%d", restriction->flags, restriction->count);
    turnsize = map_char_size;
	}
  sql_freeresult(result);

	char *append = newturn;
	append += strlen(append);
	for (int i=0; i<restriction->count; i++)
	{
    sprintf(append, ",%lld", restriction->nodes[i]);
    append += strlen(append);
	}
  turnsize += map_turnrestriction_size + restriction->count * map_long_size;

  char *sql2 = (char *)malloc(strlen(newturn)+255);
  sprintf(sql2, "UPDATE mcm_nodes SET turns=turns+1, `turn`='%s', turnsize=turnsize+%ld WHERE id=%lld", newturn, turnsize, restriction->id);
  free(newturn);
  if (sql_query(sql2) != 0)
  {
    free(sql2);
    return -1;
  }
  free(sql2);
  
  sprintf(sql, "UPDATE mcm_sectors SET turnsize=turnsize+%ld WHERE id=%ld", turnsize, sector);
  if (sql_query(sql) != 0)
  {
    return -1;
  }

	return 0;
}

int process_relation_restriction(osm_relation *relation, map_tags *tags)
{
	int i;
	osm_way *wayto = NULL, *wayfrom = NULL, *wayvia = NULL;
	long long nodevia = 0;

  //check restriction type
	unsigned int flags = 0;
	char *r = tag_str(tags, "restriction");
	if (r == NULL)
		return -1;
	if (strstr(r, "no_") == r)
		flags |= TRF_RESTRICT;
	if (strstr(r, "only_") == r)
		flags |= TRF_ALLOW;

	//check exceptions
	char *except = tag_str(tags, "except");
  if (except)
	{
	  if (strstr(except, "foot"))
			flags |= TRF_EXCEPT_FOOT;
	  if (strstr(except, "bicycle"))
			flags |= TRF_EXCEPT_BICYCLE;
	  if (strstr(except, "motorcycle"))
			flags |= TRF_EXCEPT_MOTORCYCLE;
	  if (strstr(except, "motorcar"))
			flags |= TRF_EXCEPT_MOTORCAR;
	  if (strstr(except, "hgv"))
			flags |= TRF_EXCEPT_TRUCKS;
	  if (strstr(except, "psv"))
			flags |= TRF_EXCEPT_PSV;
	}
	
	//find from and to way segments
	for (i=0; i<relation->count; i++)
	{
		if (relation->members[i].role == RIR_FROM)
		{
			map_getway(relation->members[i].id, &wayfrom);
		}
		if (relation->members[i].role == RIR_TO)
		{
			map_getway(relation->members[i].id, &wayto);
		}
	}
	if ((wayfrom == NULL) || (wayto == NULL))
	{
		if (wayfrom)
			free(wayfrom);
		if (wayto)
			free(wayto);
		return -1;
	}

	//find via segment and detect node or way
	for (i=0; i<relation->count; i++)
	{
		if (relation->members[i].role == RIR_VIA)
		{
			long long id = relation->members[i].id;
			if ((wayfrom->nodes[0] == id) || (wayfrom->nodes[wayfrom->count-1] == id)) 
			{
				if ((wayto->nodes[0] == id) || (wayto->nodes[wayto->count-1] == id)) 
				{
					nodevia = id;
				}
				else
				{
					map_log(LOG_WARNING, "Restriction %lld has bad node via\n", relation->id);
				}
			}
			else
			{
				if (map_getway(relation->members[i].id, &wayvia) == 0)
				{
					if (((wayvia->nodes[0] == wayfrom->nodes[0]) || (wayvia->nodes[0] == wayfrom->nodes[wayfrom->count-1]) ||
					      (wayvia->nodes[wayvia->count-1] == wayfrom->nodes[0]) || (wayvia->nodes[wayvia->count-1] == wayfrom->nodes[wayfrom->count-1])) &&
					     ((wayvia->nodes[0] == wayto->nodes[0]) || (wayvia->nodes[0] == wayto->nodes[wayto->count-1]) ||
					      (wayvia->nodes[wayvia->count-1] == wayto->nodes[0]) || (wayvia->nodes[wayvia->count-1] == wayto->nodes[wayto->count-1])))
					{
					}
					else
					{
					  map_log(LOG_WARNING, "Restriction %lld has bad way via\n", relation->id);
						free(wayvia);
						wayvia = NULL;
					}
				}
				else
				{
					map_log(LOG_WARNING, "Cannot get way via for restriction %lld\n", relation->id);
				}
			}
		}
	}

	if ((wayvia == NULL) && (nodevia == 0))
	{
		return -1;
	}

	//create restriction
	osm_restriction *restriction;
	if (nodevia)
	{
		//via is node
		restriction = (osm_restriction*)malloc(sizeof(osm_restriction) + 2 * sizeof(long long));
    restriction->id = nodevia;
		restriction->count = 2;
		
    if (wayto->nodes[0] == nodevia)
		{
			restriction->nodes[0] = wayto->nodes[1];
		}
    else
		{
			restriction->nodes[0] = wayto->nodes[wayto->count-2];
		}

    if (wayfrom->nodes[0] == nodevia)
    {
			restriction->nodes[1] = wayfrom->nodes[1];
    }
    else
    {
			restriction->nodes[1] = wayfrom->nodes[wayfrom->count-2];
    }
	}
	else
	{
		//via is way
		restriction = (osm_restriction*)malloc(sizeof(osm_restriction) + (wayvia->count+1) * sizeof(long long));
		restriction->count = wayvia->count+1;
		if ((wayvia->nodes[0] == wayto->nodes[0]) || (wayvia->nodes[wayvia->count-1] == wayto->nodes[0]))
		{
			restriction->id = wayto->nodes[0];
			restriction->nodes[0] = wayto->nodes[1];
		}
		else if ((wayvia->nodes[0] == wayto->nodes[wayto->count-1]) || (wayvia->nodes[wayvia->count-1] == wayto->nodes[wayto->count-1])) 
		{
			restriction->id = wayto->nodes[wayto->count-1];
			restriction->nodes[0] = wayto->nodes[wayto->count-2];
		}
    else
    {
      map_log(LOG_DEBUG, "Debug error in restriction, stupid author\n");
    }

    if (wayvia->nodes[0] == restriction->id)
		{
			for (int i=1; i<wayvia->count; i++)
			{
				restriction->nodes[i] = wayvia->nodes[i];
			}
		}
		else if (wayvia->nodes[wayvia->count-1] == restriction->id)
		{
			for (int i=1; i<wayvia->count; i++)
			{
				restriction->nodes[i] = wayvia->nodes[wayvia->count-i-1];
			}
		}
    else
    {
      map_log(LOG_DEBUG, "Debug error in restriction, stupid author\n");
    }

    if (wayfrom->nodes[0] == restriction->nodes[restriction->count-2])
    {
	    restriction->nodes[restriction->count-1] = wayfrom->nodes[1];
    }
    else
    {
      restriction->nodes[restriction->count-1] = wayfrom->nodes[wayfrom->count-2];
    }
	}
	
	restriction->flags = flags;

	free(wayfrom);
	free(wayto);
  if (wayvia)
		free(wayvia);

	save_restriction(restriction);
	return 0;

}

int process_relation(osm_relation *relation)
{
  map_tags *tags = map_getrelationtags(relation->id);
  char *type = tag_str(tags, "type");
  if (type == NULL)
  {
    free_tags(tags);
    return 0;
  }

  if (strcmp(type, "multipolygon") == 0)
  {
    process_relation_multipolygon(relation, tags);
  }
  else if (strcmp(type, "restriction") == 0)
  {
    process_relation_restriction(relation, tags);
  }
  free_tags(tags);
  return 0;
}

//process all relations
int map_process_relations()
{
  map_log(LOG_INFO, "Process relations\n");
  printf("Process relations 0%%\r");
  fflush(stdout);

  long long progress = 0;
  if (sql_query("SELECT COUNT(*) FROM `osm_relations`") != 0)
    return -1;
  sql_result *result = sql_storeresult();
  if (result == NULL)
    return -1;
  sql_row row;
  if ((row = sql_fetch_row(result)) == NULL)
    return -1;
  long long count = sql_record_longlong(row, 0);
  sql_freeresult(result);

  long long numrows = SQLCHUNK;
  long long start = 0;
  while (numrows == SQLCHUNK)
  {
    char sql[200];
    sprintf(sql, "SELECT id, members FROM `osm_relations` LIMIT %lld,%d",  start, SQLCHUNK);
    if (sql_query(sql) != 0)
      return -1;
    result = sql_storeresult();
    if (result == NULL)
      return -1;
    long long index = 0;
    while ((row = sql_fetch_row(result)) != NULL)
    {
      unsigned long *lengths = sql_fetch_lengths(result);
      long cnt = lengths[1] / 10;
      osm_relation *rel = (osm_relation *)malloc(sizeof(osm_relation)+sizeof(relation_member)*cnt);
      rel->id = sql_record_longlong(row, 0);
      rel->count = cnt;
      relation_member *ri = rel->members;
      //map_log(LOG_DEBUG, "Relation %ld\n", rel->id);

      unsigned char *arr = (unsigned char*)sql_record_str(row, 1);
      if (parse_relation_members(cnt, arr, ri) == 0)
      {
        //printf("  Process %ld\n", rel->count);
        process_relation(rel);
      }
      free(rel);
      //printf("  OK\n");

      index++;
      long long prg = (start + index) * 100 / count;
      if (prg!=progress)
      {
        progress = prg;
        printf("Process relations %lld%%\r", prg);
        fflush(stdout);
      }
    }

    numrows = index;
    start += index;
    sql_freeresult(result);
  }
  printf("\n");
  map_log(LOG_INFO, "Process relations done\n");
  return 0;
}
