/* -*- Mode: C; indent-tabs-mode: s; c-basic-offset: 2; tab-width: 2 -*- */
/*
 * map_tags.c
 * Copyright (C) Mike Crash 2009 <mike@mikecrash.com>
 *
 * main.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * main.c is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "map_tags.h"
#include "db_mysql.h"

int get_tag_str(const char *table, long long id, const char *name, char **value)
{
  char sql[100];
  sprintf(sql, "SELECT val FROM %s WHERE id='%lld' AND name='%s' LIMIT 0,1", table, id, name);
  if (sql_query(sql) != 0)
    return -1;

  sql_result *result;
  if ((result = sql_storeresult()) == NULL)
    return -1;
  sql_row row;
  if ((row = sql_fetch_row(result)) == NULL)
  {
    sql_freeresult(result);
    return -1;
  }
  unsigned long *lengths = sql_fetch_lengths(result);

  char *val = (char*)malloc(lengths[0]+1);
  strcpy(val, sql_record_str(row, 0));
  *value = val;
  sql_freeresult(result);
  return 0;
}

int get_tag_longlong(const char *table, long long id, const char *name, long long *value)
{
  char *val;
  int res = -1;
  if (get_tag_str(table, id, name, &val) == 0)
  {
    if (sscanf(val, "%lld", value) == 1)
    {
      res = 0;
    }
    free(val);
  }
  return -1;
}

int get_tag_int(const char *table, long long id, const char *name, int *value)
{
  char *val;
  int res = -1;
  if (get_tag_str(table, id, name, &val) == 0)
  {
    if (sscanf(val, "%d", value) == 1)
    {
      res = 0;
    }
    free(val);
  }
  return res;
}

map_tags *get_tags(const char *table, long long id)
{
  char sql[100];
  sprintf(sql, "SELECT name, val FROM %s WHERE id='%lld'", table, id);
  if (sql_query(sql) != 0)
    return NULL;

  sql_result *result;
  if ((result = sql_storeresult()) == NULL)
    return NULL;

  int count = sql_numrows(result);
  map_tags *tags = (map_tags *)malloc(sizeof(map_tags) + count * sizeof(map_tag));
  tags->count = count;
  sql_row row;
  int i = 0;
  while ((row = sql_fetch_row(result)) != NULL)
  {
    unsigned long *lengths = sql_fetch_lengths(result);
    char *name = (char*)malloc(lengths[0]+1);
    strcpy(name, sql_record_str(row, 0));

    char *value = (char*)malloc(lengths[1]+1);
    strcpy(value, sql_record_str(row, 1));

    tags->tags[i].name = name;
    tags->tags[i].value = value;
    i++;
  }
  sql_freeresult(result);

  return tags;
}

char *tag_str(map_tags *tags, const char *name)
{
  int i;

  for (i=0; i<tags->count; i++)
  {
    if (strcmp(tags->tags[i].name, name) == 0)
    {
      return tags->tags[i].value;
    }
  }
  return NULL;
}

void free_tags(map_tags *tags)
{
  int i;

  for (i=0; i<tags->count; i++)
  {
    free(tags->tags[i].name);
    free(tags->tags[i].value);
  }
  free(tags);
}

