/* -*- Mode: C; indent-tabs-mode: s; c-basic-offset: 2; tab-width: 2 -*- */
/*
 * map_osmway.c
 * Copyright (C) Mike Crash 2009 <mike@mikecrash.com>
 * 
 * main.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * main.c is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdlib.h>
#include <string.h>
#include "db_mysql.h"
#include "map_common.h"
#include "map_nodes.h"
#include "map_ways.h"
#include "map_convert.h"
#include "polygon.h"


//get way from database
int map_getway(long long id, osm_way **way)
{
  int res = -1;
  char sql[200];
  osm_way *w = NULL;

  sprintf(sql, "SELECT nodes FROM `osm_ways` WHERE id=%lld", id);
  if (sql_query(sql) != 0)
    return -1;
  
  sql_result *result = sql_storeresult();
  if (result == NULL)
    return -1;

  sql_row row = sql_fetch_row(result);
  unsigned long *lengths = sql_fetch_lengths(result);
  if (row)
  {
    long cnt = lengths[0] / 8;
    w = (osm_way*)malloc(sizeof(osm_way)+cnt*sizeof(long long));
    w->count = cnt;

    unsigned char *arr = (unsigned char *)sql_record_str(row, 0);
    int i;
    long long *nodes = w->nodes;

    for (i=0; i<cnt; i++)
    {
      for (int j=0; j<8; j++)
      {
        *nodes = (*nodes << 8) + *arr;
        arr++;
      }
      nodes++;
    }

    res = 0;
  }

  sql_freeresult(result);
  *way = w;
  return res;
}


//convert way to polygon
int way2polygon(osm_way *way, map_polygon **polygon)
{
  if (way==NULL)
    return -1;

  int i;

  if (way->count<2)
    return -1;

  map_polygon *mp = (map_polygon *)malloc(sizeof(map_polygon)+way->count*sizeof(map_position));
  mp->count = way->count;

  for (i=0; i<way->count; i++)
  {
    if (map_nodeposition(way->nodes[i], &mp->points[i])!=0)
    {
      map_log(LOG_ERROR, "Node %lld not found!\n", way->nodes[i]);
      free(mp);
      return -1;
    }
  }
  *polygon = mp;
  return 0;
}

map_tags *map_getwaytags(long long id)
{
  return get_tags("osm_ways_tags", id);
}

int map_wayarea(osm_way *way, long *lat1, long *lon1, long *lat2, long *lon2)
{
  int i;
  
  if (way==NULL)
    return -1;

  map_position pos;
  if (map_nodeposition(way->nodes[0], &pos) != 0)
    return -1;
  *lat1 = pos.lat;
  *lon1 = pos.lon;
  *lat2 = pos.lat;
  *lon2 = pos.lon;

  for (i=1; i<way->count; i++)
  {
    if (map_nodeposition(way->nodes[i], &pos) != 0)
      return -1;
    if (pos.lat < *lat1)
      *lat1 = pos.lat;
    if (pos.lon < *lon1)
      *lon1 = pos.lon;
    if (pos.lat > *lat2)
      *lat2 = pos.lat;
    if (pos.lon > *lon2)
      *lon2 = pos.lon;
  }
  return 0;
}

//process all relations

int parse_way_nodes(long cnt, unsigned char *arr, long long *nodes)
{
  int i;

  for (i=0; i<cnt; i++)
  {
    *nodes = 0;
    int j;
    for (j=0; j<8; j++)
    {
      *nodes = (*nodes << 8) + *arr;
      arr++;
    }
    //printf("%d - %d - %lld\n", nodes[i].type, nodes[i].role, nodes[i].id);
    nodes++;
  }
  return 0;
}

int map_process_ways()
{
  map_log(LOG_INFO, "Process ways\n");
  printf("Process ways 0%%\r");
  fflush(stdout);

  long long progress = 0;
  if (sql_query("SELECT COUNT(*) FROM `osm_ways`") != 0)
    return -1;
  sql_result *result = sql_storeresult();
  if (result == NULL)
    return -1;
  sql_row row;
  if ((row = sql_fetch_row(result)) == NULL)
    return -1;
  long long count = sql_record_longlong(row, 0);
  sql_freeresult(result);
  printf("Process ways 0%%\r");

  long long numrows = SQLCHUNK;
  long long start = 0;
  while (numrows == SQLCHUNK)
  {
    char sql[200];
    sprintf(sql, "SELECT id, nodes FROM `osm_ways` LIMIT %lld,%d", start, SQLCHUNK);
    if (sql_query(sql) != 0)
      return -1;
    result = sql_storeresult();
    if (result == NULL)
      return -1;
    long long index = 0;
    while ((row = sql_fetch_row(result)) != NULL)
    {
      unsigned long *lengths = sql_fetch_lengths(result);
      long cnt = lengths[1] / 8;
      if (cnt>1)
      {
        osm_way *way = (osm_way *)malloc(sizeof(osm_way)+sizeof(long long)*cnt);
        way->id = sql_record_longlong(row, 0);
        way->count = cnt;
        long long *nodes = way->nodes;

        unsigned char *arr = (unsigned char*)sql_record_str(row, 1);
        if (parse_way_nodes(cnt, arr, nodes) == 0)
        {
          map_tags *tags = map_getwaytags(way->id);
          if (tags)
          {
            map_convert(tags, NULL, way, NULL);
            free_tags(tags);
          }
        }
        free(way);
      }

      index++;
      long long prg = (start + index) * 100 / count;
      if (prg!=progress)
      {
        progress = prg;
        printf("Process ways %lld%%\r", prg);
        fflush(stdout);
      }
    }

    numrows = index;
    start += index;
    sql_freeresult(result);
  }

  printf("\n");
  map_log(LOG_INFO, "Process ways done\n");
  return 0;
}

map_multipolygon *map_waymultipolygon(long long id)
{
  int err = 0;
  map_multipolygon *mp = NULL;
  char sql[200];
  sprintf(sql, "SELECT id, nodes FROM `osm_ways` WHERE id=%lld", id);
  if (sql_query(sql) != 0)
    return NULL;
  sql_result *result = sql_storeresult();
  if (result == NULL)
    return NULL;
  long long index = 0;
  sql_row row;
  if ((row = sql_fetch_row(result)) != NULL)
  {
    unsigned long *lengths = sql_fetch_lengths(result);
    long cnt = lengths[1] / 8;
    if (cnt>1)
    {
      osm_way *way = (osm_way *)malloc(sizeof(osm_way)+sizeof(long long)*cnt);
      way->id = sql_record_longlong(row, 0);
      way->count = cnt;
      long long *nodes = way->nodes;

      unsigned char *arr = (unsigned char*)sql_record_str(row, 1);
      if (parse_way_nodes(cnt, arr, nodes) == 0)
      {
        mp = (map_multipolygon*)malloc(sizeof(map_multipolygon)+sizeof(map_polygon*));
        mp->count = 1;
        map_polygon *p = (map_polygon *)malloc(sizeof(map_polygon)+sizeof(map_position)*cnt);
        mp->polygon[0] = p;
        p->count = cnt;
        for (int j=0; j<cnt; j++)
        {
          if (map_nodeposition(nodes[j], &p->points[j]) != 0)
          {
            err = 1;
            break;
          }
        }
        if (err != 0)
        {
          map_freemultipolygon(mp);
          mp = NULL;
        }
      }
      free(way);
    }
  }
  sql_freeresult(result);

  return mp;
}
