/* -*- Mode: C; indent-tabs-mode: s; c-basic-offset: 2; tab-width: 2 -*- */
/*
 * map_write.cpp
 * Copyright (C) Mike Crash 2009 <mike@mikecrash.com>
 * 
 * main.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * main.c is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdlib.h>
#include <string.h>
#include "typeio.h"
#include "db_mysql.h"
#include "polygon.h"
#include "map_common.h"
#include "map_text.h"
#include "map_convert.h"

map_info mapinfo;
long map_written;
long n_sectors = 0;
long n_nodes = 0;
long n_ways = 0;
long n_planes = 0;
long n_locs = 0;
long n_pois = 0;
long n_countries = 0;
long n_cities = 0;
long n_streets = 0;
long n_addresses = 0;
long n_texts = 0;

long mynodeid(long long id)
{
  long mid = -1;
  char sql[100];
  sprintf(sql, "SELECT mid FROM mcm_nodes WHERE id=%lld", id);
  if (sql_query(sql) != 0)
    return -1;
  sql_result *result = sql_storeresult();
  if (result == NULL)
    return -1;
  sql_row row;
  if ((row = sql_fetch_row(result)) != NULL)
  {
    mid = sql_record_long(row, 0);
  }
  sql_freeresult(result);
  return mid;
}

int map_write_header(FILE *f)
{
  fwrite_char(mapinfo.header, 6, f);
  fwrite_int(&mapinfo.mapversion, 1, f);
  fwrite_long(&mapinfo.sectorsize, 1, f);
  fwrite_long(&mapinfo.flags, 1, f);
  fwrite_long(&mapinfo.nodesindex, 1, f);
  fwrite_long(&mapinfo.nodes, 1, f);
  fwrite_long(&mapinfo.planesindex, 1, f);
  fwrite_long(&mapinfo.planes, 1, f);
  fwrite_long(&mapinfo.locsindex, 1, f);
  fwrite_long(&mapinfo.locs, 1, f);
  fwrite_long(&mapinfo.poisindex, 1, f);
  fwrite_long(&mapinfo.pois, 1, f);
  fwrite_long(&mapinfo.routesindex, 1, f);
  fwrite_long(&mapinfo.routes, 1, f);
  fwrite_long(&mapinfo.contoursindex, 1, f);
  fwrite_long(&mapinfo.contours, 1, f);
  fwrite_long(&mapinfo.zoomindex, 1, f);
  fwrite_long(&mapinfo.zoom, 1, f);
  fwrite_long(&mapinfo.countries, 1, f);
  fwrite_long(&mapinfo.cities, 1, f);
  fwrite_long(&mapinfo.streets, 1, f);
  fwrite_long(&mapinfo.addresses, 1, f);
  fwrite_long(&mapinfo.text, 1, f);
  map_written = map_info_size;
  return 0;
}

int map_write_part(FILE *f, char partid, long count)
{
  fwrite_char(&partid, 1, f);
  fwrite_long(&count, 1, f);
  map_written += map_part_size;
  return 0;
}

int map_write_nodesindex(FILE *f)
{
  char sql[500];
  long sectors;
  long progress = -1;
  mapinfo.nodesindex = map_written;

  sprintf(sql, "SELECT lat, lon, nodes, segments, turnsize FROM mcm_sectors ORDER BY lat, lon ");
  if (sql_query(sql) != 0)
    return -1;
  sql_result *result1 = sql_storeresult();
  if (result1 == NULL)
    return -1;
  sectors = sql_numrows(result1);
  map_write_part(f, MPID_NODESINDEX, sectors);

  sql_row row;
  long index = 0;
  long position = 0;
  while ((row = sql_fetch_row(result1)) != NULL)
  {
    //write sector coordinates
    long lat = sql_record_long(row, 0);
    long lon = sql_record_long(row, 1);
    long nodes = sql_record_long(row, 2);
    long segments = sql_record_long(row, 3);
    long turnsize = sql_record_long(row, 4);
    long size = map_node_size*nodes + map_cross_size*segments + turnsize;

    fwrite_long(&lat, 1, f);
    fwrite_long(&lon, 1, f);
    fwrite_long(&nodes, 1, f);
    fwrite_long(&position, 1, f);
    fwrite_long(&size, 1, f);

    position += size;

    index++;
    long prg = index * 100 / sectors;
    if (prg!=progress)
    {
      progress = prg;
      printf("Writing nodes index %ld%%\r", prg);
      fflush(stdout);
    }
  }
  sql_freeresult(result1);
  map_written += index * map_sector_size;
  n_sectors = sectors;
  printf("\n");
  return 0;
}

int map_write_nodes(FILE *f)
{
  char sql[200];
  long progress = 0;
  if (sql_query("SELECT COUNT(*) FROM `mcm_nodes`") != 0)
    return -1;
  sql_result *result = sql_storeresult();
  if (result == NULL)
    return -1;
  sql_row row;
  if ((row = sql_fetch_row(result)) == NULL)
    return -1;
  long size = sql_record_long(row, 0);
  sql_freeresult(result);
  printf("Writing nodes 0%%\r");
  fflush(stdout);

  mapinfo.nodes = map_written;
  map_write_part(f, MPID_NODES, size);

  sprintf(sql, "SELECT id, nodes, segments, turnsize FROM mcm_sectors ORDER BY lat, lon ");
  if (sql_query(sql) != 0)
    return -1;
  sql_result *result1 = sql_storeresult();
  if (result1 == NULL)
    return -1;

  sql_row row1;
  long index = 0;
  while ((row1 = sql_fetch_row(result1)) != NULL)
  {
    long sector = sql_record_long(row1, 0);
    //long nodes = sql_record_long(row1, 1);
    //long segments = sql_record_long(row1, 2);
    //long turnsize = sql_record_long(row1, 3);
    //printf("Sector %ld, nodes %ld, segments %ld, turnsize %ld\n", sector, nodes, segments, turnsize);  
    sprintf(sql, "SELECT id, lat, lon, flags, segments, `cross`, turns, turn, turnsize FROM `mcm_nodes` WHERE sector=%ld ORDER BY id", sector);
    if (sql_query(sql) != 0)
      return -1;
    result = sql_storeresult();
    if (result == NULL)
      return -1;
    sql_row row;
    int ccc = 0;
    while ((row = sql_fetch_row(result)) != NULL)
    {
      ccc++;
      long id = sql_record_long(row, 0); //TODO long long?
      long lat = sql_record_long(row, 1);
      long lon = sql_record_long(row, 2);
      unsigned int flags = sql_record_int(row, 3);
      int segments = sql_record_int(row, 4);
      char count = (char)segments;
      char *cross = sql_record_str(row, 5);
      unsigned char turns = sql_record_char(row, 6);
      char *turn = sql_record_str(row, 7);
      int turnsize = sql_record_int(row, 8);
      if (turns)
        flags |= NF_TURNRESTRICTION; 

      fwrite_long(&id, 1, f);
      fwrite_long(&lat, 1, f);
      fwrite_long(&lon, 1, f);
      fwrite_uint(&flags, 1, f);
      fwrite_char(&count, 1, f);
      //printf("%ld %ld,%ld %d %d %d\n", id, lat, lon, count, turns, turnsize);
      //if (id == 975726667)
      //  printf("\n%d\n", ccc);

      char *cend = cross;
      for (int i=0; i<count; i++)
      {
        long cid = strtol(cend, &cend, 10);
        cend++;
        unsigned char waytype = (unsigned char)strtoul(cend, &cend, 10);
        cend++;
        unsigned char speed = (unsigned char)strtoul(cend, &cend, 10);
        cend++;
        unsigned int flags = (unsigned int)strtoul(cend, &cend, 10);
        cend++;
        long name = strtol(cend, &cend, 10);
        cend++;
        if (name)
          name = gettextpos(name);
        fwrite_long(&cid, 1, f);
        fwrite_uchar(&waytype, 1, f);
        fwrite_uchar(&speed, 1, f);
        fwrite_uint(&flags, 1, f);
        fwrite_long(&name, 1, f);
      }

      if (turns>0)
      {
        long checksize = map_char_size;
        fwrite_uchar(&turns, 1, f);
        cend = turn;
        for (int i=0; i<turns; i++)
        {
          unsigned int flags = (unsigned int)strtoul(cend, &cend, 10);
          cend++;
          fwrite_uint(&flags, 1, f);
          unsigned char count = (unsigned char)strtoul(cend, &cend, 10);
          cend++;
          fwrite_uchar(&count, 1, f);
          checksize += map_int_size + map_char_size;
          for (int j=0; j<count; j++)
          {
            long tid = strtol(cend, &cend, 10);
            cend++;
            fwrite_long(&tid, 1, f);
            checksize += map_long_size;
          }
        }
        if (turnsize != checksize)
        {
          map_log(LOG_DEBUG, "Something wrong! Bad turn size !!!\n");
          exit(-1);
        }
      }

      map_written += map_node_size + map_cross_size * count + turnsize;

      index++;
      long prg = index * 100 / size;
      if (prg!=progress)
      {
        progress = prg;
        printf("Writing nodes %ld%%\r", prg);
        fflush(stdout);
      }
    }

    sql_freeresult(result);
  }
  sql_freeresult(result1);

  printf("\n");
  n_nodes = size;
  return 0;
}

int map_write_planesindex(FILE *f)
{
  char sql[500];
  long sectors;
  long progress = -1;

  sprintf(sql, "SELECT lat, lon, zoom, planes, size FROM mcm_planes_sectors ORDER BY zoom DESC, lat ASC, lon ASC");
  if (sql_query(sql) != 0)
    return -1;
  sql_result *result1 = sql_storeresult();
  if (result1 == NULL)
    return -1;
  sectors = sql_numrows(result1);

  mapinfo.planesindex = map_written;
  map_write_part(f, MPID_PLANESINDEX, sectors);

  sql_row row;
  long index = 0;
  long filepos = 0;
  while ((row = sql_fetch_row(result1)) != NULL)
  {
    //write sector coordinates
    long lat = sql_record_long(row, 0);
    long lon = sql_record_long(row, 1);
    char zoom = sql_record_char(row, 2);
    long planes = sql_record_long(row, 3);
    long size = sql_record_long(row, 4);

    fwrite_long(&lat, 1, f);
    fwrite_long(&lon, 1, f);
    fwrite_char(&zoom, 1, f);
    fwrite_long(&planes, 1, f);
    fwrite_long(&filepos, 1, f);
    fwrite_long(&size, 1, f);

    filepos += size;

    index++;
    long prg = index * 100 / sectors;
    if (prg!=progress)
    {
      progress = prg;
      printf("Writing plane index %ld%%\r", prg);
      fflush(stdout);
    }
  }
  sql_freeresult(result1);
  map_written += index * map_planesector_size;
  printf("\n");
  return 0;
}

int map_write_planes(FILE *f)
{
  char sql[500];
  long sectors;
  long progress = -1;

  sprintf(sql, "SELECT id FROM mcm_planes_sectors ORDER BY zoom DESC, lat ASC, lon ASC");
  if (sql_query(sql) != 0)
    return -1;
  sql_result *result1 = sql_storeresult();
  if (result1 == NULL)
    return -1;
  sectors = sql_numrows(result1);

  mapinfo.planes = map_written;
  map_write_part(f, MPID_PLANES, sectors);

  sql_row row;
  long index = 0;
  while ((row = sql_fetch_row(result1)) != NULL)
  {
    long id = sql_record_long(row, 0);
    sprintf(sql, "SELECT typ, polygon FROM mcm_planes WHERE sector=%ld ORDER BY typ ASC", id);
    if (sql_query(sql) != 0)
      return -1;
    sql_result *result2 = sql_storeresult();
    if (result2 == NULL)
    {
      return -1;
    }

    sql_row row2;
    while ((row2 = sql_fetch_row(result2)) != NULL)
    {
      char typ = sql_record_char(row2, 0);
      char *arr = sql_record_str(row2, 1);
      map_multipolygon *mp = ascii2multipolygon(arr);
      fwrite_char(&typ, 1, f);

      int i,j;
      fwrite_int(&mp->count, 1, f);
      map_written += map_plane_size + map_multipolygon_size;
      for (i=0; i<mp->count; i++)
      {
        fwrite_int(&mp->polygon[i]->count, 1, f);
        for (j=0; j<mp->polygon[i]->count; j++)
        {
          fwrite_long(&mp->polygon[i]->points[j].lat, 1, f);
          fwrite_long(&mp->polygon[i]->points[j].lon, 1, f);
        }
        map_written += map_polygon_size + mp->polygon[i]->count * map_position_size;
      }
      map_freemultipolygon(mp);
      n_planes++;
    }

    index++;
    long prg = index * 100 / sectors;
    if (prg!=progress)
    {
      progress = prg;
      printf("Writing planes %ld%%\r", prg);
      fflush(stdout);
    }
  }
  sql_freeresult(result1);
  printf("\n");
  return 0;
}

//LOCATIONS
int map_write_locsindex(FILE *f)
{
  char sql[500];
  long sectors;
  long progress = -1;

  sprintf(sql, "SELECT lat, lon, locs, size FROM mcm_loc_sectors ORDER BY lat, lon");
  if (sql_query(sql) != 0)
    return -1;
  sql_result *result1 = sql_storeresult();
  if (result1 == NULL)
    return -1;
  sectors = sql_numrows(result1);

  mapinfo.locsindex = map_written;
  map_write_part(f, MPID_LOCSINDEX, sectors);

  sql_row row;
  long index = 0;
  long filepos = 0;
  while ((row = sql_fetch_row(result1)) != NULL)
  {
    //write sector coordinates
    long lat = sql_record_long(row, 0);
    long lon = sql_record_long(row, 1);
    long locs = sql_record_long(row, 2);
    long size = sql_record_long(row, 3);

    fwrite_long(&lat, 1, f);
    fwrite_long(&lon, 1, f);
    fwrite_long(&locs, 1, f);
    fwrite_long(&filepos, 1, f);
    fwrite_long(&size, 1, f);

    filepos += size;

    index++;
    long prg = index * 100 / sectors;
    if (prg!=progress)
    {
      progress = prg;
      printf("Writing loc index %ld%%\r", prg);
      fflush(stdout);
    }
  }
  sql_freeresult(result1);
  map_written += index * map_locindex_size;
  printf("\n");
  return 0;
}

int map_write_locs(FILE *f)
{
  char sql[500];
  long sectors;
  long progress = -1;

  sprintf(sql, "SELECT id FROM mcm_loc_sectors ORDER BY lat, lon");
  if (sql_query(sql) != 0)
    return -1;
  sql_result *result1 = sql_storeresult();
  if (result1 == NULL)
    return -1;
  sectors = sql_numrows(result1);

  sprintf(sql, "SELECT COUNT(*) as cnt FROM mcm_loc");
  if (sql_query(sql) != 0)
    return -1;
  sql_result *result2 = sql_storeresult();
  if (result2 == NULL)
    return -1;
  sql_row row2;
  if ((row2 = sql_fetch_row(result2)) == NULL)
  {
    sql_freeresult(result1);
    sql_freeresult(result2);
    return -1;
  }
  long locs = sql_record_long(row2, 0);

  mapinfo.locs = map_written;
  map_write_part(f, MPID_LOCS, locs);

  sql_row row;
  long index = 0;
  while ((row = sql_fetch_row(result1)) != NULL)
  {
    long id = sql_record_long(row, 0);
    sprintf(sql, "SELECT typ, lat, lon, name FROM mcm_loc WHERE sector=%ld ORDER BY typ ASC", id);
    if (sql_query(sql) != 0)
      return -1;
    sql_result *result2 = sql_storeresult();
    if (result2 == NULL)
    {
      return -1;
    }

    while ((row2 = sql_fetch_row(result2)) != NULL)
    {
      char typ = sql_record_char(row2, 0);
      long lat = sql_record_long(row2, 1);
      long lon = sql_record_long(row2, 2);
      long name = sql_record_long(row2, 3);
      if (name)
        name = gettextpos(name);

      fwrite_char(&typ, 1, f);
      fwrite_long(&lat, 1, f);
      fwrite_long(&lon, 1, f);
      fwrite_long(&name, 1, f);
      map_written += map_loc_size;
      n_locs++;
    }

    index++;
    long prg = index * 100 / sectors;
    if (prg!=progress)
    {
      progress = prg;
      printf("Writing locs %ld%%\r", prg);
      fflush(stdout);
    }
  }
  sql_freeresult(result1);
  printf("\n");
  return 0;
}

//POIS
int map_write_poisindex(FILE *f)
{
  char sql[500];
  long sectors;
  long progress = -1;

  sprintf(sql, "SELECT lat, lon, pois, size FROM mcm_poi_sectors ORDER BY lat, lon");
  if (sql_query(sql) != 0)
    return -1;
  sql_result *result1 = sql_storeresult();
  if (result1 == NULL)
    return -1;
  sectors = sql_numrows(result1);

  mapinfo.poisindex = map_written;
  map_write_part(f, MPID_POISINDEX, sectors);

  sql_row row;
  long index = 0;
  long filepos = 0;
  while ((row = sql_fetch_row(result1)) != NULL)
  {
    //write sector coordinates
    long lat = sql_record_long(row, 0);
    long lon = sql_record_long(row, 1);
    long pois = sql_record_long(row, 2);
    long size = sql_record_long(row, 3);

    fwrite_long(&lat, 1, f);
    fwrite_long(&lon, 1, f);
    fwrite_long(&pois, 1, f);
    fwrite_long(&filepos, 1, f);
    fwrite_long(&size, 1, f);

    filepos += size;

    index++;
    long prg = index * 100 / sectors;
    if (prg!=progress)
    {
      progress = prg;
      printf("Writing POI index %ld%%\r", prg);
      fflush(stdout);
    }
  }
  sql_freeresult(result1);
  map_written += index * map_poiindex_size;
  printf("\n");
  return 0;
}

int map_write_pois(FILE *f)
{
  char sql[500];
  long sectors;
  long progress = -1;

  sprintf(sql, "SELECT id FROM mcm_poi_sectors ORDER BY lat, lon");
  if (sql_query(sql) != 0)
    return -1;
  sql_result *result1 = sql_storeresult();
  if (result1 == NULL)
    return -1;
  sectors = sql_numrows(result1);

  sprintf(sql, "SELECT COUNT(*) as cnt FROM mcm_poi");
  if (sql_query(sql) != 0)
    return -1;
  sql_result *result2 = sql_storeresult();
  if (result2 == NULL)
    return -1;
  sql_row row2;
  if ((row2 = sql_fetch_row(result2)) == NULL)
  {
    sql_freeresult(result1);
    sql_freeresult(result2);
    return -1;
  }
  long pois = sql_record_long(row2, 0);

  mapinfo.pois = map_written;
  map_write_part(f, MPID_POIS, pois);

  sql_row row;
  long index = 0;
  while ((row = sql_fetch_row(result1)) != NULL)
  {
    long id = sql_record_long(row, 0);
    sprintf(sql, "SELECT typ, lat, lon, flags, name FROM mcm_poi WHERE sector=%ld ORDER BY typ ASC", id);
    if (sql_query(sql) != 0)
      return -1;
    sql_result *result2 = sql_storeresult();
    if (result2 == NULL)
    {
      return -1;
    }

    while ((row2 = sql_fetch_row(result2)) != NULL)
    {
      char typ = sql_record_char(row2, 0);
      long lat = sql_record_long(row2, 1);
      long lon = sql_record_long(row2, 2);
      unsigned char flags = sql_record_char(row2, 3);
      long name = sql_record_long(row2, 4);
      if (name)
        name = gettextpos(name);

      fwrite_char(&typ, 1, f);
      fwrite_long(&lat, 1, f);
      fwrite_long(&lon, 1, f);
      fwrite_uchar(&flags, 1, f);
      fwrite_long(&name, 1, f);
      map_written += map_poi_size;
      n_pois++;
    }

    index++;
    long prg = index * 100 / sectors;
    if (prg!=progress)
    {
      progress = prg;
      printf("Writing POIs %ld%%\r", prg);
      fflush(stdout);
    }
  }
  sql_freeresult(result1);
  printf("\n");
  return 0;
}

int map_write_countries(FILE *f)
{
  long progress = 0;

  if (sql_query("SELECT COUNT(*) FROM `mcm_country`") != 0)
    return -1;
  sql_result *result = sql_storeresult();
  if (result == NULL)
    return -1;
  sql_row row;
  if ((row = sql_fetch_row(result)) == NULL)
    return -1;
  long count = sql_record_long(row, 0);
  sql_freeresult(result);
  printf("Writing countries 0%%\r");

  mapinfo.countries = map_written;
  map_write_part(f, MPID_COUNTRIES, count);

  char sql[200];
  sprintf(sql, "SELECT id, size FROM mcm_country ORDER BY id");
  if (sql_query(sql) != 0)
    return -1;
  result = sql_storeresult();
  if (result == NULL)
    return -1;
  long idx = 0;
  long filepos = 0;
  while ((row = sql_fetch_row(result)) != NULL)
  {
    long id = sql_record_long(row, 0);
    long size = sql_record_long(row, 1);

    fwrite_long(&id, 1, f);
    fwrite_long(&filepos, 1, f);

    filepos += size;
    idx++;

    sprintf(sql, "UPDATE mcm_city SET sort=%ld WHERE country=%ld", idx, id);
    if (sql_query(sql) != 0)
    {
      sql_freeresult(result);
      return -1;
    }

    long prg = idx * 100 / count;
    if (prg!=progress)
    {
      progress = prg;
      printf("Writing countries %ld%%\r", prg);
      fflush(stdout);
    }
  }
  sql_freeresult(result);
  n_countries = idx;
  idx = 0;
  fwrite_long(&idx, 1, f);

  map_written += count * map_country_size + 4;
  printf("\n");
  return 0;
}


int map_write_cities(FILE *f)
{
  long progress = 0;

  if (sql_query("SELECT COUNT(*) FROM `mcm_city`") != 0)
    return -1;
  sql_result *result = sql_storeresult();
  if (result == NULL)
    return -1;
  sql_row row;
  if ((row = sql_fetch_row(result)) == NULL)
    return -1;
  long count = sql_record_long(row, 0);
  sql_freeresult(result);
  printf("Writing cities 0%%\r");

  mapinfo.cities = map_written;
  map_write_part(f, MPID_CITIES, count);

  long filepos = 0;
  long idx = 0;
  long idc = 0;
  while (idc<n_countries)
  {
    idc++;
    char sql[200];
    sprintf(sql, "SELECT id, country, lat, lon, name, size FROM mcm_city WHERE sort=%ld ORDER BY country, namec", idc);
    if (sql_query(sql) != 0)
      return -1;
    result = sql_storeresult();
    if (result == NULL)
      return -1;
    while ((row = sql_fetch_row(result)) != NULL)
    {
      long id = sql_record_long(row, 0);
      long country = sql_record_long(row, 1);
      long lat = sql_record_long(row, 2);
      long lon = sql_record_long(row, 3);
      long name = sql_record_long(row, 4);
      long size = sql_record_long(row, 5);
      long fp = size == 0 ? -1 : filepos;
      idx++;
      if (name)
        name = gettextpos(name);

      fwrite_long(&idx, 1, f);
      fwrite_long(&country, 1, f);
      fwrite_long(&lat, 1, f);
      fwrite_long(&lon, 1, f);
      fwrite_long(&name, 1, f);
      fwrite_long(&fp, 1, f);
      map_written += map_city_size;

      filepos += size;

      sprintf(sql, "UPDATE mcm_street SET sort=%ld WHERE city=%ld", idx, id);
      if (sql_query(sql) != 0)
      {
        sql_freeresult(result);
        return -1;
      }

      long prg = idx * 100 / count;
      if (prg!=progress)
      {
        progress = prg;
        printf("Writing cities %ld%%\r", prg);
        fflush(stdout);
      }
    }
    sql_freeresult(result);
    idx++;
  }

  //write ending id
  n_cities = idx;
  idx = 0;
  fwrite_long(&idx, 1, f);
  map_written += 4;
  
  printf("\n");
  return 0;
}

int map_write_streets(FILE *f)
{
  long progress = 0;

  if (sql_query("SELECT COUNT(*) FROM `mcm_street`") != 0)
    return -1;
  sql_result *result = sql_storeresult();
  if (result == NULL)
    return -1;
  sql_row row;
  if ((row = sql_fetch_row(result)) == NULL)
    return -1;
  long count = sql_record_long(row, 0);
  sql_freeresult(result);
  printf("Writing streets 0%%\r");

  mapinfo.streets = map_written;
  map_write_part(f, MPID_STREETS, count);

  long filepos = 0;
  long idx = 0;
  long idc = 0;
  while (idc<n_cities)
  {
    idc++;
    char sql[200];
    sprintf(sql, "SELECT id, city, lat, lon, name, size FROM mcm_street WHERE sort=%ld ORDER BY namec", idc);
    if (sql_query(sql) != 0)
      return -1;
    result = sql_storeresult();
    if (result == NULL)
      return -1;
    while ((row = sql_fetch_row(result)) != NULL)
    {
      long id = sql_record_long(row, 0);
      //long city = sql_record_long(row, 1);
      long lat = sql_record_long(row, 2);
      long lon = sql_record_long(row, 3);
      long name = sql_record_long(row, 4);
      long size = sql_record_long(row, 5);
      long fp = size == 0 ? -1 : filepos;
      idx++;
      if (name)
        name = gettextpos(name);

      fwrite_long(&idx, 1, f);
      fwrite_long(&idc, 1, f);
      fwrite_long(&lat, 1, f);
      fwrite_long(&lon, 1, f);
      fwrite_long(&name, 1, f);
      fwrite_long(&fp, 1, f);
      map_written += map_street_size;

      filepos += size;

      sprintf(sql, "UPDATE mcm_address SET sort=%ld WHERE street=%ld", idx, id);
      if (sql_query(sql) != 0)
      {
        sql_freeresult(result);
        return -1;
      }

      long prg = idx * 100 / count;
      if (prg!=progress)
      {
        progress = prg;
        printf("Writing streets %ld%%\r", prg);
        fflush(stdout);
      }
    }
    sql_freeresult(result);
  }
  n_streets = idx;

  //write ending id
  idx = 0;
  fwrite_long(&idx, 1, f);
  map_written += 4;

  printf("\n");
  return 0;
}

int map_write_addresses(FILE *f)
{
  long progress = 0;

  if (sql_query("SELECT COUNT(*) FROM `mcm_address`") != 0)
    return -1;
  sql_result *result = sql_storeresult();
  if (result == NULL)
    return -1;
  sql_row row;
  if ((row = sql_fetch_row(result)) == NULL)
    return -1;
  long count = sql_record_long(row, 0);
  sql_freeresult(result);
  printf("Writing addresses 0%%\r");

  mapinfo.addresses = map_written;
  map_write_part(f, MPID_ADDRESSES, count);

  long idx = 0;
  long ids = 0;
  while (ids<n_streets)
  {
    char sql[200];
    ids++;
    sprintf(sql, "SELECT id, street, lat, lon, name FROM mcm_address WHERE sort=%ld ORDER BY namec", ids);
    if (sql_query(sql) != 0)
      return -1;
    result = sql_storeresult();
    if (result == NULL)
      return -1;
    while ((row = sql_fetch_row(result)) != NULL)
    {
      //long id = sql_record_long(row, 0);
      //long city = sql_record_long(row, 1);
      long lat = sql_record_long(row, 2);
      long lon = sql_record_long(row, 3);
      long name = sql_record_long(row, 4);
      idx++;
      if (name)
        name = gettextpos(name);

      fwrite_long(&idx, 1, f);
      fwrite_long(&ids, 1, f);
      fwrite_long(&lat, 1, f);
      fwrite_long(&lon, 1, f);
      fwrite_long(&name, 1, f);
      map_written += map_address_size;

      long prg = idx * 100 / count;
      if (prg!=progress)
      {
        progress = prg;
        printf("Writing addresses %ld%%\r", prg);
        fflush(stdout);
      }
    }
    sql_freeresult(result);
  }
  n_addresses = idx;

  //write ending id
  idx = 0;
  fwrite_long(&idx, 1, f);
  map_written += 4;

  printf("\n");
  return 0;
}

int map_write_text(FILE *f)
{
  long progress = 0;
  if (sql_query("SELECT COUNT(*) FROM `mcm_text`") != 0)
    return -1;
  sql_result *result = sql_storeresult();
  if (result == NULL)
    return -1;
  sql_row row;
  if ((row = sql_fetch_row(result)) == NULL)
    return -1;
  long size = sql_record_long(row, 0);
  sql_freeresult(result);

  printf("Writing text 0%%\r");
  fflush(stdout);

  mapinfo.text = map_written;
  map_write_part(f, MPID_TEXT, size);

  long numrows = SQLCHUNK;
  long start = 0;
  while (numrows == SQLCHUNK)
  {
    char sql[200];
    sprintf(sql, "SELECT name, note FROM mcm_text LIMIT %ld,%d",  start, SQLCHUNK);
    if (sql_query(sql) != 0)
      return -1;
    result = sql_storeresult();
    if (result == NULL)
      return -1;
    long index = 0;
    while ((row = sql_fetch_row(result)) != NULL)
    {
      char *name = sql_record_str(row, 0);
      char *note = sql_record_str(row, 1);

      //TODO check text size for maximum length
      int textsize = 0;
      int namesize = (int)strlen(name);
      int notesize = (int)strlen(note);
      if (namesize>0)
        textsize += namesize + 1;
      if (notesize>0)
        textsize += notesize + 1;
      if (textsize>0)
      {
        fwrite_int(&textsize, 1, f);
        if (namesize>0)
        {
          char ch = 0;
          fwrite_char(name, namesize, f);
          fwrite_char(&ch, 1, f);
        }
        if (notesize>0)
        {
          char ch = 1;
          fwrite_char(note, notesize, f);
          fwrite_char(&ch, 1, f);
        }
        map_written += textsize + map_text_size;
        n_texts++;
      }

      index++;
      long prg = (start + index) * 100 / size;
      if (prg!=progress)
      {
        progress = prg;
        printf("Writing ways text %ld%%\r", prg);
        fflush(stdout);
      }
    }

    start += index;
    numrows = index;
    sql_freeresult(result);
  }
  printf("\n");
  return 0;
}

int map_remove_planes(maptypes typ)
{
  char sql[500];

  sprintf(sql, "SELECT id FROM mcm_planes WHERE typ=%d", (int)typ);
  if (sql_query(sql) != 0)
    return -1;
  sql_result *result = sql_storeresult();
  if (result == NULL)
    return -1;

  sql_row row;
  while ((row = sql_fetch_row(result)) != NULL)
  {
    long id = sql_record_long(row, 0);
    if (remove_plane(id) != 0)
    {
      return -1;
    }
  }
  sql_freeresult(result);
  return 0;
}
  
int map_write()
{
  map_log(LOG_INFO, "Write map\n");
  mapinfo.sectorsize = MAPSECTRECT;

  //open map file
  FILE *f = fopen(curmapname, "w+");
  map_written = 0;
  strcpy(mapinfo.header,"MCMAP");
  mapinfo.mapversion = MAP_VERSION; 

  //write
  map_write_header(f);
  map_write_nodesindex(f);
  map_write_nodes(f);
  map_write_planesindex(f);
  map_write_planes(f);
  map_write_locsindex(f);
  map_write_locs(f);
  map_write_poisindex(f);
  map_write_pois(f);
  map_write_countries(f);
  map_write_cities(f);
  map_write_streets(f);
  map_write_addresses(f);
  map_write_text(f);

  //update the header with correct file positions and close
  fseek(f, 0, SEEK_SET);
  map_write_header(f);
  fclose(f);
  map_log(LOG_INFO, "Write map done\n");

  //print statistics
  char s[200];
  sprintf(s, "\nMap statistics:\n----------------------");
  map_log(LOG_INFO, "%s\n", s); puts(s);
  sprintf(s, "Sectors: %ld", n_sectors);
  map_log(LOG_INFO, "%s\n", s); puts(s);
  sprintf(s, "Nodes: %ld", n_nodes);
  map_log(LOG_INFO, "%s\n", s); puts(s);
  sprintf(s, "Planes: %ld", n_planes);
  map_log(LOG_INFO, "%s\n", s); puts(s);
  sprintf(s, "Locations: %ld", n_locs);
  map_log(LOG_INFO, "%s\n", s); puts(s);
  sprintf(s, "POIs: %ld", n_pois);
  map_log(LOG_INFO, "%s\n", s); puts(s);
  sprintf(s, "Countries: %ld", n_countries);
  map_log(LOG_INFO, "%s\n", s); puts(s);
  sprintf(s, "Cities: %ld", n_cities);
  map_log(LOG_INFO, "%s\n", s); puts(s);
  sprintf(s, "Streets: %ld", n_streets);
  map_log(LOG_INFO, "%s\n", s); puts(s);
  sprintf(s, "Addresses: %ld", n_addresses);
  map_log(LOG_INFO, "%s\n", s); puts(s);
  sprintf(s, "Texts: %ld", n_texts);
  map_log(LOG_INFO, "%s\n", s); puts(s);

  return 0;
}
